// { dg-do compile { target c++11 } }
// { dg-options "-w" }

void f()
{
  goto l;			// { dg-message "from here" }
  if constexpr (false)		// { dg-message "enters constexpr if" }
    {
    l:;				// { dg-error "jump to label" }
    }
}
