/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/sys/wait.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( AIX_SYSWAIT_2_CHECK )
#define WSTOPSIG(__x)    (int)(WIFSTOPPED(__x) ? (int)((((unsigned int)__x) >> 8) & 0xff) : -1)
#endif  /* AIX_SYSWAIT_2_CHECK */


#if defined( NEXT_WAIT_UNION_CHECK )
extern pid_d wait(void*);
#endif  /* NEXT_WAIT_UNION_CHECK */
