/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/X11/Xmu.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( X11_SPRINTF_CHECK )
#ifndef __STDC__
extern char *	sprintf();
#endif /* !defined __STDC__ */
#endif  /* X11_SPRINTF_CHECK */
