/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/net/if.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( ALPHA_IF_SEMICOLON_CHECK )
     struct sockaddr vmif_paddr;	/* protocol address */
#endif  /* ALPHA_IF_SEMICOLON_CHECK */
