/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/mach-o/swap.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */

#ifndef FIXINC_WRAP_MACH_O_SWAP_H_DARWIN_EXTERNC
#define FIXINC_WRAP_MACH_O_SWAP_H_DARWIN_EXTERNC 1

#ifdef __cplusplus
extern "C" {
#endif


#if defined( DARWIN_EXTERNC_CHECK )
extern void swap_fat_header();

#endif  /* DARWIN_EXTERNC_CHECK */
#ifdef __cplusplus
}
#endif

#endif  /* FIXINC_WRAP_MACH_O_SWAP_H_DARWIN_EXTERNC */
