/* Definitions for option handling for Matsushita MN10300 series.
   Copyright (C) 1996-2017 Free Software Foundation, Inc.

   This file is part of GCC.

   GCC is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   GCC is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GCC; see the file COPYING3.  If not see
   <http://www.gnu.org/licenses/>.  */

#ifndef MN10300_OPTS_H
#define MN10300_OPTS_H

enum processor_type
{
  PROCESSOR_MN10300,
  PROCESSOR_AM33,
  PROCESSOR_AM33_2,
  PROCESSOR_AM34
};

#endif
