/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/sundev/vuid_event.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( BADQUOTE_CHECK )
/* does not have matched single quotes */
#endif  /* BADQUOTE_CHECK */
