/* Definitions for option handling for OpenVMS.
   Copyright (C) 2012-2017 Free Software Foundation, Inc.

   This file is part of GCC.

   GCC is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3, or (at your
   option) any later version.

   GCC is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with GCC; see the file COPYING3.  If not see
   <http://www.gnu.org/licenses/>.  */

#ifndef VMS_OPTS_H
#define VMS_OPTS_H

enum vms_pointer_size
  {
    VMS_POINTER_SIZE_NONE,
    VMS_POINTER_SIZE_32,
    VMS_POINTER_SIZE_64
  };

#endif
