/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/sys/_inttypes.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( HPUX_INTTYPE_INT8_T_CHECK )
typedef signed char int_least8_t;
typedef signed char int8_t;

#endif  /* HPUX_INTTYPE_INT8_T_CHECK */
