/* Builtin definitions for IBM S/390 and zSeries
   Copyright (C) 2015-2017 Free Software Foundation, Inc.

   Contributed by Andreas Krebbel (Andreas.Krebbel@de.ibm.com).

   This file is part of GCC.

   GCC is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   GCC is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with GCC; see the file COPYING3.  If not see
   <http://www.gnu.org/licenses/>.  */

#undef O_U1
#undef O_U2
#undef O_U3
#undef O_U4
#undef O_U5
#undef O_U8
#undef O_U12
#undef O_U16
#undef O_U32

#undef O_S2
#undef O_S3
#undef O_S4
#undef O_S5
#undef O_S8
#undef O_S12
#undef O_S16
#undef O_S32
#undef O_ELEM
#undef O_LIT

#undef O1_U1
#undef O2_U1
#undef O3_U1
#undef O4_U1

#undef O1_U2
#undef O2_U2
#undef O3_U2
#undef O4_U2

#undef O1_U3
#undef O2_U3
#undef O3_U3
#undef O4_U3

#undef O1_U4
#undef O2_U4
#undef O3_U4
#undef O4_U4

#undef O1_U5
#undef O2_U5
#undef O3_U5
#undef O4_U5

#undef O1_U8
#undef O2_U8
#undef O3_U8
#undef O4_U8

#undef O1_U12
#undef O2_U12
#undef O3_U12
#undef O4_U12

#undef O1_U16
#undef O2_U16
#undef O3_U16
#undef O4_U16

#undef O1_U32
#undef O2_U32
#undef O3_U32
#undef O4_U32

#undef O1_S2
#undef O2_S2
#undef O3_S2
#undef O4_S2

#undef O1_S3
#undef O2_S3
#undef O3_S3
#undef O4_S3

#undef O1_S4
#undef O2_S4
#undef O3_S4
#undef O4_S4

#undef O1_S5
#undef O2_S5
#undef O3_S5
#undef O4_S5

#undef O1_S8
#undef O2_S8
#undef O3_S8
#undef O4_S8

#undef O1_S12
#undef O2_S12
#undef O3_S12
#undef O4_S12

#undef O1_S16
#undef O2_S16
#undef O3_S16
#undef O4_S16

#undef O1_S32
#undef O2_S32
#undef O3_S32
#undef O4_S32

#undef O1_ELEM
#undef O2_ELEM
#undef O3_ELEM
#undef O4_ELEM

#undef O1_LIT
#undef O2_LIT
#undef O3_LIT
#undef O4_LIT

#undef O_SHIFT
#undef O_IMM_P
#undef O_UIMM_P
#undef O_SIMM_P

#define O_U1   1 /* unsigned  1 bit literal */
#define O_U2   2 /* unsigned  2 bit literal */
#define O_U3   3 /* unsigned  3 bit literal */
#define O_U4   4 /* unsigned  4 bit literal */
#define O_U5   5 /* unsigned  5 bit literal */
#define O_U8   6 /* unsigned  8 bit literal */
#define O_U12  7 /* unsigned 16 bit literal */
#define O_U16  8 /* unsigned 16 bit literal */
#define O_U32  9 /* unsigned 32 bit literal */

#define O_S2  10 /* signed  2 bit literal */
#define O_S3  11 /* signed  3 bit literal */
#define O_S4  12 /* signed  4 bit literal */
#define O_S5  13 /* signed  5 bit literal */
#define O_S8  14 /* signed  8 bit literal */
#define O_S12 15 /* signed 12 bit literal */
#define O_S16 16 /* signed 16 bit literal */
#define O_S32 17 /* signed 32 bit literal */

#define O_ELEM  18 /* Element selector requiring modulo arithmetic. */
#define O_LIT   19 /* Operand must be a literal fitting the target type.  */

#define O_SHIFT 5

#define O_UIMM_P(X) ((X) >= O_U1 && (X) <= O_U32)
#define O_SIMM_P(X) ((X) >= O_S2 && (X) <= O_S32)
#define O_IMM_P(X) ((X) == O_LIT || ((X) >= O_U1 && (X) <= O_S32))

#define O1_U1 O_U1
#define O2_U1 (O_U1 << O_SHIFT)
#define O3_U1 (O_U1 << (2 * O_SHIFT))
#define O4_U1 (O_U1 << (3 * O_SHIFT))

#define O1_U2 O_U2
#define O2_U2 (O_U2 << O_SHIFT)
#define O3_U2 (O_U2 << (2 * O_SHIFT))
#define O4_U2 (O_U2 << (3 * O_SHIFT))

#define O1_U3 O_U3
#define O2_U3 (O_U3 << O_SHIFT)
#define O3_U3 (O_U3 << (2 * O_SHIFT))
#define O4_U3 (O_U3 << (3 * O_SHIFT))

#define O1_U4 O_U4
#define O2_U4 (O_U4 << O_SHIFT)
#define O3_U4 (O_U4 << (2 * O_SHIFT))
#define O4_U4 (O_U4 << (3 * O_SHIFT))

#define O1_U5 O_U5
#define O2_U5 (O_U5 << O_SHIFT)
#define O3_U5 (O_U5 << (2 * O_SHIFT))
#define O4_U5 (O_U5 << (3 * O_SHIFT))

#define O1_U8 O_U8
#define O2_U8 (O_U8 << O_SHIFT)
#define O3_U8 (O_U8 << (2 * O_SHIFT))
#define O4_U8 (O_U8 << (3 * O_SHIFT))

#define O1_U12 O_U12
#define O2_U12 (O_U12 << O_SHIFT)
#define O3_U12 (O_U12 << (2 * O_SHIFT))
#define O4_U12 (O_U12 << (3 * O_SHIFT))

#define O1_U16 O_U16
#define O2_U16 (O_U16 << O_SHIFT)
#define O3_U16 (O_U16 << (2 * O_SHIFT))
#define O4_U16 (O_U16 << (3 * O_SHIFT))

#define O1_U32 O_U32
#define O2_U32 (O_U32 << O_SHIFT)
#define O3_U32 (O_U32 << (2 * O_SHIFT))
#define O4_U32 (O_U32 << (3 * O_SHIFT))


#define O1_S2 O_S2
#define O2_S2 (O_S2 << O_SHIFT)
#define O3_S2 (O_S2 << (2 * O_SHIFT))
#define O4_S2 (O_S2 << (3 * O_SHIFT))

#define O1_S3 O_S3
#define O2_S3 (O_S3 << O_SHIFT)
#define O3_S3 (O_S3 << (2 * O_SHIFT))
#define O4_S3 (O_S3 << (3 * O_SHIFT))

#define O1_S4 O_S4
#define O2_S4 (O_S4 << O_SHIFT)
#define O3_S4 (O_S4 << (2 * O_SHIFT))
#define O4_S4 (O_S4 << (3 * O_SHIFT))

#define O1_S5 O_S5
#define O2_S5 (O_S5 << O_SHIFT)
#define O3_S5 (O_S5 << (2 * O_SHIFT))
#define O4_S5 (O_S5 << (3 * O_SHIFT))

#define O1_S8 O_S8
#define O2_S8 (O_S8 << O_SHIFT)
#define O3_S8 (O_S8 << (2 * O_SHIFT))
#define O4_S8 (O_S8 << (3 * O_SHIFT))

#define O1_S12 O_S12
#define O2_S12 (O_S12 << O_SHIFT)
#define O3_S12 (O_S12 << (2 * O_SHIFT))
#define O4_S12 (O_S12 << (3 * O_SHIFT))

#define O1_S16 O_S16
#define O2_S16 (O_S16 << O_SHIFT)
#define O3_S16 (O_S16 << (2 * O_SHIFT))
#define O4_S16 (O_S16 << (3 * O_SHIFT))

#define O1_S32 O_S32
#define O2_S32 (O_S32 << O_SHIFT)
#define O3_S32 (O_S32 << (2 * O_SHIFT))
#define O4_S32 (O_S32 << (3 * O_SHIFT))

#define O1_ELEM O_ELEM
#define O2_ELEM (O_ELEM << O_SHIFT)
#define O3_ELEM (O_ELEM << (2 * O_SHIFT))
#define O4_ELEM (O_ELEM << (3 * O_SHIFT))

#define O1_LIT O_LIT
#define O2_LIT (O_LIT << O_SHIFT)
#define O3_LIT (O_LIT << (2 * O_SHIFT))
#define O4_LIT (O_LIT << (3 * O_SHIFT))


/* Builtin flags. Flags applying to the whole builtin definition.  */

#undef B_INT
#undef B_HTM
#undef B_VX
#undef B_VXE
#undef B_DEP

#undef BFLAGS_MASK_INIT
#define BFLAGS_MASK_INIT (B_INT)

#define B_INT   (1 << 0)  /* Internal builtins.  This builtin cannot be used in user programs.  */
#define B_HTM   (1 << 1)  /* Builtins requiring the transactional execution facility.  */
#define B_VX    (1 << 2)  /* Builtins requiring the z13 vector extensions.  */
#define B_VXE   (1 << 3)  /* Builtins requiring the arch12 vector extensions.  */
#define B_DEP   (1 << 4)  /* Builtin has been deprecated and a warning should be issued.  */

/* B_DEF defines a standard (not overloaded) builtin
   B_DEF (<builtin name>, <RTL expander name>, <function attributes>, <builtin flags>, <operand flags, see above>, <fntype>)

   OB_DEF defines an overloaded builtin
   OB_DEF (<builtin name>, <start variant>, <end variant>, <builtin flags>, <fntype>)
   The builtin flags apply to all its variants and do not need to be mentioned there again.

   OB_DEF_VAR defines a variant of an overloaded builtin
   OB_DEF_VAR (<variant name>, <standard builtin name>, <flags>, <operand flags>, <fntype>)
   flags: Flags applying to all its variants should be mentioned in the OB_DEF line instead.  */


B_DEF      (tbeginc,                    tbeginc,            0,                  B_HTM,              0,                  BT_FN_INT)
B_DEF      (tbegin,                     tbegin,             returns_twice_attr, B_HTM,              0,                  BT_FN_INT_VOIDPTR)
B_DEF      (tbegin_nofloat,             tbegin_nofloat,     returns_twice_attr, B_HTM,              0,                  BT_FN_INT_VOIDPTR)
B_DEF      (tbegin_retry,               tbegin_retry,       returns_twice_attr, B_HTM,              0,                  BT_FN_INT_VOIDPTR_INT)
B_DEF      (tbegin_retry_nofloat,       tbegin_retry_nofloat,returns_twice_attr,B_HTM,              0,                  BT_FN_INT_VOIDPTR_INT)
B_DEF      (tend,                       tend,               0,                  B_HTM,              0,                  BT_FN_INT)
B_DEF      (tabort,                     tabort,             noreturn_attr,      B_HTM,              0,                  BT_FN_VOID_INT)
B_DEF      (tx_nesting_depth,           etnd,               0,                  B_HTM,              0,                  BT_FN_INT)
B_DEF      (non_tx_store,               ntstg,              0,                  B_HTM,              0,                  BT_FN_VOID_UINT64PTR_UINT64)
B_DEF      (tx_assist,                  tx_assist,          0,                  B_HTM,              0,                  BT_FN_VOID_INT)
B_DEF      (s390_sfpc,                  sfpc,               0,                  0,                  0,                  BT_FN_VOID_UINT)
B_DEF      (s390_efpc,                  efpc,               0,                  0,                  0,                  BT_FN_UINT)
B_DEF      (s390_lcbb,                  lcbb,               0,                  B_VX,               O2_U4,              BT_FN_UINT_VOIDCONSTPTR_INT)

OB_DEF     (s390_vec_step,              MAX,                MAX,                B_VX,               BT_FN_INT_INT)

OB_DEF     (s390_vec_gather_element,    s390_vec_gather_element_s32,s390_vec_gather_element_dbl,B_VX,BT_FN_OV4SI_OV4SI_OUV4SI_INTCONSTPTR_UCHAR)
OB_DEF_VAR (s390_vec_gather_element_s32,s390_vgef,          0,                  O4_U2,              BT_OV_V4SI_V4SI_UV4SI_INTCONSTPTR_UCHAR)
OB_DEF_VAR (s390_vec_gather_element_b32,s390_vgef,          0,                  O4_U2,              BT_OV_BV4SI_BV4SI_UV4SI_UINTCONSTPTR_UCHAR)
OB_DEF_VAR (s390_vec_gather_element_u32,s390_vgef,          0,                  O4_U2,              BT_OV_UV4SI_UV4SI_UV4SI_UINTCONSTPTR_UCHAR)
OB_DEF_VAR (s390_vec_gather_element_flt,s390_vgef,          B_VXE,              O4_U2,              BT_OV_V4SF_V4SF_UV4SI_FLTCONSTPTR_UCHAR)
OB_DEF_VAR (s390_vec_gather_element_s64,s390_vgeg,          0,                  O4_U1,              BT_OV_V2DI_V2DI_UV2DI_LONGLONGCONSTPTR_UCHAR)
OB_DEF_VAR (s390_vec_gather_element_b64,s390_vgeg,          0,                  O4_U1,              BT_OV_BV2DI_BV2DI_UV2DI_ULONGLONGCONSTPTR_UCHAR)
OB_DEF_VAR (s390_vec_gather_element_u64,s390_vgeg,          0,                  O4_U1,              BT_OV_UV2DI_UV2DI_UV2DI_ULONGLONGCONSTPTR_UCHAR)
OB_DEF_VAR (s390_vec_gather_element_dbl,s390_vgeg,          0,                  O4_U1,              BT_OV_V2DF_V2DF_UV2DI_DBLCONSTPTR_UCHAR)

B_DEF      (s390_vgef,                  vec_gather_elementv4si,0,               B_VX,               O4_U2,              BT_FN_UV4SI_UV4SI_UV4SI_UINTCONSTPTR_UCHAR)
B_DEF      (s390_vgeg,                  vec_gather_elementv2di,0,               B_VX,               O4_U1,              BT_FN_UV2DI_UV2DI_UV2DI_ULONGLONGCONSTPTR_UCHAR)
B_DEF      (s390_vgbm,                  vec_genbytemaskv16qi,0,                 B_VX,               O1_U16,             BT_FN_UV16QI_USHORT)
B_DEF      (s390_vgmb,                  vec_genmaskv16qi,   0,                  B_VX,               O1_U8 | O2_U8,      BT_FN_UV16QI_UCHAR_UCHAR)
B_DEF      (s390_vgmh,                  vec_genmaskv8hi,    0,                  B_VX,               O1_U8 | O2_U8,      BT_FN_UV8HI_UCHAR_UCHAR)
B_DEF      (s390_vgmf,                  vec_genmaskv4si,    0,                  B_VX,               O1_U8 | O2_U8,      BT_FN_UV4SI_UCHAR_UCHAR)
B_DEF      (s390_vgmg,                  vec_genmaskv2di,    0,                  B_VX,               O1_U8 | O2_U8,      BT_FN_UV2DI_UCHAR_UCHAR)

OB_DEF     (s390_vec_xl,                s390_vec_xl_s8,     s390_vec_xl_dbl,    B_VX,               BT_FN_V4SI_INT_VOIDPTR)
OB_DEF_VAR (s390_vec_xl_s8,             MAX,                0,                  O1_LIT,             BT_OV_V16QI_LONG_SCHARPTR)               /* vl */
OB_DEF_VAR (s390_vec_xl_u8,             MAX,                0,                  O1_LIT,             BT_OV_UV16QI_LONG_UCHARPTR)              /* vl */
OB_DEF_VAR (s390_vec_xl_s16,            MAX,                0,                  O1_LIT,             BT_OV_V8HI_LONG_SHORTPTR)                /* vl */
OB_DEF_VAR (s390_vec_xl_u16,            MAX,                0,                  O1_LIT,             BT_OV_UV8HI_LONG_USHORTPTR)              /* vl */
OB_DEF_VAR (s390_vec_xl_s32,            MAX,                0,                  O1_LIT,             BT_OV_V4SI_LONG_INTPTR)                  /* vl */
OB_DEF_VAR (s390_vec_xl_u32,            MAX,                0,                  O1_LIT,             BT_OV_UV4SI_LONG_UINTPTR)                /* vl */
OB_DEF_VAR (s390_vec_xl_s64,            MAX,                0,                  O1_LIT,             BT_OV_V2DI_LONG_LONGLONGPTR)             /* vl */
OB_DEF_VAR (s390_vec_xl_u64,            MAX,                0,                  O1_LIT,             BT_OV_UV2DI_LONG_ULONGLONGPTR)           /* vl */
OB_DEF_VAR (s390_vec_xl_flt,            MAX,                0,                  O1_LIT,             BT_OV_V4SF_LONG_FLTPTR)                  /* vl */
OB_DEF_VAR (s390_vec_xl_dbl,            MAX,                0,                  O1_LIT,             BT_OV_V2DF_LONG_DBLPTR)                  /* vl */

OB_DEF     (s390_vec_xld2,              s390_vec_xld2_s8,   s390_vec_xld2_dbl,  B_DEP | B_VX,       BT_FN_V4SI_INT_VOIDPTR)
OB_DEF_VAR (s390_vec_xld2_s8,           MAX,                0,                  O1_LIT,             BT_OV_V16QI_LONG_SCHARPTR)               /* vl */
OB_DEF_VAR (s390_vec_xld2_u8,           MAX,                0,                  O1_LIT,             BT_OV_UV16QI_LONG_UCHARPTR)              /* vl */
OB_DEF_VAR (s390_vec_xld2_s16,          MAX,                0,                  O1_LIT,             BT_OV_V8HI_LONG_SHORTPTR)                /* vl */
OB_DEF_VAR (s390_vec_xld2_u16,          MAX,                0,                  O1_LIT,             BT_OV_UV8HI_LONG_USHORTPTR)              /* vl */
OB_DEF_VAR (s390_vec_xld2_s32,          MAX,                0,                  O1_LIT,             BT_OV_V4SI_LONG_INTPTR)                  /* vl */
OB_DEF_VAR (s390_vec_xld2_u32,          MAX,                0,                  O1_LIT,             BT_OV_UV4SI_LONG_UINTPTR)                /* vl */
OB_DEF_VAR (s390_vec_xld2_s64,          MAX,                0,                  O1_LIT,             BT_OV_V2DI_LONG_LONGLONGPTR)             /* vl */
OB_DEF_VAR (s390_vec_xld2_u64,          MAX,                0,                  O1_LIT,             BT_OV_UV2DI_LONG_ULONGLONGPTR)           /* vl */
OB_DEF_VAR (s390_vec_xld2_dbl,          MAX,                0,                  O1_LIT,             BT_OV_V2DF_LONG_DBLPTR)                  /* vl */

OB_DEF     (s390_vec_xlw4,              s390_vec_xlw4_s8,   s390_vec_xlw4_u32,  B_DEP | B_VX,       BT_FN_V4SI_INT_VOIDPTR)
OB_DEF_VAR (s390_vec_xlw4_s8,           MAX,                0,                  O1_LIT,             BT_OV_V16QI_LONG_SCHARPTR)               /* vl */
OB_DEF_VAR (s390_vec_xlw4_u8,           MAX,                0,                  O1_LIT,             BT_OV_UV16QI_LONG_UCHARPTR)              /* vl */
OB_DEF_VAR (s390_vec_xlw4_s16,          MAX,                0,                  O1_LIT,             BT_OV_V8HI_LONG_SHORTPTR)                /* vl */
OB_DEF_VAR (s390_vec_xlw4_u16,          MAX,                0,                  O1_LIT,             BT_OV_UV8HI_LONG_USHORTPTR)              /* vl */
OB_DEF_VAR (s390_vec_xlw4_s32,          MAX,                0,                  O1_LIT,             BT_OV_V4SI_LONG_INTPTR)                  /* vl */
OB_DEF_VAR (s390_vec_xlw4_u32,          MAX,                0,                  O1_LIT,             BT_OV_UV4SI_LONG_UINTPTR)                /* vl */

OB_DEF     (s390_vec_splats,            s390_vec_splats_s8, s390_vec_splats_dbl,B_VX,               BT_FN_OV4SI_INT)
OB_DEF_VAR (s390_vec_splats_s8,         s390_vlrepb,        0,                  0,                  BT_OV_V16QI_SCHAR)
OB_DEF_VAR (s390_vec_splats_u8,         s390_vlrepb,        0,                  0,                  BT_OV_UV16QI_UCHAR)
OB_DEF_VAR (s390_vec_splats_s16,        s390_vlreph,        0,                  0,                  BT_OV_V8HI_SHORT)
OB_DEF_VAR (s390_vec_splats_u16,        s390_vlreph,        0,                  0,                  BT_OV_UV8HI_USHORT)
OB_DEF_VAR (s390_vec_splats_s32,        s390_vlrepf,        0,                  0,                  BT_OV_V4SI_INT)
OB_DEF_VAR (s390_vec_splats_u32,        s390_vlrepf,        0,                  0,                  BT_OV_UV4SI_UINT)
OB_DEF_VAR (s390_vec_splats_s64,        s390_vlrepg,        0,                  0,                  BT_OV_V2DI_LONGLONG)
OB_DEF_VAR (s390_vec_splats_u64,        s390_vlrepg,        0,                  0,                  BT_OV_UV2DI_ULONGLONG)
OB_DEF_VAR (s390_vec_splats_flt,        s390_vlrepf_flt,    B_VXE,              0,                  BT_OV_V4SF_FLT)                          /* vlrepf */
OB_DEF_VAR (s390_vec_splats_dbl,        s390_vlrepg_dbl,    0,                  0,                  BT_OV_V2DF_DBL)                          /* vlrepg */

B_DEF      (s390_vlrepb,                vec_splatsv16qi,    0,                  B_VX,               0,                  BT_FN_UV16QI_UCHAR)
B_DEF      (s390_vlreph,                vec_splatsv8hi,     0,                  B_VX,               0,                  BT_FN_UV8HI_USHORT)
B_DEF      (s390_vlrepf,                vec_splatsv4si,     0,                  B_VX,               0,                  BT_FN_UV4SI_UINT)
B_DEF      (s390_vlrepf_flt,            vec_splatsv4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_V4SF_FLT)
B_DEF      (s390_vlrepg,                vec_splatsv2di,     0,                  B_VX,               0,                  BT_FN_UV2DI_ULONGLONG)
B_DEF      (s390_vlrepg_dbl,            vec_splatsv2df,     0,                  B_INT | B_VX,       0,                  BT_FN_V2DF_DBL)
B_DEF      (s390_vrepib,                vec_splatsv16qi,    0,                  B_VX,               O1_U8,              BT_FN_V16QI_UCHAR)
B_DEF      (s390_vrepih,                vec_splatsv8hi,     0,                  B_VX,               O1_S16,             BT_FN_V8HI_SHORT)
B_DEF      (s390_vrepif,                vec_splatsv4si,     0,                  B_VX,               O1_S16,             BT_FN_V4SI_SHORT)
B_DEF      (s390_vrepig,                vec_splatsv2di,     0,                  B_VX,               O1_S16,             BT_FN_V2DI_SHORT)

B_DEF      (s390_vec_splat_u8,          vec_splatsv16qi,    0,                  B_VX,               O1_U8,              BT_FN_UV16QI_UCHAR)
B_DEF      (s390_vec_splat_s8,          vec_splatsv16qi,    0,                  B_VX,               O1_S8,              BT_FN_V16QI_SCHAR)
B_DEF      (s390_vec_splat_u16,         vec_splatsv8hi,     0,                  B_VX,               O1_U16,             BT_FN_UV8HI_USHORT)
B_DEF      (s390_vec_splat_s16,         vec_splatsv8hi,     0,                  B_VX,               O1_S16,             BT_FN_V8HI_SHORT)
B_DEF      (s390_vec_splat_u32,         vec_splatsv4si,     0,                  B_VX,               O1_U16,             BT_FN_UV4SI_USHORT)
B_DEF      (s390_vec_splat_s32,         vec_splatsv4si,     0,                  B_VX,               O1_S16,             BT_FN_V4SI_SHORT)
B_DEF      (s390_vec_splat_u64,         vec_splatsv2di,     0,                  B_VX,               O1_U16,             BT_FN_UV2DI_USHORT)
B_DEF      (s390_vec_splat_s64,         vec_splatsv2di,     0,                  B_VX,               O1_S16,             BT_FN_V2DI_SHORT)

/* First two operands are swapped in s390-c.c */
OB_DEF     (s390_vec_insert,            s390_vec_insert_s8, s390_vec_insert_dbl,B_VX,               BT_FN_OV4SI_INT_OV4SI_INT)
OB_DEF_VAR (s390_vec_insert_s8,         s390_vlvgb,         0,                  O3_ELEM,            BT_OV_V16QI_SCHAR_V16QI_INT)
OB_DEF_VAR (s390_vec_insert_u8,         s390_vlvgb,         0,                  O3_ELEM,            BT_OV_UV16QI_UCHAR_UV16QI_INT)
OB_DEF_VAR (s390_vec_insert_b8,         s390_vlvgb,         0,                  O3_ELEM,            BT_OV_UV16QI_UCHAR_BV16QI_INT)
OB_DEF_VAR (s390_vec_insert_s16,        s390_vlvgh,         0,                  O3_ELEM,            BT_OV_V8HI_SHORT_V8HI_INT)
OB_DEF_VAR (s390_vec_insert_u16,        s390_vlvgh,         0,                  O3_ELEM,            BT_OV_UV8HI_USHORT_UV8HI_INT)
OB_DEF_VAR (s390_vec_insert_b16,        s390_vlvgh,         0,                  O3_ELEM,            BT_OV_UV8HI_USHORT_BV8HI_INT)
OB_DEF_VAR (s390_vec_insert_s32,        s390_vlvgf,         0,                  O3_ELEM,            BT_OV_V4SI_INT_V4SI_INT)
OB_DEF_VAR (s390_vec_insert_u32,        s390_vlvgf,         0,                  O3_ELEM,            BT_OV_UV4SI_UINT_UV4SI_INT)
OB_DEF_VAR (s390_vec_insert_b32,        s390_vlvgf,         0,                  O3_ELEM,            BT_OV_UV4SI_UINT_BV4SI_INT)
OB_DEF_VAR (s390_vec_insert_s64,        s390_vlvgg,         0,                  O3_ELEM,            BT_OV_V2DI_LONGLONG_V2DI_INT)
OB_DEF_VAR (s390_vec_insert_u64,        s390_vlvgg,         0,                  O3_ELEM,            BT_OV_UV2DI_ULONGLONG_UV2DI_INT)
OB_DEF_VAR (s390_vec_insert_b64,        s390_vlvgg,         0,                  O3_ELEM,            BT_OV_UV2DI_ULONGLONG_BV2DI_INT)
OB_DEF_VAR (s390_vec_insert_flt,        s390_vlvgf_flt,     B_VXE,              O3_ELEM,            BT_OV_V4SF_FLT_V4SF_INT)                 /* vlvgf */
OB_DEF_VAR (s390_vec_insert_dbl,        s390_vlvgg_dbl,     0,                  O3_ELEM,            BT_OV_V2DF_DBL_V2DF_INT)                 /* vlvgg */

B_DEF      (s390_vlvgb,                 vec_insertv16qi,    0,                  B_VX,               O3_ELEM,            BT_FN_UV16QI_UV16QI_UCHAR_INT)
B_DEF      (s390_vlvgh,                 vec_insertv8hi,     0,                  B_VX,               O3_ELEM,            BT_FN_UV8HI_UV8HI_USHORT_INT)
B_DEF      (s390_vlvgf,                 vec_insertv4si,     0,                  B_VX,               O3_ELEM,            BT_FN_UV4SI_UV4SI_UINT_INT)
B_DEF      (s390_vlvgg,                 vec_insertv2di,     0,                  B_VX,               O3_ELEM,            BT_FN_UV2DI_UV2DI_ULONGLONG_INT)
B_DEF      (s390_vlvgf_flt,             vec_insertv4sf,     0,                  B_INT | B_VXE,      O3_ELEM,            BT_FN_V4SF_V4SF_FLT_INT)
B_DEF      (s390_vlvgg_dbl,             vec_insertv2df,     0,                  B_INT | B_VX,       O3_ELEM,            BT_FN_V2DF_V2DF_DBL_INT)

OB_DEF     (s390_vec_promote,           s390_vec_promote_s8,s390_vec_promote_dbl,B_VX,              BT_FN_OV4SI_INT_INT)
OB_DEF_VAR (s390_vec_promote_s8,        s390_vlvgb_noin,    0,                  O2_ELEM,            BT_OV_V16QI_SCHAR_INT)                   /* vlvgb */
OB_DEF_VAR (s390_vec_promote_u8,        s390_vlvgb_noin,    0,                  O2_ELEM,            BT_OV_UV16QI_UCHAR_INT)                  /* vlvgb */
OB_DEF_VAR (s390_vec_promote_s16,       s390_vlvgh_noin,    0,                  O2_ELEM,            BT_OV_V8HI_SHORT_INT)                    /* vlvgh */
OB_DEF_VAR (s390_vec_promote_u16,       s390_vlvgh_noin,    0,                  O2_ELEM,            BT_OV_UV8HI_USHORT_INT)                  /* vlvgh */
OB_DEF_VAR (s390_vec_promote_s32,       s390_vlvgf_noin,    0,                  O2_ELEM,            BT_OV_V4SI_INT_INT)                      /* vlvgf */
OB_DEF_VAR (s390_vec_promote_u32,       s390_vlvgf_noin,    0,                  O2_ELEM,            BT_OV_UV4SI_UINT_INT)                    /* vlvgf */
OB_DEF_VAR (s390_vec_promote_s64,       s390_vlvgg_noin,    0,                  O2_ELEM,            BT_OV_V2DI_LONGLONG_INT)                 /* vlvgg */
OB_DEF_VAR (s390_vec_promote_u64,       s390_vlvgg_noin,    0,                  O2_ELEM,            BT_OV_UV2DI_ULONGLONG_INT)               /* vlvgg */
OB_DEF_VAR (s390_vec_promote_flt,       s390_vlvgf_flt_noin,B_VXE,              O2_ELEM,            BT_OV_V4SF_FLT_INT)                      /* vlvgf */
OB_DEF_VAR (s390_vec_promote_dbl,       s390_vlvgg_dbl_noin,0,                  O2_ELEM,            BT_OV_V2DF_DBL_INT)                      /* vlvgg */

B_DEF      (s390_vlvgb_noin,            vec_promotev16qi,   0,                  B_INT | B_VX,       0,                  BT_FN_UV16QI_UCHAR_INT)
B_DEF      (s390_vlvgh_noin,            vec_promotev8hi,    0,                  B_INT | B_VX,       0,                  BT_FN_UV8HI_USHORT_INT)
B_DEF      (s390_vlvgf_noin,            vec_promotev4si,    0,                  B_INT | B_VX,       0,                  BT_FN_UV4SI_UINT_INT)
B_DEF      (s390_vlvgg_noin,            vec_promotev2di,    0,                  B_INT | B_VX,       0,                  BT_FN_UV2DI_ULONGLONG_INT)
B_DEF      (s390_vlvgf_flt_noin,        vec_promotev4sf,    0,                  B_INT | B_VXE,      0,                  BT_FN_V4SF_FLT_INT)
B_DEF      (s390_vlvgg_dbl_noin,        vec_promotev2df,    0,                  B_INT | B_VX,       0,                  BT_FN_V2DF_DBL_INT)

OB_DEF     (s390_vec_extract,           s390_vec_extract_s8,s390_vec_extract_dbl,B_VX,              BT_FN_INT_OV4SI_INT)
OB_DEF_VAR (s390_vec_extract_s8,        s390_vlgvb,         0,                  O2_ELEM,            BT_OV_SCHAR_V16QI_INT)
OB_DEF_VAR (s390_vec_extract_u8,        s390_vlgvb,         0,                  O2_ELEM,            BT_OV_UCHAR_UV16QI_INT)
OB_DEF_VAR (s390_vec_extract_b8,        s390_vlgvb,         0,                  O2_ELEM,            BT_OV_UCHAR_BV16QI_INT)
OB_DEF_VAR (s390_vec_extract_s16,       s390_vlgvh,         0,                  O2_ELEM,            BT_OV_SHORT_V8HI_INT)
OB_DEF_VAR (s390_vec_extract_u16,       s390_vlgvh,         0,                  O2_ELEM,            BT_OV_USHORT_UV8HI_INT)
OB_DEF_VAR (s390_vec_extract_b16,       s390_vlgvh,         0,                  O2_ELEM,            BT_OV_USHORT_BV8HI_INT)
OB_DEF_VAR (s390_vec_extract_s32,       s390_vlgvf,         0,                  O2_ELEM,            BT_OV_INT_V4SI_INT)
OB_DEF_VAR (s390_vec_extract_u32,       s390_vlgvf,         0,                  O2_ELEM,            BT_OV_UINT_UV4SI_INT)
OB_DEF_VAR (s390_vec_extract_b32,       s390_vlgvf,         0,                  O2_ELEM,            BT_OV_UINT_BV4SI_INT)
OB_DEF_VAR (s390_vec_extract_flt,       s390_vlgvf_flt,     B_VXE,              O2_ELEM,            BT_OV_FLT_V4SF_INT)
OB_DEF_VAR (s390_vec_extract_s64,       s390_vlgvg,         0,                  O2_ELEM,            BT_OV_LONGLONG_V2DI_INT)
OB_DEF_VAR (s390_vec_extract_u64,       s390_vlgvg,         0,                  O2_ELEM,            BT_OV_ULONGLONG_UV2DI_INT)
OB_DEF_VAR (s390_vec_extract_b64,       s390_vlgvg,         0,                  O2_ELEM,            BT_OV_ULONGLONG_BV2DI_INT)
OB_DEF_VAR (s390_vec_extract_dbl,       s390_vlgvg_dbl,     0,                  O2_ELEM,            BT_OV_DBL_V2DF_INT)                      /* vlgvg */

B_DEF      (s390_vlgvb,                 vec_extractv16qi,   0,                  B_VX,               O2_ELEM,            BT_FN_UCHAR_UV16QI_INT)
B_DEF      (s390_vlgvh,                 vec_extractv8hi,    0,                  B_VX,               O2_ELEM,            BT_FN_USHORT_UV8HI_INT)
B_DEF      (s390_vlgvf,                 vec_extractv4si,    0,                  B_VX,               O2_ELEM,            BT_FN_UINT_UV4SI_INT)
B_DEF      (s390_vlgvf_flt,             vec_extractv4sf,    0,                  B_INT | B_VXE,      O2_ELEM,            BT_FN_FLT_V4SF_INT)
B_DEF      (s390_vlgvg,                 vec_extractv2di,    0,                  B_VX,               O2_ELEM,            BT_FN_ULONGLONG_UV2DI_INT)
B_DEF      (s390_vlgvg_dbl,             vec_extractv2df,    0,                  B_INT | B_VX,       O2_ELEM,            BT_FN_DBL_V2DF_INT)

OB_DEF     (s390_vec_insert_and_zero,   s390_vec_insert_and_zero_s8,s390_vec_insert_and_zero_dbl,B_VX,BT_FN_OV4SI_INTCONSTPTR)
OB_DEF_VAR (s390_vec_insert_and_zero_s8,s390_vllezb,        0,                  0,                  BT_OV_V16QI_SCHARCONSTPTR)
OB_DEF_VAR (s390_vec_insert_and_zero_u8,s390_vllezb,        0,                  0,                  BT_OV_UV16QI_UCHARCONSTPTR)
OB_DEF_VAR (s390_vec_insert_and_zero_s16,s390_vllezh,       0,                  0,                  BT_OV_V8HI_SHORTCONSTPTR)
OB_DEF_VAR (s390_vec_insert_and_zero_u16,s390_vllezh,       0,                  0,                  BT_OV_UV8HI_USHORTCONSTPTR)
OB_DEF_VAR (s390_vec_insert_and_zero_s32,s390_vllezf,       0,                  0,                  BT_OV_V4SI_INTCONSTPTR)
OB_DEF_VAR (s390_vec_insert_and_zero_u32,s390_vllezf,       0,                  0,                  BT_OV_UV4SI_UINTCONSTPTR)
OB_DEF_VAR (s390_vec_insert_and_zero_flt,s390_vllezf_flt,   B_VXE,              0,                  BT_OV_V4SF_FLTCONSTPTR)
OB_DEF_VAR (s390_vec_insert_and_zero_s64,s390_vllezg,       0,                  0,                  BT_OV_V2DI_LONGLONGCONSTPTR)
OB_DEF_VAR (s390_vec_insert_and_zero_u64,s390_vllezg,       0,                  0,                  BT_OV_UV2DI_ULONGLONGCONSTPTR)
OB_DEF_VAR (s390_vec_insert_and_zero_dbl,s390_vllezg_dbl,   0,                  0,                  BT_OV_V2DF_DBLCONSTPTR)

B_DEF      (s390_vllezb,                vec_insert_and_zerov16qi,0,             B_VX,               0,                  BT_FN_UV16QI_UCHARCONSTPTR)
B_DEF      (s390_vllezh,                vec_insert_and_zerov8hi,0,              B_VX,               0,                  BT_FN_UV8HI_USHORTCONSTPTR)
B_DEF      (s390_vllezf,                vec_insert_and_zerov4si,0,              B_VX,               0,                  BT_FN_UV4SI_UINTCONSTPTR)
B_DEF      (s390_vllezf_flt,            vec_insert_and_zerov4sf,0,              B_INT | B_VXE,      0,                  BT_FN_V4SF_FLTCONSTPTR)
B_DEF      (s390_vllezg,                vec_insert_and_zerov2di,0,              B_VX,               0,                  BT_FN_UV2DI_ULONGLONGCONSTPTR)
B_DEF      (s390_vllezg_dbl,            vec_insert_and_zerov2df,0,              B_INT | B_VX,       0,                  BT_FN_V2DF_DBLCONSTPTR)

/* The 2nd operand will by translated in s390-c.c from 64, 128, 256, ... to 0, 1, 2, ... */
OB_DEF     (s390_vec_load_bndry,        s390_vec_load_bndry_s8,s390_vec_load_bndry_dbl,B_VX,        BT_FN_OV4SI_INTCONSTPTR_INT)
OB_DEF_VAR (s390_vec_load_bndry_s8,     s390_vlbb,          0,                  O2_U16,             BT_OV_V16QI_SCHARCONSTPTR_USHORT)
OB_DEF_VAR (s390_vec_load_bndry_u8,     s390_vlbb,          0,                  O2_U16,             BT_OV_UV16QI_UCHARCONSTPTR_USHORT)
OB_DEF_VAR (s390_vec_load_bndry_s16,    s390_vlbb,          0,                  O2_U16,             BT_OV_V8HI_SHORTCONSTPTR_USHORT)
OB_DEF_VAR (s390_vec_load_bndry_u16,    s390_vlbb,          0,                  O2_U16,             BT_OV_UV8HI_USHORTCONSTPTR_USHORT)
OB_DEF_VAR (s390_vec_load_bndry_s32,    s390_vlbb,          0,                  O2_U16,             BT_OV_V4SI_INTCONSTPTR_USHORT)
OB_DEF_VAR (s390_vec_load_bndry_u32,    s390_vlbb,          0,                  O2_U16,             BT_OV_UV4SI_UINTCONSTPTR_USHORT)
OB_DEF_VAR (s390_vec_load_bndry_s64,    s390_vlbb,          0,                  O2_U16,             BT_OV_V2DI_LONGLONGCONSTPTR_USHORT)
OB_DEF_VAR (s390_vec_load_bndry_u64,    s390_vlbb,          0,                  O2_U16,             BT_OV_UV2DI_ULONGLONGCONSTPTR_USHORT)
OB_DEF_VAR (s390_vec_load_bndry_flt,    s390_vlbb,          B_VXE,              O2_U16,             BT_OV_V4SF_FLTCONSTPTR_USHORT)
OB_DEF_VAR (s390_vec_load_bndry_dbl,    s390_vlbb,          0,                  O2_U16,             BT_OV_V2DF_DBLCONSTPTR_USHORT)

B_DEF      (s390_vlbb,                  vlbb,               0,                  B_VX,               O2_U3,              BT_FN_UV16QI_UCHARCONSTPTR_USHORT)

OB_DEF     (s390_vec_load_pair,         s390_vec_load_pair_s64,s390_vec_load_pair_u64,B_VX,         BT_FN_OV2DI_LONGLONG_LONGLONG)
OB_DEF_VAR (s390_vec_load_pair_s64,     MAX,                0,                  0,                  BT_OV_V2DI_LONGLONG_LONGLONG)            /* vlvgp */
OB_DEF_VAR (s390_vec_load_pair_u64,     MAX,                0,                  0,                  BT_OV_UV2DI_ULONGLONG_ULONGLONG)         /* vlvgp */

/* First two operands are swapped in s390-c.c */
OB_DEF     (s390_vec_load_len,          s390_vec_load_len_s8,s390_vec_load_len_dbl,B_VX,            BT_FN_OV4SI_INTCONSTPTR_UINT)
OB_DEF_VAR (s390_vec_load_len_s8,       s390_vll,           0,                  0,                  BT_OV_V16QI_SCHARCONSTPTR_UINT)
OB_DEF_VAR (s390_vec_load_len_u8,       s390_vll,           0,                  0,                  BT_OV_UV16QI_UCHARCONSTPTR_UINT)
OB_DEF_VAR (s390_vec_load_len_s16,      s390_vll,           0,                  0,                  BT_OV_V8HI_SHORTCONSTPTR_UINT)
OB_DEF_VAR (s390_vec_load_len_u16,      s390_vll,           0,                  0,                  BT_OV_UV8HI_USHORTCONSTPTR_UINT)
OB_DEF_VAR (s390_vec_load_len_s32,      s390_vll,           0,                  0,                  BT_OV_V4SI_INTCONSTPTR_UINT)
OB_DEF_VAR (s390_vec_load_len_u32,      s390_vll,           0,                  0,                  BT_OV_UV4SI_UINTCONSTPTR_UINT)
OB_DEF_VAR (s390_vec_load_len_s64,      s390_vll,           0,                  0,                  BT_OV_V2DI_LONGLONGCONSTPTR_UINT)
OB_DEF_VAR (s390_vec_load_len_u64,      s390_vll,           0,                  0,                  BT_OV_UV2DI_ULONGLONGCONSTPTR_UINT)
OB_DEF_VAR (s390_vec_load_len_flt,      s390_vll,           B_VXE,              0,                  BT_OV_V4SF_FLTCONSTPTR_UINT)
OB_DEF_VAR (s390_vec_load_len_dbl,      s390_vll,           0,                  0,                  BT_OV_V2DF_DBLCONSTPTR_UINT)

B_DEF      (s390_vll,                   vllv16qi,           0,                  B_VX,               0,                  BT_FN_V16QI_UINT_VOIDCONSTPTR)
B_DEF      (s390_vlrlr,                 vlrlrv16qi,         0,                  B_VXE,              0,                  BT_FN_V16QI_UINT_VOIDCONSTPTR)

OB_DEF     (s390_vec_mergeh,            s390_vec_mergeh_s8, s390_vec_mergeh_dbl,B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_mergeh_s8,         s390_vmrhb,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_mergeh_u8,         s390_vmrhb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_mergeh_b8,         s390_vmrhb,         0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_mergeh_s16,        s390_vmrhh,         0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_mergeh_u16,        s390_vmrhh,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_mergeh_b16,        s390_vmrhh,         0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_mergeh_s32,        s390_vmrhf,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_mergeh_u32,        s390_vmrhf,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_mergeh_b32,        s390_vmrhf,         0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_mergeh_s64,        s390_vmrhg,         0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_mergeh_u64,        s390_vmrhg,         0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_mergeh_b64,        s390_vmrhg,         0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_mergeh_flt,        s390_vmrhf_flt,     B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_mergeh_dbl,        s390_vmrhg_dbl,     0,                  0,                  BT_OV_V2DF_V2DF_V2DF)

B_DEF      (s390_vmrhb,                 vec_mergehv16qi,    0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vmrhh,                 vec_mergehv8hi,     0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vmrhf,                 vec_mergehv4si,     0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vmrhf_flt,             vec_mergehv4sf,     0,                  B_INT | B_VX,       0,                  BT_FN_V4SF_V4SF_V4SF)
B_DEF      (s390_vmrhg,                 vec_mergehv2di,     0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI_UV2DI)
B_DEF      (s390_vmrhg_dbl,             vec_mergehv2df,     0,                  B_INT | B_VX,       0,                  BT_FN_V2DF_V2DF_V2DF)

OB_DEF     (s390_vec_mergel,            s390_vec_mergel_s8, s390_vec_mergel_dbl,B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_mergel_s8,         s390_vmrlb,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_mergel_u8,         s390_vmrlb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_mergel_b8,         s390_vmrlb,         0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_mergel_s16,        s390_vmrlh,         0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_mergel_u16,        s390_vmrlh,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_mergel_b16,        s390_vmrlh,         0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_mergel_s32,        s390_vmrlf,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_mergel_u32,        s390_vmrlf,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_mergel_b32,        s390_vmrlf,         0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_mergel_s64,        s390_vmrlg,         0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_mergel_u64,        s390_vmrlg,         0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_mergel_b64,        s390_vmrlg,         0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_mergel_flt,        s390_vmrlf_flt,     B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_mergel_dbl,        s390_vmrlg_dbl,     0,                  0,                  BT_OV_V2DF_V2DF_V2DF)

B_DEF      (s390_vmrlb,                 vec_mergelv16qi,    0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vmrlh,                 vec_mergelv8hi,     0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vmrlf,                 vec_mergelv4si,     0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vmrlf_flt,             vec_mergelv4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_V4SF_V4SF_V4SF)
B_DEF      (s390_vmrlg,                 vec_mergelv2di,     0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI_UV2DI)
B_DEF      (s390_vmrlg_dbl,             vec_mergelv2df,     0,                  B_INT | B_VX,       0,                  BT_FN_V2DF_V2DF_V2DF)

OB_DEF     (s390_vec_pack,              s390_vec_pack_s16,  s390_vec_pack_b64,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_pack_s16,          s390_vpkh,          0,                  0,                  BT_OV_V16QI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_pack_u16,          s390_vpkh,          0,                  0,                  BT_OV_UV16QI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_pack_b16,          s390_vpkh,          0,                  0,                  BT_OV_BV16QI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_pack_s32,          s390_vpkf,          0,                  0,                  BT_OV_V8HI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_pack_u32,          s390_vpkf,          0,                  0,                  BT_OV_UV8HI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_pack_b32,          s390_vpkf,          0,                  0,                  BT_OV_BV8HI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_pack_s64,          s390_vpkg,          0,                  0,                  BT_OV_V4SI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_pack_u64,          s390_vpkg,          0,                  0,                  BT_OV_UV4SI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_pack_b64,          s390_vpkg,          0,                  0,                  BT_OV_BV4SI_BV2DI_BV2DI)

B_DEF      (s390_vpkh,                  vec_packv8hi,       0,                  B_VX,               0,                  BT_FN_UV16QI_UV8HI_UV8HI)
B_DEF      (s390_vpkf,                  vec_packv4si,       0,                  B_VX,               0,                  BT_FN_UV8HI_UV4SI_UV4SI)
B_DEF      (s390_vpkg,                  vec_packv2di,       0,                  B_VX,               0,                  BT_FN_UV4SI_UV2DI_UV2DI)

OB_DEF     (s390_vec_packs,             s390_vec_packs_s16, s390_vec_packs_u64, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_packs_s16,         s390_vpksh,         0,                  0,                  BT_OV_V16QI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_packs_u16,         s390_vpklsh,        0,                  0,                  BT_OV_UV16QI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_packs_s32,         s390_vpksf,         0,                  0,                  BT_OV_V8HI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_packs_u32,         s390_vpklsf,        0,                  0,                  BT_OV_UV8HI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_packs_s64,         s390_vpksg,         0,                  0,                  BT_OV_V4SI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_packs_u64,         s390_vpklsg,        0,                  0,                  BT_OV_UV4SI_UV2DI_UV2DI)

B_DEF      (s390_vpksh,                 vec_packsv8hi,      0,                  B_VX,               0,                  BT_FN_V16QI_V8HI_V8HI)
B_DEF      (s390_vpklsh,                vec_packsuv8hi,     0,                  B_VX,               0,                  BT_FN_UV16QI_UV8HI_UV8HI)
B_DEF      (s390_vpksf,                 vec_packsv4si,      0,                  B_VX,               0,                  BT_FN_V8HI_V4SI_V4SI)
B_DEF      (s390_vpklsf,                vec_packsuv4si,     0,                  B_VX,               0,                  BT_FN_UV8HI_UV4SI_UV4SI)
B_DEF      (s390_vpksg,                 vec_packsv2di,      0,                  B_VX,               0,                  BT_FN_V4SI_V2DI_V2DI)
B_DEF      (s390_vpklsg,                vec_packsuv2di,     0,                  B_VX,               0,                  BT_FN_UV4SI_UV2DI_UV2DI)

OB_DEF     (s390_vec_packs_cc,          s390_vec_packs_cc_s16,s390_vec_packs_cc_u64,B_VX,           BT_FN_OV4SI_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vec_packs_cc_s16,      s390_vpkshs,        0,                  0,                  BT_OV_V16QI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vec_packs_cc_u16,      s390_vpklshs,       0,                  0,                  BT_OV_UV16QI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vec_packs_cc_s32,      s390_vpksfs,        0,                  0,                  BT_OV_V8HI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vec_packs_cc_u32,      s390_vpklsfs,       0,                  0,                  BT_OV_UV8HI_UV4SI_UV4SI_INTPTR)
OB_DEF_VAR (s390_vec_packs_cc_s64,      s390_vpksgs,        0,                  0,                  BT_OV_V4SI_V2DI_V2DI_INTPTR)
OB_DEF_VAR (s390_vec_packs_cc_u64,      s390_vpklsgs,       0,                  0,                  BT_OV_UV4SI_UV2DI_UV2DI_INTPTR)

B_DEF      (s390_vpkshs,                vec_packs_ccv8hi,   0,                  B_VX,               0,                  BT_FN_V16QI_V8HI_V8HI_INTPTR)
B_DEF      (s390_vpklshs,               vec_packsu_ccv8hi,  0,                  B_VX,               0,                  BT_FN_UV16QI_UV8HI_UV8HI_INTPTR)
B_DEF      (s390_vpksfs,                vec_packs_ccv4si,   0,                  B_VX,               0,                  BT_FN_V8HI_V4SI_V4SI_INTPTR)
B_DEF      (s390_vpklsfs,               vec_packsu_ccv4si,  0,                  B_VX,               0,                  BT_FN_UV8HI_UV4SI_UV4SI_INTPTR)
B_DEF      (s390_vpksgs,                vec_packs_ccv2di,   0,                  B_VX,               0,                  BT_FN_V4SI_V2DI_V2DI_INTPTR)
B_DEF      (s390_vpklsgs,               vec_packsu_ccv2di,  0,                  B_VX,               0,                  BT_FN_UV4SI_UV2DI_UV2DI_INTPTR)

OB_DEF     (s390_vec_packsu,            s390_vec_packsu_s16,s390_vec_packsu_u64,B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_packsu_s16,        s390_vec_packsu_u16,0,                  0,                  BT_OV_UV16QI_V8HI_V8HI)                  /* vpklsh */
OB_DEF_VAR (s390_vec_packsu_u16,        s390_vpklsh,        0,                  0,                  BT_OV_UV16QI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_packsu_s32,        s390_vec_packsu_u32,0,                  0,                  BT_OV_UV8HI_V4SI_V4SI)                   /* vpklsf */
OB_DEF_VAR (s390_vec_packsu_u32,        s390_vpklsf,        0,                  0,                  BT_OV_UV8HI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_packsu_s64,        s390_vec_packsu_u64,0,                  0,                  BT_OV_UV4SI_V2DI_V2DI)                   /* vpklsg */
OB_DEF_VAR (s390_vec_packsu_u64,        s390_vpklsg,        0,                  0,                  BT_OV_UV4SI_UV2DI_UV2DI)

B_DEF      (s390_vec_packsu_u16,        vec_packsu_uv8hi,   0,                  B_INT | B_VX,       0,                  BT_FN_UV16QI_UV8HI_UV8HI)                /* vpklsh */
B_DEF      (s390_vec_packsu_u32,        vec_packsu_uv4si,   0,                  B_INT | B_VX,       0,                  BT_FN_UV8HI_UV4SI_UV4SI)                 /* vpklsf */
B_DEF      (s390_vec_packsu_u64,        vec_packsu_uv2di,   0,                  B_INT | B_VX,       0,                  BT_FN_UV4SI_UV2DI_UV2DI)                 /* vpklsg */

OB_DEF     (s390_vec_packsu_cc,         s390_vec_packsu_cc_u16,s390_vec_packsu_cc_u64,B_VX,         BT_FN_OV4SI_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vec_packsu_cc_u16,     s390_vpklshs,       0,                  0,                  BT_OV_UV16QI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vec_packsu_cc_u32,     s390_vpklsfs,       0,                  0,                  BT_OV_UV8HI_UV4SI_UV4SI_INTPTR)
OB_DEF_VAR (s390_vec_packsu_cc_u64,     s390_vpklsgs,       0,                  0,                  BT_OV_UV4SI_UV2DI_UV2DI_INTPTR)

OB_DEF     (s390_vec_perm,              s390_vec_perm_s8,   s390_vec_perm_dbl,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_perm_s8,           s390_vperm,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI_UV16QI)
OB_DEF_VAR (s390_vec_perm_b8,           s390_vperm,         0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_perm_u8,           s390_vperm,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_perm_s16,          s390_vperm,         0,                  0,                  BT_OV_V8HI_V8HI_V8HI_UV16QI)
OB_DEF_VAR (s390_vec_perm_b16,          s390_vperm,         0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI_UV16QI)
OB_DEF_VAR (s390_vec_perm_u16,          s390_vperm,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV16QI)
OB_DEF_VAR (s390_vec_perm_s32,          s390_vperm,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI_UV16QI)
OB_DEF_VAR (s390_vec_perm_b32,          s390_vperm,         0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI_UV16QI)
OB_DEF_VAR (s390_vec_perm_u32,          s390_vperm,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV16QI)
OB_DEF_VAR (s390_vec_perm_s64,          s390_vperm,         0,                  0,                  BT_OV_V2DI_V2DI_V2DI_UV16QI)
OB_DEF_VAR (s390_vec_perm_b64,          s390_vperm,         0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI_UV16QI)
OB_DEF_VAR (s390_vec_perm_u64,          s390_vperm,         0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI_UV16QI)
OB_DEF_VAR (s390_vec_perm_flt,          s390_vperm,         B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF_UV16QI)
OB_DEF_VAR (s390_vec_perm_dbl,          s390_vperm,         0,                  0,                  BT_OV_V2DF_V2DF_V2DF_UV16QI)

B_DEF      (s390_vperm,                 vec_permv16qi,      0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_permi,             s390_vec_permi_s64, s390_vec_permi_dbl, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_INT)
OB_DEF_VAR (s390_vec_permi_s64,         s390_vpdi,          0,                  O3_U2,              BT_OV_V2DI_V2DI_V2DI_INT)
OB_DEF_VAR (s390_vec_permi_b64,         s390_vpdi,          0,                  O3_U2,              BT_OV_BV2DI_BV2DI_BV2DI_INT)
OB_DEF_VAR (s390_vec_permi_u64,         s390_vpdi,          0,                  O3_U2,              BT_OV_UV2DI_UV2DI_UV2DI_INT)
OB_DEF_VAR (s390_vec_permi_dbl,         s390_vpdi,          0,                  O3_U2,              BT_OV_V2DF_V2DF_V2DF_INT)

B_DEF      (s390_vpdi,                  vec_permiv2di,      0,                  B_VX,               O3_U2,              BT_FN_UV2DI_UV2DI_UV2DI_INT)

OB_DEF     (s390_vec_splat,             s390_vec_splat2_s8, s390_vec_splat2_dbl,B_VX,               BT_FN_OV4SI_OV4SI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_s8,         s390_vrepb,         0,                  O2_U4,              BT_OV_V16QI_V16QI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_b8,         s390_vrepb,         0,                  O2_U4,              BT_OV_BV16QI_BV16QI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_u8,         s390_vrepb,         0,                  O2_U4,              BT_OV_UV16QI_UV16QI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_s16,        s390_vreph,         0,                  O2_U3,              BT_OV_V8HI_V8HI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_b16,        s390_vreph,         0,                  O2_U3,              BT_OV_BV8HI_BV8HI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_u16,        s390_vreph,         0,                  O2_U3,              BT_OV_UV8HI_UV8HI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_s32,        s390_vrepf,         0,                  O2_U2,              BT_OV_V4SI_V4SI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_b32,        s390_vrepf,         0,                  O2_U2,              BT_OV_BV4SI_BV4SI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_u32,        s390_vrepf,         0,                  O2_U2,              BT_OV_UV4SI_UV4SI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_s64,        s390_vrepg,         0,                  O2_U1,              BT_OV_V2DI_V2DI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_b64,        s390_vrepg,         0,                  O2_U1,              BT_OV_BV2DI_BV2DI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_u64,        s390_vrepg,         0,                  O2_U1,              BT_OV_UV2DI_UV2DI_UCHAR)
OB_DEF_VAR (s390_vec_splat2_flt,        s390_vrepf_flt,     B_VXE,              O2_U2,              BT_OV_V4SF_V4SF_UCHAR)                   /* vrepf */
OB_DEF_VAR (s390_vec_splat2_dbl,        s390_vrepg_dbl,     0,                  O2_U1,              BT_OV_V2DF_V2DF_UCHAR)                   /* vrepg */

B_DEF      (s390_vrepb,                 vec_splatv16qi,     0,                  B_VX,               O2_U4,              BT_FN_UV16QI_UV16QI_UCHAR)
B_DEF      (s390_vreph,                 vec_splatv8hi,      0,                  B_VX,               O2_U3,              BT_FN_UV8HI_UV8HI_UCHAR)
B_DEF      (s390_vrepf,                 vec_splatv4si,      0,                  B_VX,               O2_U2,              BT_FN_UV4SI_UV4SI_UCHAR)
B_DEF      (s390_vrepf_flt,             vec_splatv4sf,      0,                  B_INT | B_VXE,      O2_U2,              BT_FN_V4SF_V4SF_UCHAR)
B_DEF      (s390_vrepg,                 vec_splatv2di,      0,                  B_VX,               O2_U1,              BT_FN_UV2DI_UV2DI_UCHAR)
B_DEF      (s390_vrepg_dbl,             vec_splatv2df,      0,                  B_INT | B_VX,       O2_U1,              BT_FN_V2DF_V2DF_UCHAR)

OB_DEF     (s390_vec_scatter_element,   s390_vec_scatter_element_s32,s390_vec_scatter_element_dbl,B_VX,BT_FN_VOID_V4SI_V4SI_INTPTR_ULONGLONG)
OB_DEF_VAR (s390_vec_scatter_element_s32,s390_vscef,        0,                  O4_U2,              BT_OV_VOID_V4SI_UV4SI_INTPTR_ULONGLONG)
OB_DEF_VAR (s390_vec_scatter_element_b32,s390_vscef,        0,                  O4_U2,              BT_OV_VOID_BV4SI_UV4SI_UINTPTR_ULONGLONG)
OB_DEF_VAR (s390_vec_scatter_element_u32,s390_vscef,        0,                  O4_U2,              BT_OV_VOID_UV4SI_UV4SI_UINTPTR_ULONGLONG)
OB_DEF_VAR (s390_vec_scatter_element_s64,s390_vsceg,        0,                  O4_U1,              BT_OV_VOID_V2DI_UV2DI_LONGLONGPTR_ULONGLONG)
OB_DEF_VAR (s390_vec_scatter_element_b64,s390_vsceg,        0,                  O4_U1,              BT_OV_VOID_BV2DI_UV2DI_ULONGLONGPTR_ULONGLONG)
OB_DEF_VAR (s390_vec_scatter_element_u64,s390_vsceg,        0,                  O4_U1,              BT_OV_VOID_UV2DI_UV2DI_ULONGLONGPTR_ULONGLONG)
OB_DEF_VAR (s390_vec_scatter_element_flt,s390_vscef,        B_VXE,              O4_U2,              BT_OV_VOID_V4SF_V4SF_FLTPTR_ULONGLONG)
OB_DEF_VAR (s390_vec_scatter_element_dbl,s390_vsceg,        0,                  O4_U1,              BT_OV_VOID_V2DF_UV2DI_DBLPTR_ULONGLONG)

B_DEF      (s390_vscef,                 vec_scatter_elementv4si,0,              B_VX,               O4_U2,              BT_FN_VOID_UV4SI_UV4SI_UINTPTR_ULONGLONG)
B_DEF      (s390_vsceg,                 vec_scatter_elementv2di,0,              B_VX,               O4_U1,              BT_FN_VOID_UV2DI_UV2DI_ULONGLONGPTR_ULONGLONG)

/* First two operands are swapped in s390-c.c */
OB_DEF     (s390_vec_sel,               s390_vec_sel_b8_a,  s390_vec_sel_dbl_b, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_sel_b8_a,          s390_vsel,          0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_sel_b8_b,          s390_vsel,          0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_sel_s8_a,          s390_vsel,          0,                  0,                  BT_OV_V16QI_V16QI_V16QI_UV16QI)
OB_DEF_VAR (s390_vec_sel_s8_b,          s390_vsel,          0,                  0,                  BT_OV_V16QI_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_sel_u8_a,          s390_vsel,          0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_sel_u8_b,          s390_vsel,          0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_sel_b16_a,         s390_vsel,          0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_sel_b16_b,         s390_vsel,          0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_sel_s16_a,         s390_vsel,          0,                  0,                  BT_OV_V8HI_V8HI_V8HI_UV8HI)
OB_DEF_VAR (s390_vec_sel_s16_b,         s390_vsel,          0,                  0,                  BT_OV_V8HI_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_sel_u16_a,         s390_vsel,          0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_sel_u16_b,         s390_vsel,          0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_sel_b32_a,         s390_vsel,          0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_sel_b32_b,         s390_vsel,          0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_sel_s32_a,         s390_vsel,          0,                  0,                  BT_OV_V4SI_V4SI_V4SI_UV4SI)
OB_DEF_VAR (s390_vec_sel_s32_b,         s390_vsel,          0,                  0,                  BT_OV_V4SI_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_sel_u32_a,         s390_vsel,          0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_sel_u32_b,         s390_vsel,          0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_sel_b64_a,         s390_vsel,          0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_sel_b64_b,         s390_vsel,          0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_sel_s64_a,         s390_vsel,          0,                  0,                  BT_OV_V2DI_V2DI_V2DI_UV2DI)
OB_DEF_VAR (s390_vec_sel_s64_b,         s390_vsel,          0,                  0,                  BT_OV_V2DI_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_sel_u64_a,         s390_vsel,          0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_sel_u64_b,         s390_vsel,          0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_sel_flt_a,         s390_vsel,          B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF_UV4SI)
OB_DEF_VAR (s390_vec_sel_flt_b,         s390_vsel,          B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF_BV4SI)
OB_DEF_VAR (s390_vec_sel_dbl_a,         s390_vsel,          0,                  0,                  BT_OV_V2DF_V2DF_V2DF_UV2DI)
OB_DEF_VAR (s390_vec_sel_dbl_b,         s390_vsel,          0,                  0,                  BT_OV_V2DF_V2DF_V2DF_BV2DI)

B_DEF      (s390_vsel,                  vec_selv16qi,       0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_extend_s64,        s390_vec_extend_s64_s8,s390_vec_extend_s64_s32,B_VX,        BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_extend_s64_s8,     s390_vsegb,         0,                  0,                  BT_OV_V2DI_V16QI)
OB_DEF_VAR (s390_vec_extend_s64_s16,    s390_vsegh,         0,                  0,                  BT_OV_V2DI_V8HI)
OB_DEF_VAR (s390_vec_extend_s64_s32,    s390_vsegf,         0,                  0,                  BT_OV_V2DI_V4SI)

B_DEF      (s390_vsegb,                 vec_extendv16qi,    0,                  B_VX,               0,                  BT_FN_V2DI_V16QI)
B_DEF      (s390_vsegh,                 vec_extendv8hi,     0,                  B_VX,               0,                  BT_FN_V2DI_V8HI)
B_DEF      (s390_vsegf,                 vec_extendv4si,     0,                  B_VX,               0,                  BT_FN_V2DI_V4SI)

OB_DEF     (s390_vec_xst,               s390_vec_xst_s8,    s390_vec_xst_dbl,   B_VX,               BT_FN_VOID_OV4SI_INT_VOIDPTR)
OB_DEF_VAR (s390_vec_xst_s8,            MAX,                0,                  O2_LIT,             BT_OV_VOID_V16QI_LONG_SCHARPTR)          /* vst */
OB_DEF_VAR (s390_vec_xst_u8,            MAX,                0,                  O2_LIT,             BT_OV_VOID_UV16QI_LONG_UCHARPTR)         /* vst */
OB_DEF_VAR (s390_vec_xst_s16,           MAX,                0,                  O2_LIT,             BT_OV_VOID_V8HI_LONG_SHORTPTR)           /* vst */
OB_DEF_VAR (s390_vec_xst_u16,           MAX,                0,                  O2_LIT,             BT_OV_VOID_UV8HI_LONG_USHORTPTR)         /* vst */
OB_DEF_VAR (s390_vec_xst_s32,           MAX,                0,                  O2_LIT,             BT_OV_VOID_V4SI_LONG_INTPTR)             /* vst */
OB_DEF_VAR (s390_vec_xst_u32,           MAX,                0,                  O2_LIT,             BT_OV_VOID_UV4SI_LONG_UINTPTR)           /* vst */
OB_DEF_VAR (s390_vec_xst_s64,           MAX,                0,                  O2_LIT,             BT_OV_VOID_V2DI_LONG_LONGLONGPTR)        /* vst */
OB_DEF_VAR (s390_vec_xst_u64,           MAX,                0,                  O2_LIT,             BT_OV_VOID_UV2DI_LONG_ULONGLONGPTR)      /* vst */
OB_DEF_VAR (s390_vec_xst_flt,           MAX,                0,                  O2_LIT,             BT_OV_VOID_V4SF_LONG_FLTPTR)             /* vst */
OB_DEF_VAR (s390_vec_xst_dbl,           MAX,                0,                  O2_LIT,             BT_OV_VOID_V2DF_LONG_DBLPTR)             /* vst */

OB_DEF     (s390_vec_xstd2,             s390_vec_xstd2_s8,  s390_vec_xstd2_dbl, B_DEP | B_VX,       BT_FN_VOID_OV4SI_INT_VOIDPTR)
OB_DEF_VAR (s390_vec_xstd2_s8,          MAX,                0,                  O2_LIT,             BT_OV_VOID_V16QI_LONG_SCHARPTR)          /* vst */
OB_DEF_VAR (s390_vec_xstd2_u8,          MAX,                0,                  O2_LIT,             BT_OV_VOID_UV16QI_LONG_UCHARPTR)         /* vst */
OB_DEF_VAR (s390_vec_xstd2_s16,         MAX,                0,                  O2_LIT,             BT_OV_VOID_V8HI_LONG_SHORTPTR)           /* vst */
OB_DEF_VAR (s390_vec_xstd2_u16,         MAX,                0,                  O2_LIT,             BT_OV_VOID_UV8HI_LONG_USHORTPTR)         /* vst */
OB_DEF_VAR (s390_vec_xstd2_s32,         MAX,                0,                  O2_LIT,             BT_OV_VOID_V4SI_LONG_INTPTR)             /* vst */
OB_DEF_VAR (s390_vec_xstd2_u32,         MAX,                0,                  O2_LIT,             BT_OV_VOID_UV4SI_LONG_UINTPTR)           /* vst */
OB_DEF_VAR (s390_vec_xstd2_s64,         MAX,                0,                  O2_LIT,             BT_OV_VOID_V2DI_LONG_LONGLONGPTR)        /* vst */
OB_DEF_VAR (s390_vec_xstd2_u64,         MAX,                0,                  O2_LIT,             BT_OV_VOID_UV2DI_LONG_ULONGLONGPTR)      /* vst */
OB_DEF_VAR (s390_vec_xstd2_dbl,         MAX,                0,                  O2_LIT,             BT_OV_VOID_V2DF_LONG_DBLPTR)             /* vst */

OB_DEF     (s390_vec_xstw4,             s390_vec_xstw4_s8,  s390_vec_xstw4_u32, B_DEP | B_VX,       BT_FN_VOID_OV4SI_INT_VOIDPTR)
OB_DEF_VAR (s390_vec_xstw4_s8,          MAX,                0,                  O2_LIT,             BT_OV_VOID_V16QI_LONG_SCHARPTR)          /* vst */
OB_DEF_VAR (s390_vec_xstw4_u8,          MAX,                0,                  O2_LIT,             BT_OV_VOID_UV16QI_LONG_UCHARPTR)         /* vst */
OB_DEF_VAR (s390_vec_xstw4_s16,         MAX,                0,                  O2_LIT,             BT_OV_VOID_V8HI_LONG_SHORTPTR)           /* vst */
OB_DEF_VAR (s390_vec_xstw4_u16,         MAX,                0,                  O2_LIT,             BT_OV_VOID_UV8HI_LONG_USHORTPTR)         /* vst */
OB_DEF_VAR (s390_vec_xstw4_s32,         MAX,                0,                  O2_LIT,             BT_OV_VOID_V4SI_LONG_INTPTR)             /* vst */
OB_DEF_VAR (s390_vec_xstw4_u32,         MAX,                0,                  O2_LIT,             BT_OV_VOID_UV4SI_LONG_UINTPTR)           /* vst */

OB_DEF     (s390_vec_store_len,         s390_vec_store_len_s8,s390_vec_store_len_dbl,B_VX,          BT_FN_VOID_OV4SI_VOIDPTR_UINT)
OB_DEF_VAR (s390_vec_store_len_s8,      s390_vstl,          0,                  0,                  BT_OV_VOID_V16QI_SCHARPTR_UINT)
OB_DEF_VAR (s390_vec_store_len_u8,      s390_vstl,          0,                  0,                  BT_OV_VOID_UV16QI_UCHARPTR_UINT)
OB_DEF_VAR (s390_vec_store_len_s16,     s390_vstl,          0,                  0,                  BT_OV_VOID_V8HI_SHORTPTR_UINT)
OB_DEF_VAR (s390_vec_store_len_u16,     s390_vstl,          0,                  0,                  BT_OV_VOID_UV8HI_USHORTPTR_UINT)
OB_DEF_VAR (s390_vec_store_len_s32,     s390_vstl,          0,                  0,                  BT_OV_VOID_V4SI_INTPTR_UINT)
OB_DEF_VAR (s390_vec_store_len_u32,     s390_vstl,          0,                  0,                  BT_OV_VOID_UV4SI_UINTPTR_UINT)
OB_DEF_VAR (s390_vec_store_len_s64,     s390_vstl,          0,                  0,                  BT_OV_VOID_V2DI_LONGLONGPTR_UINT)
OB_DEF_VAR (s390_vec_store_len_u64,     s390_vstl,          0,                  0,                  BT_OV_VOID_UV2DI_ULONGLONGPTR_UINT)
OB_DEF_VAR (s390_vec_store_len_flt,     s390_vstl,          B_VXE,              0,                  BT_OV_VOID_V4SF_FLTPTR_UINT)
OB_DEF_VAR (s390_vec_store_len_dbl,     s390_vstl,          0,                  0,                  BT_OV_VOID_V2DF_DBLPTR_UINT)

B_DEF      (s390_vstl,                  vstlv16qi,          0,                  B_VX,               0,                  BT_FN_VOID_V16QI_UINT_VOIDPTR)
B_DEF      (s390_vstrlr,                vstrlrv16qi,        0,                  B_VXE,              0,                  BT_FN_VOID_V16QI_UINT_VOIDPTR)

B_DEF      (s390_vec_bperm_u128,        vbpermv16qi,        0,                  B_VXE,              0,                  BT_FN_UV2DI_UV16QI_UV16QI)               /* vbperm */
B_DEF      (s390_vbperm,                vbpermv16qi,        0,                  B_VXE,              0,                  BT_FN_UV2DI_UV16QI_UV16QI)

OB_DEF     (s390_vec_unpackh,           s390_vec_unpackh_s8,s390_vec_unpackh_u32,B_VX,              BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_unpackh_s8,        s390_vuphb,         0,                  0,                  BT_OV_V8HI_V16QI)
OB_DEF_VAR (s390_vec_unpackh_b8,        s390_vuphb,         0,                  0,                  BT_OV_BV8HI_BV16QI)
OB_DEF_VAR (s390_vec_unpackh_u8,        s390_vuplhb,        0,                  0,                  BT_OV_UV8HI_UV16QI)
OB_DEF_VAR (s390_vec_unpackh_s16,       s390_vuphh,         0,                  0,                  BT_OV_V4SI_V8HI)
OB_DEF_VAR (s390_vec_unpackh_b16,       s390_vuphh,         0,                  0,                  BT_OV_BV4SI_BV8HI)
OB_DEF_VAR (s390_vec_unpackh_u16,       s390_vuplhh,        0,                  0,                  BT_OV_UV4SI_UV8HI)
OB_DEF_VAR (s390_vec_unpackh_s32,       s390_vuphf,         0,                  0,                  BT_OV_V2DI_V4SI)
OB_DEF_VAR (s390_vec_unpackh_b32,       s390_vuphf,         0,                  0,                  BT_OV_BV2DI_BV4SI)
OB_DEF_VAR (s390_vec_unpackh_u32,       s390_vuplhf,        0,                  0,                  BT_OV_UV2DI_UV4SI)

B_DEF      (s390_vuphb,                 vec_unpackhv16qi,   0,                  B_VX,               0,                  BT_FN_V8HI_V16QI)
B_DEF      (s390_vuplhb,                vec_unpackh_lv16qi, 0,                  B_VX,               0,                  BT_FN_UV8HI_UV16QI)
B_DEF      (s390_vuphh,                 vec_unpackhv8hi,    0,                  B_VX,               0,                  BT_FN_V4SI_V8HI)
B_DEF      (s390_vuplhh,                vec_unpackh_lv8hi,  0,                  B_VX,               0,                  BT_FN_UV4SI_UV8HI)
B_DEF      (s390_vuphf,                 vec_unpackhv4si,    0,                  B_VX,               0,                  BT_FN_V2DI_V4SI)
B_DEF      (s390_vuplhf,                vec_unpackh_lv4si,  0,                  B_VX,               0,                  BT_FN_UV2DI_UV4SI)

OB_DEF     (s390_vec_unpackl,           s390_vec_unpackl_s8,s390_vec_unpackl_u32,B_VX,              BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_unpackl_s8,        s390_vuplb,         0,                  0,                  BT_OV_V8HI_V16QI)
OB_DEF_VAR (s390_vec_unpackl_b8,        s390_vuplb,         0,                  0,                  BT_OV_BV8HI_BV16QI)
OB_DEF_VAR (s390_vec_unpackl_u8,        s390_vupllb,        0,                  0,                  BT_OV_UV8HI_UV16QI)
OB_DEF_VAR (s390_vec_unpackl_s16,       s390_vuplhw,        0,                  0,                  BT_OV_V4SI_V8HI)
OB_DEF_VAR (s390_vec_unpackl_b16,       s390_vupllh,        0,                  0,                  BT_OV_BV4SI_BV8HI)
OB_DEF_VAR (s390_vec_unpackl_u16,       s390_vupllh,        0,                  0,                  BT_OV_UV4SI_UV8HI)
OB_DEF_VAR (s390_vec_unpackl_s32,       s390_vuplf,         0,                  0,                  BT_OV_V2DI_V4SI)
OB_DEF_VAR (s390_vec_unpackl_b32,       s390_vuplf,         0,                  0,                  BT_OV_BV2DI_BV4SI)
OB_DEF_VAR (s390_vec_unpackl_u32,       s390_vupllf,        0,                  0,                  BT_OV_UV2DI_UV4SI)

B_DEF      (s390_vuplb,                 vec_unpacklv16qi,   0,                  B_VX,               0,                  BT_FN_V8HI_V16QI)
B_DEF      (s390_vupllb,                vec_unpackl_lv16qi, 0,                  B_VX,               0,                  BT_FN_UV8HI_UV16QI)
B_DEF      (s390_vuplhw,                vec_unpacklv8hi,    0,                  B_VX,               0,                  BT_FN_V4SI_V8HI)
B_DEF      (s390_vupllh,                vec_unpackl_lv8hi,  0,                  B_VX,               0,                  BT_FN_UV4SI_UV8HI)
B_DEF      (s390_vuplf,                 vec_unpacklv4si,    0,                  B_VX,               0,                  BT_FN_V2DI_V4SI)
B_DEF      (s390_vupllf,                vec_unpackl_lv4si,  0,                  B_VX,               0,                  BT_FN_UV2DI_UV4SI)

OB_DEF     (s390_vec_addc,              s390_vec_addc_u8,   s390_vec_addc_u64,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_addc_u8,           s390_vaccb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_addc_u16,          s390_vacch,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_addc_u32,          s390_vaccf,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_addc_u64,          s390_vaccg,         0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)

B_DEF      (s390_vaccb,                 vaccb_v16qi,        0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vacch,                 vacch_v8hi,         0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vaccf,                 vaccf_v4si,         0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vaccg,                 vaccg_v2di,         0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI_UV2DI)

B_DEF      (s390_vec_add_u128,          addti3,             0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vec_addc_u128,         vaccq_ti,           0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vec_adde_u128,         vacq,               0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vec_addec_u128,        vacccq,             0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_UV16QI)

B_DEF      (s390_vaq,                   addti3,             0,                  B_VX,               0,                  BT_FN_INT128_INT128_INT128)
B_DEF      (s390_vaccq,                 vaccq_ti,           0,                  B_VX,               0,                  BT_FN_INT128_INT128_INT128)
B_DEF      (s390_vacq,                  vacq,               0,                  B_VX,               0,                  BT_FN_INT128_INT128_INT128_INT128)
B_DEF      (s390_vacccq,                vacccq,             0,                  B_VX,               0,                  BT_FN_INT128_INT128_INT128_INT128)

OB_DEF     (s390_vec_and,               s390_vec_and_b8,    s390_vec_and_dbl_c, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_and_b8,            s390_vn,            0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_and_s8_a,          s390_vn,            B_DEP,              0,                  BT_OV_V16QI_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_and_s8_b,          s390_vn,            0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_and_s8_c,          s390_vn,            B_DEP,              0,                  BT_OV_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_and_u8_a,          s390_vn,            B_DEP,              0,                  BT_OV_UV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_and_u8_b,          s390_vn,            0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_and_u8_c,          s390_vn,            B_DEP,              0,                  BT_OV_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_and_b16,           s390_vn,            0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_and_s16_a,         s390_vn,            B_DEP,              0,                  BT_OV_V8HI_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_and_s16_b,         s390_vn,            0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_and_s16_c,         s390_vn,            B_DEP,              0,                  BT_OV_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_and_u16_a,         s390_vn,            B_DEP,              0,                  BT_OV_UV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_and_u16_b,         s390_vn,            0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_and_u16_c,         s390_vn,            B_DEP,              0,                  BT_OV_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_and_b32,           s390_vn,            0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_and_s32_a,         s390_vn,            B_DEP,              0,                  BT_OV_V4SI_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_and_s32_b,         s390_vn,            0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_and_s32_c,         s390_vn,            B_DEP,              0,                  BT_OV_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_and_u32_a,         s390_vn,            B_DEP,              0,                  BT_OV_UV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_and_u32_b,         s390_vn,            0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_and_u32_c,         s390_vn,            B_DEP,              0,                  BT_OV_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_and_b64,           s390_vn,            0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_and_s64_a,         s390_vn,            B_DEP,              0,                  BT_OV_V2DI_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_and_s64_b,         s390_vn,            0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_and_s64_c,         s390_vn,            B_DEP,              0,                  BT_OV_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_and_u64_a,         s390_vn,            B_DEP,              0,                  BT_OV_UV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_and_u64_b,         s390_vn,            0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_and_u64_c,         s390_vn,            B_DEP,              0,                  BT_OV_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_and_flt_a,         s390_vn,            B_VXE | B_DEP,      0,                  BT_OV_V4SF_BV4SI_V4SF)
OB_DEF_VAR (s390_vec_and_flt_b,         s390_vn,            B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_and_flt_c,         s390_vn,            B_VXE | B_DEP,      0,                  BT_OV_V4SF_V4SF_BV4SI)
OB_DEF_VAR (s390_vec_and_dbl_a,         s390_vn,            B_DEP,              0,                  BT_OV_V2DF_BV2DI_V2DF)
OB_DEF_VAR (s390_vec_and_dbl_b,         s390_vn,            0,                  0,                  BT_OV_V2DF_V2DF_V2DF)
OB_DEF_VAR (s390_vec_and_dbl_c,         s390_vn,            B_DEP,              0,                  BT_OV_V2DF_V2DF_BV2DI)

B_DEF      (s390_vn,                    andv16qi3,          0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_andc,              s390_vec_andc_b8,   s390_vec_andc_dbl_c,B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_andc_b8,           s390_vnc,           0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_andc_s8_a,         s390_vnc,           B_DEP,              0,                  BT_OV_V16QI_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_andc_s8_b,         s390_vnc,           0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_andc_s8_c,         s390_vnc,           B_DEP,              0,                  BT_OV_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_andc_u8_a,         s390_vnc,           B_DEP,              0,                  BT_OV_UV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_andc_u8_b,         s390_vnc,           0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_andc_u8_c,         s390_vnc,           B_DEP,              0,                  BT_OV_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_andc_b16,          s390_vnc,           0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_andc_s16_a,        s390_vnc,           B_DEP,              0,                  BT_OV_V8HI_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_andc_s16_b,        s390_vnc,           0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_andc_s16_c,        s390_vnc,           B_DEP,              0,                  BT_OV_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_andc_u16_a,        s390_vnc,           B_DEP,              0,                  BT_OV_UV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_andc_u16_b,        s390_vnc,           0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_andc_u16_c,        s390_vnc,           B_DEP,              0,                  BT_OV_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_andc_b32,          s390_vnc,           0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_andc_s32_a,        s390_vnc,           B_DEP,              0,                  BT_OV_V4SI_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_andc_s32_b,        s390_vnc,           0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_andc_s32_c,        s390_vnc,           B_DEP,              0,                  BT_OV_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_andc_u32_a,        s390_vnc,           B_DEP,              0,                  BT_OV_UV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_andc_u32_b,        s390_vnc,           0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_andc_u32_c,        s390_vnc,           B_DEP,              0,                  BT_OV_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_andc_b64,          s390_vnc,           0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_andc_s64_a,        s390_vnc,           B_DEP,              0,                  BT_OV_V2DI_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_andc_s64_b,        s390_vnc,           0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_andc_s64_c,        s390_vnc,           B_DEP,              0,                  BT_OV_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_andc_u64_a,        s390_vnc,           B_DEP,              0,                  BT_OV_UV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_andc_u64_b,        s390_vnc,           0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_andc_u64_c,        s390_vnc,           B_DEP,              0,                  BT_OV_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_andc_flt_a,        s390_vnc,           B_VXE | B_DEP,      0,                  BT_OV_V4SF_BV4SI_V4SF)
OB_DEF_VAR (s390_vec_andc_flt_b,        s390_vnc,           B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_andc_flt_c,        s390_vnc,           B_VXE | B_DEP,      0,                  BT_OV_V4SF_V4SF_BV4SI)
OB_DEF_VAR (s390_vec_andc_dbl_a,        s390_vnc,           B_DEP,              0,                  BT_OV_V2DF_BV2DI_V2DF)
OB_DEF_VAR (s390_vec_andc_dbl_b,        s390_vnc,           0,                  0,                  BT_OV_V2DF_V2DF_V2DF)
OB_DEF_VAR (s390_vec_andc_dbl_c,        s390_vnc,           B_DEP,              0,                  BT_OV_V2DF_V2DF_BV2DI)

B_DEF      (s390_vnc,                   vec_andcv16qi3,     0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_avg,               s390_vec_avg_s8,    s390_vec_avg_u64,   B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_avg_s8,            s390_vavgb,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_avg_u8,            s390_vavglb,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_avg_s16,           s390_vavgh,         0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_avg_u16,           s390_vavglh,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_avg_s32,           s390_vavgf,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_avg_u32,           s390_vavglf,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_avg_s64,           s390_vavgg,         0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_avg_u64,           s390_vavglg,        0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)

B_DEF      (s390_vavgb,                 vec_avgv16qi,       0,                  B_VX,               0,                  BT_FN_V16QI_V16QI_V16QI)
B_DEF      (s390_vavglb,                vec_avguv16qi,      0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vavgh,                 vec_avgv8hi,        0,                  B_VX,               0,                  BT_FN_V8HI_V8HI_V8HI)
B_DEF      (s390_vavglh,                vec_avguv8hi,       0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vavgf,                 vec_avgv4si,        0,                  B_VX,               0,                  BT_FN_V4SI_V4SI_V4SI)
B_DEF      (s390_vavglf,                vec_avguv4si,       0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vavgg,                 vec_avgv2di,        0,                  B_VX,               0,                  BT_FN_V2DI_V2DI_V2DI)
B_DEF      (s390_vavglg,                vec_avguv2di,       0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI_UV2DI)

B_DEF      (s390_vcksm,                 vec_checksum,       0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)

B_DEF      (s390_vceqbs,                vec_cmpeqv16qi_cc,  0,                  B_VX,               0,                  BT_FN_V16QI_UV16QI_UV16QI_INTPTR)
B_DEF      (s390_vceqhs,                vec_cmpeqv8hi_cc,   0,                  B_VX,               0,                  BT_FN_V8HI_UV8HI_UV8HI_INTPTR)
B_DEF      (s390_vceqfs,                vec_cmpeqv4si_cc,   0,                  B_VX,               0,                  BT_FN_V4SI_UV4SI_UV4SI_INTPTR)
B_DEF      (s390_vceqgs,                vec_cmpeqv2di_cc,   0,                  B_VX,               0,                  BT_FN_V2DI_UV2DI_UV2DI_INTPTR)
B_DEF      (s390_vfcesbs,               vec_cmpeqv4sf_cc,   0,                  B_VXE,              0,                  BT_FN_V4SI_V4SF_V4SF_INTPTR)
B_DEF      (s390_vfcedbs,               vec_cmpeqv2df_cc,   0,                  B_VX,               0,                  BT_FN_V2DI_V2DF_V2DF_INTPTR)

B_DEF      (s390_vchbs,                 vec_cmphv16qi_cc,   0,                  B_VX,               0,                  BT_FN_V16QI_V16QI_V16QI_INTPTR)
B_DEF      (s390_vchlbs,                vec_cmphlv16qi_cc,  0,                  B_VX,               0,                  BT_FN_V16QI_UV16QI_UV16QI_INTPTR)
B_DEF      (s390_vchhs,                 vec_cmphv8hi_cc,    0,                  B_VX,               0,                  BT_FN_V8HI_V8HI_V8HI_INTPTR)
B_DEF      (s390_vchlhs,                vec_cmphlv8hi_cc,   0,                  B_VX,               0,                  BT_FN_V8HI_UV8HI_UV8HI_INTPTR)
B_DEF      (s390_vchfs,                 vec_cmphv4si_cc,    0,                  B_VX,               0,                  BT_FN_V4SI_V4SI_V4SI_INTPTR)
B_DEF      (s390_vchlfs,                vec_cmphlv4si_cc,   0,                  B_VX,               0,                  BT_FN_V4SI_UV4SI_UV4SI_INTPTR)
B_DEF      (s390_vchgs,                 vec_cmphv2di_cc,    0,                  B_VX,               0,                  BT_FN_V2DI_V2DI_V2DI_INTPTR)
B_DEF      (s390_vchlgs,                vec_cmphlv2di_cc,   0,                  B_VX,               0,                  BT_FN_V2DI_UV2DI_UV2DI_INTPTR)
B_DEF      (s390_vfchsbs,               vec_cmphv4sf_cc,    0,                  B_VXE,              0,                  BT_FN_V4SI_V4SF_V4SF_INTPTR)
B_DEF      (s390_vfchdbs,               vec_cmphv2df_cc,    0,                  B_VX,               0,                  BT_FN_V2DI_V2DF_V2DF_INTPTR)

B_DEF      (s390_vfchesbs,              vec_cmphev4sf_cc,   0,                  B_VXE,              0,                  BT_FN_V4SI_V4SF_V4SF_INTPTR)
B_DEF      (s390_vfchedbs,              vec_cmphev2df_cc,   0,                  B_VX,               0,                  BT_FN_V2DI_V2DF_V2DF_INTPTR)

B_DEF      (vec_all_eqv16qi,            vec_all_eqv16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_all_eqv8hi,             vec_all_eqv8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_all_eqv4si,             vec_all_eqv4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_all_eqv2di,             vec_all_eqv2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_all_eqv4sf,             vec_all_eqv4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_all_eqv2df,             vec_all_eqv2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_all_nev16qi,            vec_all_nev16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_all_nev8hi,             vec_all_nev8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_all_nev4si,             vec_all_nev4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_all_nev2di,             vec_all_nev2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_all_nev4sf,             vec_all_nev4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_all_nev2df,             vec_all_nev2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_all_gev16qi,            vec_all_gev16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_V16QI_V16QI)
B_DEF      (vec_all_geuv16qi,           vec_all_geuv16qi,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_all_gev8hi,             vec_all_gev8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V8HI_V8HI)
B_DEF      (vec_all_geuv8hi,            vec_all_geuv8hi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_all_gev4si,             vec_all_gev4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V4SI_V4SI)
B_DEF      (vec_all_geuv4si,            vec_all_geuv4si,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_all_gev2di,             vec_all_gev2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DI_V2DI)
B_DEF      (vec_all_geuv2di,            vec_all_geuv2di,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_all_gev4sf,             vec_all_gev4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_all_gev2df,             vec_all_gev2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_all_gtv16qi,            vec_all_gtv16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_V16QI_V16QI)
B_DEF      (vec_all_gtuv16qi,           vec_all_gtuv16qi,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_all_gtv8hi,             vec_all_gtv8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V8HI_V8HI)
B_DEF      (vec_all_gtuv8hi,            vec_all_gtuv8hi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_all_gtv4si,             vec_all_gtv4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V4SI_V4SI)
B_DEF      (vec_all_gtuv4si,            vec_all_gtuv4si,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_all_gtv2di,             vec_all_gtv2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DI_V2DI)
B_DEF      (vec_all_gtuv2di,            vec_all_gtuv2di,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_all_gtv4sf,             vec_all_gtv4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_all_gtv2df,             vec_all_gtv2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_all_lev16qi,            vec_all_lev16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_V16QI_V16QI)
B_DEF      (vec_all_leuv16qi,           vec_all_leuv16qi,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_all_lev8hi,             vec_all_lev8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V8HI_V8HI)
B_DEF      (vec_all_leuv8hi,            vec_all_leuv8hi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_all_lev4si,             vec_all_lev4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V4SI_V4SI)
B_DEF      (vec_all_leuv4si,            vec_all_leuv4si,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_all_lev2di,             vec_all_lev2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DI_V2DI)
B_DEF      (vec_all_leuv2di,            vec_all_leuv2di,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_all_lev4sf,             vec_all_lev4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_all_lev2df,             vec_all_lev2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_all_ltv16qi,            vec_all_ltv16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_V16QI_V16QI)
B_DEF      (vec_all_ltuv16qi,           vec_all_ltuv16qi,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_all_ltv8hi,             vec_all_ltv8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V8HI_V8HI)
B_DEF      (vec_all_ltuv8hi,            vec_all_ltuv8hi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_all_ltv4si,             vec_all_ltv4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V4SI_V4SI)
B_DEF      (vec_all_ltuv4si,            vec_all_ltuv4si,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_all_ltv2di,             vec_all_ltv2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DI_V2DI)
B_DEF      (vec_all_ltuv2di,            vec_all_ltuv2di,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_all_ltv4sf,             vec_all_ltv4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_all_ltv2df,             vec_all_ltv2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

OB_DEF     (s390_vec_all_eq,            s390_vec_all_eq_s8_a,s390_vec_all_eq_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_all_eq_s8_a,       vec_all_eqv16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_all_eq_s8_b,       vec_all_eqv16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_eq_b8_a,       vec_all_eqv16qi,    0,                  0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_eq_b8_b,       vec_all_eqv16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_all_eq_b8_c,       vec_all_eqv16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_eq_u8_a,       vec_all_eqv16qi,    0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_eq_u8_b,       vec_all_eqv16qi,    B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_eq_s16_a,      vec_all_eqv8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_all_eq_s16_b,      vec_all_eqv8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_eq_b16_a,      vec_all_eqv8hi,     0,                  0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_eq_b16_b,      vec_all_eqv8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_all_eq_b16_c,      vec_all_eqv8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_eq_u16_a,      vec_all_eqv8hi,     0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_eq_u16_b,      vec_all_eqv8hi,     B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_eq_s32_a,      vec_all_eqv4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_all_eq_s32_b,      vec_all_eqv4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_eq_b32_a,      vec_all_eqv4si,     0,                  0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_eq_b32_b,      vec_all_eqv4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_all_eq_b32_c,      vec_all_eqv4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_eq_u32_a,      vec_all_eqv4si,     0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_eq_u32_b,      vec_all_eqv4si,     B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_eq_s64_a,      vec_all_eqv2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_all_eq_s64_b,      vec_all_eqv2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_eq_b64_a,      vec_all_eqv2di,     0,                  0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_eq_b64_b,      vec_all_eqv2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_all_eq_b64_c,      vec_all_eqv2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_eq_u64_a,      vec_all_eqv2di,     0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_eq_u64_b,      vec_all_eqv2di,     B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_eq_flt,        vec_all_eqv4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_all_eq_dbl,        vec_all_eqv2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_all_ne,            s390_vec_all_ne_s8_a,s390_vec_all_ne_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_all_ne_s8_a,       vec_all_nev16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_all_ne_s8_b,       vec_all_nev16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_ne_b8_a,       vec_all_nev16qi,    0,                  0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_ne_b8_b,       vec_all_nev16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_all_ne_b8_c,       vec_all_nev16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_ne_u8_a,       vec_all_nev16qi,    0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_ne_u8_b,       vec_all_nev16qi,    B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_ne_s16_a,      vec_all_nev8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_all_ne_s16_b,      vec_all_nev8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_ne_b16_a,      vec_all_nev8hi,     0,                  0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_ne_b16_b,      vec_all_nev8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_all_ne_b16_c,      vec_all_nev8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_ne_u16_a,      vec_all_nev8hi,     0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_ne_u16_b,      vec_all_nev8hi,     B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_ne_s32_a,      vec_all_nev4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_all_ne_s32_b,      vec_all_nev4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_ne_b32_a,      vec_all_nev4si,     0,                  0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_ne_b32_b,      vec_all_nev4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_all_ne_b32_c,      vec_all_nev4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_ne_u32_a,      vec_all_nev4si,     0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_ne_u32_b,      vec_all_nev4si,     B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_ne_s64_a,      vec_all_nev2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_all_ne_s64_b,      vec_all_nev2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_ne_b64_a,      vec_all_nev2di,     0,                  0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_ne_b64_b,      vec_all_nev2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_all_ne_b64_c,      vec_all_nev2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_ne_u64_a,      vec_all_nev2di,     0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_ne_u64_b,      vec_all_nev2di,     B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_ne_flt,        vec_all_nev4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_all_ne_dbl,        vec_all_nev2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_all_ge,            s390_vec_all_ge_s8_a,s390_vec_all_ge_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_all_ge_s8_a,       vec_all_gev16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_all_ge_s8_b,       vec_all_gev16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_ge_b8_a,       vec_all_geuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_ge_b8_b,       vec_all_gev16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_all_ge_b8_c,       vec_all_geuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_ge_u8_a,       vec_all_geuv16qi,   0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_ge_u8_b,       vec_all_geuv16qi,   B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_ge_s16_a,      vec_all_gev8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_all_ge_s16_b,      vec_all_gev8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_ge_b16_a,      vec_all_geuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_ge_b16_b,      vec_all_gev8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_all_ge_b16_c,      vec_all_geuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_ge_u16_a,      vec_all_geuv8hi,    0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_ge_u16_b,      vec_all_geuv8hi,    B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_ge_s32_a,      vec_all_gev4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_all_ge_s32_b,      vec_all_gev4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_ge_b32_a,      vec_all_geuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_ge_b32_b,      vec_all_gev4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_all_ge_b32_c,      vec_all_geuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_ge_u32_a,      vec_all_geuv4si,    0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_ge_u32_b,      vec_all_geuv4si,    B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_ge_s64_a,      vec_all_gev2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_all_ge_s64_b,      vec_all_gev2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_ge_b64_a,      vec_all_geuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_ge_b64_b,      vec_all_gev2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_all_ge_b64_c,      vec_all_geuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_ge_u64_a,      vec_all_geuv2di,    0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_ge_u64_b,      vec_all_geuv2di,    B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_ge_flt,        vec_all_gev4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_all_ge_dbl,        vec_all_gev2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_all_gt,            s390_vec_all_gt_s8_a,s390_vec_all_gt_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_all_gt_s8_a,       vec_all_gtv16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_all_gt_s8_b,       vec_all_gtv16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_gt_b8_a,       vec_all_gtuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_gt_b8_b,       vec_all_gtv16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_all_gt_b8_c,       vec_all_gtuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_gt_u8_a,       vec_all_gtuv16qi,   0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_gt_u8_b,       vec_all_gtuv16qi,   B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_gt_s16_a,      vec_all_gtv8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_all_gt_s16_b,      vec_all_gtv8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_gt_b16_a,      vec_all_gtuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_gt_b16_b,      vec_all_gtv8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_all_gt_b16_c,      vec_all_gtuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_gt_u16_a,      vec_all_gtuv8hi,    0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_gt_u16_b,      vec_all_gtuv8hi,    B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_gt_s32_a,      vec_all_gtv4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_all_gt_s32_b,      vec_all_gtv4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_gt_b32_a,      vec_all_gtuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_gt_b32_b,      vec_all_gtv4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_all_gt_b32_c,      vec_all_gtuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_gt_u32_a,      vec_all_gtuv4si,    0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_gt_u32_b,      vec_all_gtuv4si,    B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_gt_s64_a,      vec_all_gtv2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_all_gt_s64_b,      vec_all_gtv2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_gt_b64_a,      vec_all_gtuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_gt_b64_b,      vec_all_gtv2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_all_gt_b64_c,      vec_all_gtuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_gt_u64_a,      vec_all_gtuv2di,    0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_gt_u64_b,      vec_all_gtuv2di,    B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_gt_flt,        vec_all_gtv4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_all_gt_dbl,        vec_all_gtv2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_all_le,            s390_vec_all_le_s8_a,s390_vec_all_le_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_all_le_s8_a,       vec_all_lev16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_all_le_s8_b,       vec_all_lev16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_le_b8_a,       vec_all_leuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_le_b8_b,       vec_all_lev16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_all_le_b8_c,       vec_all_leuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_le_u8_a,       vec_all_leuv16qi,   0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_le_u8_b,       vec_all_leuv16qi,   B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_le_s16_a,      vec_all_lev8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_all_le_s16_b,      vec_all_lev8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_le_b16_a,      vec_all_leuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_le_b16_b,      vec_all_lev8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_all_le_b16_c,      vec_all_leuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_le_u16_a,      vec_all_leuv8hi,    0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_le_u16_b,      vec_all_leuv8hi,    B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_le_s32_a,      vec_all_lev4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_all_le_s32_b,      vec_all_lev4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_le_b32_a,      vec_all_leuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_le_b32_b,      vec_all_lev4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_all_le_b32_c,      vec_all_leuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_le_u32_a,      vec_all_leuv4si,    0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_le_u32_b,      vec_all_leuv4si,    B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_le_s64_a,      vec_all_lev2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_all_le_s64_b,      vec_all_lev2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_le_b64_a,      vec_all_leuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_le_b64_b,      vec_all_lev2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_all_le_b64_c,      vec_all_leuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_le_u64_a,      vec_all_leuv2di,    0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_le_u64_b,      vec_all_leuv2di,    B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_le_flt,        vec_all_lev4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_all_le_dbl,        vec_all_lev2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_all_lt,            s390_vec_all_lt_s8_a,s390_vec_all_lt_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_all_lt_s8_a,       vec_all_ltv16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_all_lt_s8_b,       vec_all_ltv16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_lt_b8_a,       vec_all_ltuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_lt_b8_b,       vec_all_ltv16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_all_lt_b8_c,       vec_all_ltuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_lt_u8_a,       vec_all_ltuv16qi,   0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_all_lt_u8_b,       vec_all_ltuv16qi,   B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_all_lt_s16_a,      vec_all_ltv8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_all_lt_s16_b,      vec_all_ltv8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_lt_b16_a,      vec_all_ltuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_lt_b16_b,      vec_all_ltv8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_all_lt_b16_c,      vec_all_ltuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_lt_u16_a,      vec_all_ltuv8hi,    0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_all_lt_u16_b,      vec_all_ltuv8hi,    B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_all_lt_s32_a,      vec_all_ltv4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_all_lt_s32_b,      vec_all_ltv4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_lt_b32_a,      vec_all_ltuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_lt_b32_b,      vec_all_ltv4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_all_lt_b32_c,      vec_all_ltuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_lt_u32_a,      vec_all_ltuv4si,    0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_all_lt_u32_b,      vec_all_ltuv4si,    B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_all_lt_s64_a,      vec_all_ltv2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_all_lt_s64_b,      vec_all_ltv2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_lt_b64_a,      vec_all_ltuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_lt_b64_b,      vec_all_ltv2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_all_lt_b64_c,      vec_all_ltuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_lt_u64_a,      vec_all_ltuv2di,    0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_all_lt_u64_b,      vec_all_ltuv2di,    B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_all_lt_flt,        vec_all_ltv4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_all_lt_dbl,        vec_all_ltv2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

B_DEF      (vec_any_eqv16qi,            vec_any_eqv16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_any_eqv8hi,             vec_any_eqv8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_any_eqv4si,             vec_any_eqv4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_any_eqv2di,             vec_any_eqv2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_any_eqv4sf,             vec_any_eqv4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_any_eqv2df,             vec_any_eqv2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_any_nev16qi,            vec_any_nev16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_any_nev8hi,             vec_any_nev8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_any_nev4si,             vec_any_nev4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_any_nev2di,             vec_any_nev2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_any_nev4sf,             vec_any_nev4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_any_nev2df,             vec_any_nev2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_any_gev16qi,            vec_any_gev16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_V16QI_V16QI)
B_DEF      (vec_any_geuv16qi,           vec_any_geuv16qi,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_any_gev8hi,             vec_any_gev8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V8HI_V8HI)
B_DEF      (vec_any_geuv8hi,            vec_any_geuv8hi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_any_gev4si,             vec_any_gev4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V4SI_V4SI)
B_DEF      (vec_any_geuv4si,            vec_any_geuv4si,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_any_gev2di,             vec_any_gev2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DI_V2DI)
B_DEF      (vec_any_geuv2di,            vec_any_geuv2di,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_any_gev4sf,             vec_any_gev4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_any_gev2df,             vec_any_gev2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_any_gtv16qi,            vec_any_gtv16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_V16QI_V16QI)
B_DEF      (vec_any_gtuv16qi,           vec_any_gtuv16qi,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_any_gtv8hi,             vec_any_gtv8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V8HI_V8HI)
B_DEF      (vec_any_gtuv8hi,            vec_any_gtuv8hi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_any_gtv4si,             vec_any_gtv4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V4SI_V4SI)
B_DEF      (vec_any_gtuv4si,            vec_any_gtuv4si,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_any_gtv2di,             vec_any_gtv2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DI_V2DI)
B_DEF      (vec_any_gtuv2di,            vec_any_gtuv2di,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_any_gtv4sf,             vec_any_gtv4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_any_gtv2df,             vec_any_gtv2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_any_lev16qi,            vec_any_lev16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_V16QI_V16QI)
B_DEF      (vec_any_leuv16qi,           vec_any_leuv16qi,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_any_lev8hi,             vec_any_lev8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V8HI_V8HI)
B_DEF      (vec_any_leuv8hi,            vec_any_leuv8hi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_any_lev4si,             vec_any_lev4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V4SI_V4SI)
B_DEF      (vec_any_leuv4si,            vec_any_leuv4si,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_any_lev2di,             vec_any_lev2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DI_V2DI)
B_DEF      (vec_any_leuv2di,            vec_any_leuv2di,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_any_lev4sf,             vec_any_lev4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_any_lev2df,             vec_any_lev2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_any_ltv16qi,            vec_any_ltv16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_V16QI_V16QI)
B_DEF      (vec_any_ltuv16qi,           vec_any_ltuv16qi,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV16QI_UV16QI)
B_DEF      (vec_any_ltv8hi,             vec_any_ltv8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V8HI_V8HI)
B_DEF      (vec_any_ltuv8hi,            vec_any_ltuv8hi,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV8HI_UV8HI)
B_DEF      (vec_any_ltv4si,             vec_any_ltv4si,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V4SI_V4SI)
B_DEF      (vec_any_ltuv4si,            vec_any_ltuv4si,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV4SI_UV4SI)
B_DEF      (vec_any_ltv2di,             vec_any_ltv2di,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DI_V2DI)
B_DEF      (vec_any_ltuv2di,            vec_any_ltuv2di,    0,                  B_INT | B_VX,       0,                  BT_FN_INT_UV2DI_UV2DI)
B_DEF      (vec_any_ltv4sf,             vec_any_ltv4sf,     0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_any_ltv2df,             vec_any_ltv2df,     0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

OB_DEF     (s390_vec_any_eq,            s390_vec_any_eq_s8_a,s390_vec_any_eq_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_any_eq_s8_a,       vec_any_eqv16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_any_eq_s8_b,       vec_any_eqv16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_eq_b8_a,       vec_any_eqv16qi,    0,                  0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_eq_b8_b,       vec_any_eqv16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_any_eq_b8_c,       vec_any_eqv16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_eq_u8_a,       vec_any_eqv16qi,    0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_eq_u8_b,       vec_any_eqv16qi,    B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_eq_s16_a,      vec_any_eqv8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_any_eq_s16_b,      vec_any_eqv8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_eq_b16_a,      vec_any_eqv8hi,     0,                  0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_eq_b16_b,      vec_any_eqv8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_any_eq_b16_c,      vec_any_eqv8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_eq_u16_a,      vec_any_eqv8hi,     0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_eq_u16_b,      vec_any_eqv8hi,     B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_eq_s32_a,      vec_any_eqv4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_any_eq_s32_b,      vec_any_eqv4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_eq_b32_a,      vec_any_eqv4si,     0,                  0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_eq_b32_b,      vec_any_eqv4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_any_eq_b32_c,      vec_any_eqv4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_eq_u32_a,      vec_any_eqv4si,     0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_eq_u32_b,      vec_any_eqv4si,     B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_eq_s64_a,      vec_any_eqv2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_any_eq_s64_b,      vec_any_eqv2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_eq_b64_a,      vec_any_eqv2di,     0,                  0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_eq_b64_b,      vec_any_eqv2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_any_eq_b64_c,      vec_any_eqv2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_eq_u64_a,      vec_any_eqv2di,     0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_eq_u64_b,      vec_any_eqv2di,     B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_eq_flt,        vec_any_eqv4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_any_eq_dbl,        vec_any_eqv2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_any_ne,            s390_vec_any_ne_s8_a,s390_vec_any_ne_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_any_ne_s8_a,       vec_any_nev16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_any_ne_s8_b,       vec_any_nev16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_ne_b8_a,       vec_any_nev16qi,    0,                  0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_ne_b8_b,       vec_any_nev16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_any_ne_b8_c,       vec_any_nev16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_ne_u8_a,       vec_any_nev16qi,    0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_ne_u8_b,       vec_any_nev16qi,    B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_ne_s16_a,      vec_any_nev8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_any_ne_s16_b,      vec_any_nev8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_ne_b16_a,      vec_any_nev8hi,     0,                  0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_ne_b16_b,      vec_any_nev8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_any_ne_b16_c,      vec_any_nev8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_ne_u16_a,      vec_any_nev8hi,     0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_ne_u16_b,      vec_any_nev8hi,     B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_ne_s32_a,      vec_any_nev4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_any_ne_s32_b,      vec_any_nev4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_ne_b32_a,      vec_any_nev4si,     0,                  0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_ne_b32_b,      vec_any_nev4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_any_ne_b32_c,      vec_any_nev4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_ne_u32_a,      vec_any_nev4si,     0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_ne_u32_b,      vec_any_nev4si,     B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_ne_s64_a,      vec_any_nev2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_any_ne_s64_b,      vec_any_nev2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_ne_b64_a,      vec_any_nev2di,     0,                  0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_ne_b64_b,      vec_any_nev2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_any_ne_b64_c,      vec_any_nev2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_ne_u64_a,      vec_any_nev2di,     0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_ne_u64_b,      vec_any_nev2di,     B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_ne_flt,        vec_any_nev4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_any_ne_dbl,        vec_any_nev2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_any_ge,            s390_vec_any_ge_s8_a,s390_vec_any_ge_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_any_ge_s8_a,       vec_any_gev16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_any_ge_s8_b,       vec_any_gev16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_ge_b8_a,       vec_any_geuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_ge_b8_b,       vec_any_gev16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_any_ge_b8_c,       vec_any_geuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_ge_u8_a,       vec_any_geuv16qi,   0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_ge_u8_b,       vec_any_geuv16qi,   B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_ge_s16_a,      vec_any_gev8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_any_ge_s16_b,      vec_any_gev8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_ge_b16_a,      vec_any_geuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_ge_b16_b,      vec_any_gev8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_any_ge_b16_c,      vec_any_geuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_ge_u16_a,      vec_any_geuv8hi,    0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_ge_u16_b,      vec_any_geuv8hi,    B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_ge_s32_a,      vec_any_gev4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_any_ge_s32_b,      vec_any_gev4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_ge_b32_a,      vec_any_geuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_ge_b32_b,      vec_any_gev4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_any_ge_b32_c,      vec_any_geuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_ge_u32_a,      vec_any_geuv4si,    0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_ge_u32_b,      vec_any_geuv4si,    B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_ge_s64_a,      vec_any_gev2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_any_ge_s64_b,      vec_any_gev2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_ge_b64_a,      vec_any_geuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_ge_b64_b,      vec_any_gev2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_any_ge_b64_c,      vec_any_geuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_ge_u64_a,      vec_any_geuv2di,    0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_ge_u64_b,      vec_any_geuv2di,    B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_ge_flt,        vec_any_gev4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_any_ge_dbl,        vec_any_gev2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_any_gt,            s390_vec_any_gt_s8_a,s390_vec_any_gt_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_any_gt_s8_a,       vec_any_gtv16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_any_gt_s8_b,       vec_any_gtv16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_gt_b8_a,       vec_any_gtuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_gt_b8_b,       vec_any_gtv16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_any_gt_b8_c,       vec_any_gtuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_gt_u8_a,       vec_any_gtuv16qi,   0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_gt_u8_b,       vec_any_gtuv16qi,   B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_gt_s16_a,      vec_any_gtv8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_any_gt_s16_b,      vec_any_gtv8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_gt_b16_a,      vec_any_gtuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_gt_b16_b,      vec_any_gtv8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_any_gt_b16_c,      vec_any_gtuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_gt_u16_a,      vec_any_gtuv8hi,    0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_gt_u16_b,      vec_any_gtuv8hi,    B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_gt_s32_a,      vec_any_gtv4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_any_gt_s32_b,      vec_any_gtv4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_gt_b32_a,      vec_any_gtuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_gt_b32_b,      vec_any_gtv4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_any_gt_b32_c,      vec_any_gtuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_gt_u32_a,      vec_any_gtuv4si,    0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_gt_u32_b,      vec_any_gtuv4si,    B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_gt_s64_a,      vec_any_gtv2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_any_gt_s64_b,      vec_any_gtv2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_gt_b64_a,      vec_any_gtuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_gt_b64_b,      vec_any_gtv2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_any_gt_b64_c,      vec_any_gtuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_gt_u64_a,      vec_any_gtuv2di,    0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_gt_u64_b,      vec_any_gtuv2di,    B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_gt_flt,        vec_any_gtv4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_any_gt_dbl,        vec_any_gtv2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_any_le,            s390_vec_any_le_s8_a,s390_vec_any_le_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_any_le_s8_a,       vec_any_lev16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_any_le_s8_b,       vec_any_lev16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_le_b8_a,       vec_any_leuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_le_b8_b,       vec_any_lev16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_any_le_b8_c,       vec_any_leuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_le_u8_a,       vec_any_leuv16qi,   0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_le_u8_b,       vec_any_leuv16qi,   B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_le_s16_a,      vec_any_lev8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_any_le_s16_b,      vec_any_lev8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_le_b16_a,      vec_any_leuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_le_b16_b,      vec_any_lev8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_any_le_b16_c,      vec_any_leuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_le_u16_a,      vec_any_leuv8hi,    0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_le_u16_b,      vec_any_leuv8hi,    B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_le_s32_a,      vec_any_lev4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_any_le_s32_b,      vec_any_lev4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_le_b32_a,      vec_any_leuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_le_b32_b,      vec_any_lev4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_any_le_b32_c,      vec_any_leuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_le_u32_a,      vec_any_leuv4si,    0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_le_u32_b,      vec_any_leuv4si,    B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_le_s64_a,      vec_any_lev2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_any_le_s64_b,      vec_any_lev2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_le_b64_a,      vec_any_leuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_le_b64_b,      vec_any_lev2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_any_le_b64_c,      vec_any_leuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_le_u64_a,      vec_any_leuv2di,    0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_le_u64_b,      vec_any_leuv2di,    B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_le_flt,        vec_any_lev4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_any_le_dbl,        vec_any_lev2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_any_lt,            s390_vec_any_lt_s8_a,s390_vec_any_lt_dbl,B_VX,              BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_any_lt_s8_a,       vec_any_ltv16qi,    0,                  0,                  BT_OV_INT_V16QI_V16QI)
OB_DEF_VAR (s390_vec_any_lt_s8_b,       vec_any_ltv16qi,    B_DEP,              0,                  BT_OV_INT_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_lt_b8_a,       vec_any_ltuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_lt_b8_b,       vec_any_ltv16qi,    B_DEP,              0,                  BT_OV_INT_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_any_lt_b8_c,       vec_any_ltuv16qi,   B_DEP,              0,                  BT_OV_INT_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_lt_u8_a,       vec_any_ltuv16qi,   0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_any_lt_u8_b,       vec_any_ltuv16qi,   B_DEP,              0,                  BT_OV_INT_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_any_lt_s16_a,      vec_any_ltv8hi,     0,                  0,                  BT_OV_INT_V8HI_V8HI)
OB_DEF_VAR (s390_vec_any_lt_s16_b,      vec_any_ltv8hi,     B_DEP,              0,                  BT_OV_INT_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_lt_b16_a,      vec_any_ltuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_lt_b16_b,      vec_any_ltv8hi,     B_DEP,              0,                  BT_OV_INT_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_any_lt_b16_c,      vec_any_ltuv8hi,    B_DEP,              0,                  BT_OV_INT_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_lt_u16_a,      vec_any_ltuv8hi,    0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_any_lt_u16_b,      vec_any_ltuv8hi,    B_DEP,              0,                  BT_OV_INT_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_any_lt_s32_a,      vec_any_ltv4si,     0,                  0,                  BT_OV_INT_V4SI_V4SI)
OB_DEF_VAR (s390_vec_any_lt_s32_b,      vec_any_ltv4si,     B_DEP,              0,                  BT_OV_INT_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_lt_b32_a,      vec_any_ltuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_lt_b32_b,      vec_any_ltv4si,     B_DEP,              0,                  BT_OV_INT_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_any_lt_b32_c,      vec_any_ltuv4si,    B_DEP,              0,                  BT_OV_INT_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_lt_u32_a,      vec_any_ltuv4si,    0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_any_lt_u32_b,      vec_any_ltuv4si,    B_DEP,              0,                  BT_OV_INT_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_any_lt_s64_a,      vec_any_ltv2di,     0,                  0,                  BT_OV_INT_V2DI_V2DI)
OB_DEF_VAR (s390_vec_any_lt_s64_b,      vec_any_ltv2di,     B_DEP,              0,                  BT_OV_INT_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_lt_b64_a,      vec_any_ltuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_lt_b64_b,      vec_any_ltv2di,     B_DEP,              0,                  BT_OV_INT_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_any_lt_b64_c,      vec_any_ltuv2di,    B_DEP,              0,                  BT_OV_INT_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_lt_u64_a,      vec_any_ltuv2di,    0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_any_lt_u64_b,      vec_any_ltuv2di,    B_DEP,              0,                  BT_OV_INT_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_any_lt_flt,        vec_any_ltv4sf,     B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_any_lt_dbl,        vec_any_ltv2df,     0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_cmpeq,             s390_vec_cmpeq_s8,  s390_vec_cmpeq_dbl, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_cmpeq_s8,          s390_vceqb,         0,                  0,                  BT_OV_BV16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_cmpeq_u8,          s390_vceqb,         0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_cmpeq_b8,          s390_vceqb,         0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_cmpeq_s16,         s390_vceqh,         0,                  0,                  BT_OV_BV8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_cmpeq_u16,         s390_vceqh,         0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_cmpeq_b16,         s390_vceqh,         0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_cmpeq_s32,         s390_vceqf,         0,                  0,                  BT_OV_BV4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_cmpeq_u32,         s390_vceqf,         0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_cmpeq_b32,         s390_vceqf,         0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_cmpeq_s64,         s390_vceqg,         0,                  0,                  BT_OV_BV2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_cmpeq_u64,         s390_vceqg,         0,                  0,                  BT_OV_BV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_cmpeq_b64,         s390_vceqg,         0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_cmpeq_flt,         s390_vfcesb,        B_VXE,              0,                  BT_OV_BV4SI_V4SF_V4SF)
OB_DEF_VAR (s390_vec_cmpeq_dbl,         s390_vfcedb,        0,                  0,                  BT_OV_BV2DI_V2DF_V2DF)

B_DEF      (s390_vceqb,                 vec_cmpeqv16qi,     0,                  B_VX,               0,                  BT_FN_V16QI_UV16QI_UV16QI)
B_DEF      (s390_vceqh,                 vec_cmpeqv8hi,      0,                  B_VX,               0,                  BT_FN_V8HI_UV8HI_UV8HI)
B_DEF      (s390_vceqf,                 vec_cmpeqv4si,      0,                  B_VX,               0,                  BT_FN_V4SI_UV4SI_UV4SI)
B_DEF      (s390_vceqg,                 vec_cmpeqv2di,      0,                  B_VX,               0,                  BT_FN_V2DI_UV2DI_UV2DI)
B_DEF      (s390_vfcesb,                vec_cmpeqv4sf,      0,                  B_VXE,              0,                  BT_FN_V4SI_V4SF_V4SF)
B_DEF      (s390_vfcedb,                vec_cmpeqv2df,      0,                  B_VX,               0,                  BT_FN_V2DI_V2DF_V2DF)

OB_DEF     (s390_vec_cmpge,             s390_vec_cmpge_s8,  s390_vec_cmpge_dbl, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_cmpge_s8,          vec_cmpgev16qi,     0,                  0,                  BT_OV_BV16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_cmpge_u8,          vec_cmpgeuv16qi,    0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_cmpge_s16,         vec_cmpgev8hi,      0,                  0,                  BT_OV_BV8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_cmpge_u16,         vec_cmpgeuv8hi,     0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_cmpge_s32,         vec_cmpgev4si,      0,                  0,                  BT_OV_BV4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_cmpge_u32,         vec_cmpgeuv4si,     0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_cmpge_s64,         vec_cmpgev2di,      0,                  0,                  BT_OV_BV2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_cmpge_u64,         vec_cmpgeuv2di,     0,                  0,                  BT_OV_BV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_cmpge_flt,         s390_vfchesb,       B_VXE,              0,                  BT_OV_BV4SI_V4SF_V4SF)
OB_DEF_VAR (s390_vec_cmpge_dbl,         s390_vfchedb,       0,                  0,                  BT_OV_BV2DI_V2DF_V2DF)

B_DEF      (vec_cmpgev16qi,             vec_cmpgev16qi,     0,                  B_INT | B_VX,       0,                  BT_FN_V16QI_UV16QI_UV16QI)
B_DEF      (vec_cmpgeuv16qi,            vec_cmpgeuv16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_V16QI_UV16QI_UV16QI)
B_DEF      (vec_cmpgev8hi,              vec_cmpgev8hi,      0,                  B_INT | B_VX,       0,                  BT_FN_V8HI_UV8HI_UV8HI)
B_DEF      (vec_cmpgeuv8hi,             vec_cmpgeuv8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_V8HI_UV8HI_UV8HI)
B_DEF      (vec_cmpgev4si,              vec_cmpgev4si,      0,                  B_INT | B_VX,       0,                  BT_FN_V4SI_UV4SI_UV4SI)
B_DEF      (vec_cmpgeuv4si,             vec_cmpgeuv4si,     0,                  B_INT | B_VX,       0,                  BT_FN_V4SI_UV4SI_UV4SI)
B_DEF      (vec_cmpgev2di,              vec_cmpgev2di,      0,                  B_INT | B_VX,       0,                  BT_FN_V2DI_UV2DI_UV2DI)
B_DEF      (vec_cmpgeuv2di,             vec_cmpgeuv2di,     0,                  B_INT | B_VX,       0,                  BT_FN_V2DI_UV2DI_UV2DI)
B_DEF      (s390_vfchesb,               vec_cmpgev4sf,      0,                  B_VXE,              0,                  BT_FN_V4SI_V4SF_V4SF)
B_DEF      (s390_vfchedb,               vec_cmpgev2df,      0,                  B_VX,               0,                  BT_FN_V2DI_V2DF_V2DF)

OB_DEF     (s390_vec_cmpgt,             s390_vec_cmpgt_s8,  s390_vec_cmpgt_dbl, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_cmpgt_s8,          s390_vchb,          0,                  0,                  BT_OV_BV16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_cmpgt_u8,          s390_vchlb,         0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_cmpgt_s16,         s390_vchh,          0,                  0,                  BT_OV_BV8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_cmpgt_u16,         s390_vchlh,         0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_cmpgt_s32,         s390_vchf,          0,                  0,                  BT_OV_BV4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_cmpgt_u32,         s390_vchlf,         0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_cmpgt_s64,         s390_vchg,          0,                  0,                  BT_OV_BV2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_cmpgt_u64,         s390_vchlg,         0,                  0,                  BT_OV_BV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_cmpgt_flt,         s390_vfchsb,        B_VXE,              0,                  BT_OV_BV4SI_V4SF_V4SF)
OB_DEF_VAR (s390_vec_cmpgt_dbl,         s390_vfchdb,        0,                  0,                  BT_OV_BV2DI_V2DF_V2DF)

B_DEF      (s390_vchb,                  vec_cmpgtv16qi,     0,                  B_VX,               0,                  BT_FN_V16QI_V16QI_V16QI)
B_DEF      (s390_vchlb,                 vec_cmpgtuv16qi,    0,                  B_VX,               0,                  BT_FN_V16QI_UV16QI_UV16QI)
B_DEF      (s390_vchh,                  vec_cmpgtv8hi,      0,                  B_VX,               0,                  BT_FN_V8HI_V8HI_V8HI)
B_DEF      (s390_vchlh,                 vec_cmpgtuv8hi,     0,                  B_VX,               0,                  BT_FN_V8HI_UV8HI_UV8HI)
B_DEF      (s390_vchf,                  vec_cmpgtv4si,      0,                  B_VX,               0,                  BT_FN_V4SI_V4SI_V4SI)
B_DEF      (s390_vchlf,                 vec_cmpgtuv4si,     0,                  B_VX,               0,                  BT_FN_V4SI_UV4SI_UV4SI)
B_DEF      (s390_vchg,                  vec_cmpgtv2di,      0,                  B_VX,               0,                  BT_FN_V2DI_V2DI_V2DI)
B_DEF      (s390_vchlg,                 vec_cmpgtuv2di,     0,                  B_VX,               0,                  BT_FN_V2DI_UV2DI_UV2DI)
B_DEF      (s390_vfchsb,                vec_cmpgtv4sf,      0,                  B_VXE,              0,                  BT_FN_V4SI_V4SF_V4SF)
B_DEF      (s390_vfchdb,                vec_cmpgtv2df,      0,                  B_VX,               0,                  BT_FN_V2DI_V2DF_V2DF)

OB_DEF     (s390_vec_cmple,             s390_vec_cmple_s8,  s390_vec_cmple_dbl, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_cmple_s8,          vec_cmplev16qi,     0,                  0,                  BT_OV_BV16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_cmple_u8,          vec_cmpleuv16qi,    0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_cmple_s16,         vec_cmplev8hi,      0,                  0,                  BT_OV_BV8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_cmple_u16,         vec_cmpleuv8hi,     0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_cmple_s32,         vec_cmplev4si,      0,                  0,                  BT_OV_BV4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_cmple_u32,         vec_cmpleuv4si,     0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_cmple_s64,         vec_cmplev2di,      0,                  0,                  BT_OV_BV2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_cmple_u64,         vec_cmpleuv2di,     0,                  0,                  BT_OV_BV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_cmple_flt,         vec_cmplev4sf,      B_VXE,              0,                  BT_OV_BV4SI_V4SF_V4SF)
OB_DEF_VAR (s390_vec_cmple_dbl,         vec_cmplev2df,      0,                  0,                  BT_OV_BV2DI_V2DF_V2DF)

B_DEF      (vec_cmplev16qi,             vec_cmplev16qi,     0,                  B_INT | B_VX,       0,                  BT_FN_V16QI_UV16QI_UV16QI)
B_DEF      (vec_cmpleuv16qi,            vec_cmpleuv16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_V16QI_UV16QI_UV16QI)
B_DEF      (vec_cmplev8hi,              vec_cmplev8hi,      0,                  B_INT | B_VX,       0,                  BT_FN_V8HI_UV8HI_UV8HI)
B_DEF      (vec_cmpleuv8hi,             vec_cmpleuv8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_V8HI_UV8HI_UV8HI)
B_DEF      (vec_cmplev4si,              vec_cmplev4si,      0,                  B_INT | B_VX,       0,                  BT_FN_V4SI_UV4SI_UV4SI)
B_DEF      (vec_cmpleuv4si,             vec_cmpleuv4si,     0,                  B_INT | B_VX,       0,                  BT_FN_V4SI_UV4SI_UV4SI)
B_DEF      (vec_cmplev2di,              vec_cmplev2di,      0,                  B_INT | B_VX,       0,                  BT_FN_V2DI_UV2DI_UV2DI)
B_DEF      (vec_cmpleuv2di,             vec_cmpleuv2di,     0,                  B_INT | B_VX,       0,                  BT_FN_V2DI_UV2DI_UV2DI)
B_DEF      (vec_cmplev4sf,              vec_cmplev4sf,      0,                  B_INT | B_VXE,      0,                  BT_FN_V4SI_V4SF_V4SF)
B_DEF      (vec_cmplev2df,              vec_cmplev2df,      0,                  B_INT | B_VX,       0,                  BT_FN_V2DI_V2DF_V2DF)

OB_DEF     (s390_vec_cmplt,             s390_vec_cmplt_s8,  s390_vec_cmplt_dbl, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_cmplt_s8,          vec_cmpltv16qi,     0,                  0,                  BT_OV_BV16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_cmplt_u8,          vec_cmpltuv16qi,    0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_cmplt_s16,         vec_cmpltv8hi,      0,                  0,                  BT_OV_BV8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_cmplt_u16,         vec_cmpltuv8hi,     0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_cmplt_s32,         vec_cmpltv4si,      0,                  0,                  BT_OV_BV4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_cmplt_u32,         vec_cmpltuv4si,     0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_cmplt_s64,         vec_cmpltv2di,      0,                  0,                  BT_OV_BV2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_cmplt_u64,         vec_cmpltuv2di,     0,                  0,                  BT_OV_BV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_cmplt_flt,         vec_cmpltv4sf,      B_VXE,              0,                  BT_OV_BV4SI_V4SF_V4SF)
OB_DEF_VAR (s390_vec_cmplt_dbl,         vec_cmpltv2df,      0,                  0,                  BT_OV_BV2DI_V2DF_V2DF)

B_DEF      (vec_cmpltv16qi,             vec_cmpltv16qi,     0,                  B_INT | B_VX,       0,                  BT_FN_V16QI_UV16QI_UV16QI)
B_DEF      (vec_cmpltuv16qi,            vec_cmpltuv16qi,    0,                  B_INT | B_VX,       0,                  BT_FN_V16QI_UV16QI_UV16QI)
B_DEF      (vec_cmpltv8hi,              vec_cmpltv8hi,      0,                  B_INT | B_VX,       0,                  BT_FN_V8HI_UV8HI_UV8HI)
B_DEF      (vec_cmpltuv8hi,             vec_cmpltuv8hi,     0,                  B_INT | B_VX,       0,                  BT_FN_V8HI_UV8HI_UV8HI)
B_DEF      (vec_cmpltv4si,              vec_cmpltv4si,      0,                  B_INT | B_VX,       0,                  BT_FN_V4SI_UV4SI_UV4SI)
B_DEF      (vec_cmpltuv4si,             vec_cmpltuv4si,     0,                  B_INT | B_VX,       0,                  BT_FN_V4SI_UV4SI_UV4SI)
B_DEF      (vec_cmpltv2di,              vec_cmpltv2di,      0,                  B_INT | B_VX,       0,                  BT_FN_V2DI_UV2DI_UV2DI)
B_DEF      (vec_cmpltuv2di,             vec_cmpltuv2di,     0,                  B_INT | B_VX,       0,                  BT_FN_V2DI_UV2DI_UV2DI)
B_DEF      (vec_cmpltv4sf,              vec_cmpltv4sf,      0,                  B_INT | B_VXE,      0,                  BT_FN_V4SI_V4SF_V4SF)
B_DEF      (vec_cmpltv2df,              vec_cmpltv2df,      0,                  B_INT | B_VX,       0,                  BT_FN_V2DI_V2DF_V2DF)

OB_DEF     (s390_vec_cntlz,             s390_vec_cntlz_s8,  s390_vec_cntlz_u64, B_VX,               BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_cntlz_s8,          s390_vclzb,         0,                  0,                  BT_OV_UV16QI_V16QI)
OB_DEF_VAR (s390_vec_cntlz_u8,          s390_vclzb,         0,                  0,                  BT_OV_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_cntlz_s16,         s390_vclzh,         0,                  0,                  BT_OV_UV8HI_V8HI)
OB_DEF_VAR (s390_vec_cntlz_u16,         s390_vclzh,         0,                  0,                  BT_OV_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_cntlz_s32,         s390_vclzf,         0,                  0,                  BT_OV_UV4SI_V4SI)
OB_DEF_VAR (s390_vec_cntlz_u32,         s390_vclzf,         0,                  0,                  BT_OV_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_cntlz_s64,         s390_vclzg,         0,                  0,                  BT_OV_UV2DI_V2DI)
OB_DEF_VAR (s390_vec_cntlz_u64,         s390_vclzg,         0,                  0,                  BT_OV_UV2DI_UV2DI)

B_DEF      (s390_vclzb,                 clzv16qi2,          0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI)
B_DEF      (s390_vclzh,                 clzv8hi2,           0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI)
B_DEF      (s390_vclzf,                 clzv4si2,           0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI)
B_DEF      (s390_vclzg,                 clzv2di2,           0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI)

OB_DEF     (s390_vec_cnttz,             s390_vec_cnttz_s8,  s390_vec_cnttz_u64, B_VX,               BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_cnttz_s8,          s390_vctzb,         0,                  0,                  BT_OV_UV16QI_V16QI)
OB_DEF_VAR (s390_vec_cnttz_u8,          s390_vctzb,         0,                  0,                  BT_OV_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_cnttz_s16,         s390_vctzh,         0,                  0,                  BT_OV_UV8HI_V8HI)
OB_DEF_VAR (s390_vec_cnttz_u16,         s390_vctzh,         0,                  0,                  BT_OV_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_cnttz_s32,         s390_vctzf,         0,                  0,                  BT_OV_UV4SI_V4SI)
OB_DEF_VAR (s390_vec_cnttz_u32,         s390_vctzf,         0,                  0,                  BT_OV_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_cnttz_s64,         s390_vctzg,         0,                  0,                  BT_OV_UV2DI_V2DI)
OB_DEF_VAR (s390_vec_cnttz_u64,         s390_vctzg,         0,                  0,                  BT_OV_UV2DI_UV2DI)

B_DEF      (s390_vctzb,                 ctzv16qi2,          0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI)
B_DEF      (s390_vctzh,                 ctzv8hi2,           0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI)
B_DEF      (s390_vctzf,                 ctzv4si2,           0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI)
B_DEF      (s390_vctzg,                 ctzv2di2,           0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI)

OB_DEF     (s390_vec_xor,               s390_vec_xor_b8,    s390_vec_xor_dbl_c, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_xor_b8,            s390_vx,            0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_xor_s8_a,          s390_vx,            B_DEP,              0,                  BT_OV_V16QI_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_xor_s8_b,          s390_vx,            0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_xor_s8_c,          s390_vx,            B_DEP,              0,                  BT_OV_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_xor_u8_a,          s390_vx,            B_DEP,              0,                  BT_OV_UV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_xor_u8_b,          s390_vx,            0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_xor_u8_c,          s390_vx,            B_DEP,              0,                  BT_OV_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_xor_b16,           s390_vx,            0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_xor_s16_a,         s390_vx,            B_DEP,              0,                  BT_OV_V8HI_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_xor_s16_b,         s390_vx,            0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_xor_s16_c,         s390_vx,            B_DEP,              0,                  BT_OV_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_xor_u16_a,         s390_vx,            B_DEP,              0,                  BT_OV_UV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_xor_u16_b,         s390_vx,            0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_xor_u16_c,         s390_vx,            B_DEP,              0,                  BT_OV_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_xor_b32,           s390_vx,            0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_xor_s32_a,         s390_vx,            B_DEP,              0,                  BT_OV_V4SI_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_xor_s32_b,         s390_vx,            0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_xor_s32_c,         s390_vx,            B_DEP,              0,                  BT_OV_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_xor_u32_a,         s390_vx,            B_DEP,              0,                  BT_OV_UV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_xor_u32_b,         s390_vx,            0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_xor_u32_c,         s390_vx,            B_DEP,              0,                  BT_OV_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_xor_b64,           s390_vx,            0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_xor_s64_a,         s390_vx,            B_DEP,              0,                  BT_OV_V2DI_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_xor_s64_b,         s390_vx,            0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_xor_s64_c,         s390_vx,            B_DEP,              0,                  BT_OV_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_xor_u64_a,         s390_vx,            B_DEP,              0,                  BT_OV_UV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_xor_u64_b,         s390_vx,            0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_xor_u64_c,         s390_vx,            B_DEP,              0,                  BT_OV_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_xor_dbl_a,         s390_vx,            B_DEP,              0,                  BT_OV_V2DF_BV2DI_V2DF)
OB_DEF_VAR (s390_vec_xor_dbl_b,         s390_vx,            0,                  0,                  BT_OV_V2DF_V2DF_V2DF)
OB_DEF_VAR (s390_vec_xor_dbl_c,         s390_vx,            B_DEP,              0,                  BT_OV_V2DF_V2DF_BV2DI)

B_DEF      (s390_vx,                    xorv16qi3,          0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_gfmsum,            s390_vec_gfmsum_u8, s390_vec_gfmsum_u32,B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_gfmsum_u8,         s390_vgfmb,         0,                  0,                  BT_OV_UV8HI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_gfmsum_u16,        s390_vgfmh,         0,                  0,                  BT_OV_UV4SI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_gfmsum_u32,        s390_vgfmf,         0,                  0,                  BT_OV_UV2DI_UV4SI_UV4SI)

B_DEF      (s390_vgfmb,                 vec_gfmsumv16qi,    0,                  B_VX,               0,                  BT_FN_UV8HI_UV16QI_UV16QI)
B_DEF      (s390_vgfmh,                 vec_gfmsumv8hi,     0,                  B_VX,               0,                  BT_FN_UV4SI_UV8HI_UV8HI)
B_DEF      (s390_vgfmf,                 vec_gfmsumv4si,     0,                  B_VX,               0,                  BT_FN_UV2DI_UV4SI_UV4SI)
B_DEF      (s390_vgfmg,                 vec_gfmsum_128,     0,                  B_VX,               0,                  BT_FN_UV16QI_UV2DI_UV2DI)

OB_DEF     (s390_vec_gfmsum_accum,      s390_vec_gfmsum_accum_u8,s390_vec_gfmsum_accum_u32,B_VX,    BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_gfmsum_accum_u8,   s390_vgfmab,        0,                  0,                  BT_OV_UV8HI_UV16QI_UV16QI_UV8HI)
OB_DEF_VAR (s390_vec_gfmsum_accum_u16,  s390_vgfmah,        0,                  0,                  BT_OV_UV4SI_UV8HI_UV8HI_UV4SI)
OB_DEF_VAR (s390_vec_gfmsum_accum_u32,  s390_vgfmaf,        0,                  0,                  BT_OV_UV2DI_UV4SI_UV4SI_UV2DI)

B_DEF      (s390_vgfmab,                vec_gfmsum_accumv16qi,0,                B_VX,               0,                  BT_FN_UV8HI_UV16QI_UV16QI_UV8HI)
B_DEF      (s390_vgfmah,                vec_gfmsum_accumv8hi,0,                 B_VX,               0,                  BT_FN_UV4SI_UV8HI_UV8HI_UV4SI)
B_DEF      (s390_vgfmaf,                vec_gfmsum_accumv4si,0,                 B_VX,               0,                  BT_FN_UV2DI_UV4SI_UV4SI_UV2DI)
B_DEF      (s390_vgfmag,                vec_gfmsum_accum_128,0,                 B_VX,               0,                  BT_FN_UV16QI_UV2DI_UV2DI_UV16QI)

OB_DEF     (s390_vec_abs,               s390_vec_abs_s8,    s390_vec_abs_dbl,   B_VX,               BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_abs_s8,            s390_vlpb,          0,                  0,                  BT_OV_V16QI_V16QI)
OB_DEF_VAR (s390_vec_abs_s16,           s390_vlph,          0,                  0,                  BT_OV_V8HI_V8HI)
OB_DEF_VAR (s390_vec_abs_s32,           s390_vlpf,          0,                  0,                  BT_OV_V4SI_V4SI)
OB_DEF_VAR (s390_vec_abs_s64,           s390_vlpg,          0,                  0,                  BT_OV_V2DI_V2DI)
OB_DEF_VAR (s390_vec_abs_flt,           s390_vflpsb,        B_VXE,              0,                  BT_OV_V4SF_V4SF)
OB_DEF_VAR (s390_vec_abs_dbl,           s390_vflpdb,        0,                  0,                  BT_OV_V2DF_V2DF)

B_DEF      (s390_vlpb,                  absv16qi2,          0,                  B_VX,               0,                  BT_FN_V16QI_V16QI)
B_DEF      (s390_vlph,                  absv8hi2,           0,                  B_VX,               0,                  BT_FN_V8HI_V8HI)
B_DEF      (s390_vlpf,                  absv4si2,           0,                  B_VX,               0,                  BT_FN_V4SI_V4SI)
B_DEF      (s390_vlpg,                  absv2di2,           0,                  B_VX,               0,                  BT_FN_V2DI_V2DI)
B_DEF      (s390_vflpsb,                absv4sf2,           0,                  B_VXE,              0,                  BT_FN_V4SF_V4SF)
B_DEF      (s390_vflpdb,                absv2df2,           0,                  B_VX,               0,                  BT_FN_V2DF_V2DF)

OB_DEF     (s390_vec_max,               s390_vec_max_s8_a,  s390_vec_max_dbl,   B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_max_s8_a,          s390_vmxb,          B_DEP,              0,                  BT_OV_V16QI_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_max_s8_b,          s390_vmxb,          0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_max_s8_c,          s390_vmxb,          B_DEP,              0,                  BT_OV_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_max_u8_a,          s390_vmxlb,         B_DEP,              0,                  BT_OV_UV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_max_u8_b,          s390_vmxlb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_max_u8_c,          s390_vmxlb,         B_DEP,              0,                  BT_OV_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_max_s16_a,         s390_vmxh,          B_DEP,              0,                  BT_OV_V8HI_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_max_s16_b,         s390_vmxh,          0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_max_s16_c,         s390_vmxh,          B_DEP,              0,                  BT_OV_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_max_u16_a,         s390_vmxlh,         B_DEP,              0,                  BT_OV_UV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_max_u16_b,         s390_vmxlh,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_max_u16_c,         s390_vmxlh,         B_DEP,              0,                  BT_OV_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_max_s32_a,         s390_vmxf,          B_DEP,              0,                  BT_OV_V4SI_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_max_s32_b,         s390_vmxf,          0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_max_s32_c,         s390_vmxf,          B_DEP,              0,                  BT_OV_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_max_u32_a,         s390_vmxlf,         B_DEP,              0,                  BT_OV_UV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_max_u32_b,         s390_vmxlf,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_max_u32_c,         s390_vmxlf,         B_DEP,              0,                  BT_OV_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_max_s64_a,         s390_vmxg,          B_DEP,              0,                  BT_OV_V2DI_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_max_s64_b,         s390_vmxg,          0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_max_s64_c,         s390_vmxg,          B_DEP,              0,                  BT_OV_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_max_u64_a,         s390_vmxlg,         B_DEP,              0,                  BT_OV_UV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_max_u64_b,         s390_vmxlg,         0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_max_u64_c,         s390_vmxlg,         B_DEP,              0,                  BT_OV_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_max_flt,           s390_vfmaxsb_4,     B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_max_dbl,           s390_vfmaxdb_4,     0,                  0,                  BT_OV_V2DF_V2DF_V2DF)

B_DEF      (s390_vmxb,                  smaxv16qi3,         0,                  B_VX,               0,                  BT_FN_V16QI_BV16QI_V16QI)
B_DEF      (s390_vmxlb,                 umaxv16qi3,         0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vmxh,                  smaxv8hi3,          0,                  B_VX,               0,                  BT_FN_V8HI_BV8HI_V8HI)
B_DEF      (s390_vmxlh,                 umaxv8hi3,          0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vmxf,                  smaxv4si3,          0,                  B_VX,               0,                  BT_FN_V4SI_BV4SI_V4SI)
B_DEF      (s390_vmxlf,                 umaxv4si3,          0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vmxg,                  smaxv2di3,          0,                  B_VX,               0,                  BT_FN_V2DI_BV2DI_V2DI)
B_DEF      (s390_vmxlg,                 umaxv2di3,          0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI_UV2DI)
B_DEF      (s390_vfmaxsb,               vfmaxv4sf,          0,                  B_VXE,              O3_U4,              BT_FN_V4SF_V4SF_V4SF_INT)
B_DEF      (s390_vfmaxdb,               vfmaxv2df,          0,                  B_VXE,              O3_U4,              BT_FN_V2DF_V2DF_V2DF_INT)
B_DEF      (s390_vfmaxsb_4,             smaxv4sf3,          0,                  B_INT | B_VXE,      0,                  BT_FN_V4SF_V4SF_V4SF)
B_DEF      (s390_vfmaxdb_4,             smaxv2df3,          0,                  B_INT | B_VXE,      0,                  BT_FN_V2DF_V2DF_V2DF)

OB_DEF     (s390_vec_min,               s390_vec_min_s8_a,  s390_vec_min_dbl,   B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_min_s8_a,          s390_vmnb,          B_DEP,              0,                  BT_OV_V16QI_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_min_s8_b,          s390_vmnb,          0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_min_s8_c,          s390_vmnb,          B_DEP,              0,                  BT_OV_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_min_u8_a,          s390_vmnlb,         B_DEP,              0,                  BT_OV_UV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_min_u8_b,          s390_vmnlb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_min_u8_c,          s390_vmnlb,         B_DEP,              0,                  BT_OV_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_min_s16_a,         s390_vmnh,          B_DEP,              0,                  BT_OV_V8HI_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_min_s16_b,         s390_vmnh,          0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_min_s16_c,         s390_vmnh,          B_DEP,              0,                  BT_OV_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_min_u16_a,         s390_vmnlh,         B_DEP,              0,                  BT_OV_UV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_min_u16_b,         s390_vmnlh,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_min_u16_c,         s390_vmnlh,         B_DEP,              0,                  BT_OV_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_min_s32_a,         s390_vmnf,          B_DEP,              0,                  BT_OV_V4SI_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_min_s32_b,         s390_vmnf,          0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_min_s32_c,         s390_vmnf,          B_DEP,              0,                  BT_OV_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_min_u32_a,         s390_vmnlf,         B_DEP,              0,                  BT_OV_UV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_min_u32_b,         s390_vmnlf,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_min_u32_c,         s390_vmnlf,         B_DEP,              0,                  BT_OV_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_min_s64_a,         s390_vmng,          B_DEP,              0,                  BT_OV_V2DI_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_min_s64_b,         s390_vmng,          0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_min_s64_c,         s390_vmng,          B_DEP,              0,                  BT_OV_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_min_u64_a,         s390_vmnlg,         B_DEP,              0,                  BT_OV_UV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_min_u64_b,         s390_vmnlg,         0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_min_u64_c,         s390_vmnlg,         B_DEP,              0,                  BT_OV_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_min_flt,           s390_vfminsb_4,     B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_min_dbl,           s390_vfmindb_4,     B_VXE,              0,                  BT_OV_V2DF_V2DF_V2DF)

B_DEF      (s390_vmnb,                  sminv16qi3,         0,                  B_VX,               0,                  BT_FN_V16QI_BV16QI_V16QI)
B_DEF      (s390_vmnlb,                 uminv16qi3,         0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vmnh,                  sminv8hi3,          0,                  B_VX,               0,                  BT_FN_V8HI_BV8HI_V8HI)
B_DEF      (s390_vmnlh,                 uminv8hi3,          0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vmnf,                  sminv4si3,          0,                  B_VX,               0,                  BT_FN_V4SI_BV4SI_V4SI)
B_DEF      (s390_vmnlf,                 uminv4si3,          0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vmng,                  sminv2di3,          0,                  B_VX,               0,                  BT_FN_V2DI_BV2DI_V2DI)
B_DEF      (s390_vmnlg,                 uminv2di3,          0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI_UV2DI)
B_DEF      (s390_vfminsb,               vfminv4sf,          0,                  B_VXE,              O3_U4,              BT_FN_V4SF_V4SF_V4SF_INT)
B_DEF      (s390_vfmindb,               vfminv2df,          0,                  B_VXE,              O3_U4,              BT_FN_V2DF_V2DF_V2DF_INT)
B_DEF      (s390_vfminsb_4,             sminv4sf3,          0,                  B_INT | B_VXE,      0,                  BT_FN_V4SF_V4SF_V4SF) /* vfminsb */
B_DEF      (s390_vfmindb_4,             sminv2df3,          0,                  B_INT | B_VXE,      0,                  BT_FN_V2DF_V2DF_V2DF) /* vfmindb */

OB_DEF     (s390_vec_mladd,             s390_vec_mladd_u8,  s390_vec_mladd_s32_c,B_VX,              BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_mladd_u8,          s390_vmalb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_mladd_s8_a,        s390_vmalb,         0,                  0,                  BT_OV_V16QI_UV16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_mladd_s8_b,        s390_vmalb,         0,                  0,                  BT_OV_V16QI_V16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_mladd_s8_c,        s390_vmalb,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_mladd_u16,         s390_vmalhw,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_mladd_s16_a,       s390_vmalhw,        0,                  0,                  BT_OV_V8HI_UV8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_mladd_s16_b,       s390_vmalhw,        0,                  0,                  BT_OV_V8HI_V8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_mladd_s16_c,       s390_vmalhw,        0,                  0,                  BT_OV_V8HI_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_mladd_u32,         s390_vmalf,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_mladd_s32_a,       s390_vmalf,         0,                  0,                  BT_OV_V4SI_UV4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_mladd_s32_b,       s390_vmalf,         0,                  0,                  BT_OV_V4SI_V4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_mladd_s32_c,       s390_vmalf,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI_V4SI)

B_DEF      (s390_vmalb,                 vec_vmalv16qi,      0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vmalhw,                vec_vmalv8hi,       0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vmalf,                 vec_vmalv4si,       0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_mhadd,             s390_vec_mhadd_u8,  s390_vec_mhadd_s32, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_mhadd_u8,          s390_vmalhb,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_mhadd_s8,          s390_vmahb,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_mhadd_u16,         s390_vmalhh,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_mhadd_s16,         s390_vmahh,         0,                  0,                  BT_OV_V8HI_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_mhadd_u32,         s390_vmalhf,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_mhadd_s32,         s390_vmahf,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI_V4SI)

B_DEF      (s390_vmalhb,                vec_vmalhv16qi,     0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vmahb,                 vec_vmahv16qi,      0,                  B_VX,               0,                  BT_FN_V16QI_V16QI_V16QI_V16QI)
B_DEF      (s390_vmalhh,                vec_vmalhv8hi,      0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vmahh,                 vec_vmahv8hi,       0,                  B_VX,               0,                  BT_FN_V8HI_V8HI_V8HI_V8HI)
B_DEF      (s390_vmalhf,                vec_vmalhv4si,      0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vmahf,                 vec_vmahv4si,       0,                  B_VX,               0,                  BT_FN_V4SI_V4SI_V4SI_V4SI)

OB_DEF     (s390_vec_meadd,             s390_vec_meadd_u8,  s390_vec_meadd_s32, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_meadd_u8,          s390_vmaleb,        0,                  0,                  BT_OV_UV8HI_UV16QI_UV16QI_UV8HI)
OB_DEF_VAR (s390_vec_meadd_s8,          s390_vmaeb,         0,                  0,                  BT_OV_V8HI_V16QI_V16QI_V8HI)
OB_DEF_VAR (s390_vec_meadd_u16,         s390_vmaleh,        0,                  0,                  BT_OV_UV4SI_UV8HI_UV8HI_UV4SI)
OB_DEF_VAR (s390_vec_meadd_s16,         s390_vmaeh,         0,                  0,                  BT_OV_V4SI_V8HI_V8HI_V4SI)
OB_DEF_VAR (s390_vec_meadd_u32,         s390_vmalef,        0,                  0,                  BT_OV_UV2DI_UV4SI_UV4SI_UV2DI)
OB_DEF_VAR (s390_vec_meadd_s32,         s390_vmaef,         0,                  0,                  BT_OV_V2DI_V4SI_V4SI_V2DI)

B_DEF      (s390_vmaleb,                vec_vmalev16qi,     0,                  B_VX,               0,                  BT_FN_UV8HI_UV16QI_UV16QI_UV8HI)
B_DEF      (s390_vmaeb,                 vec_vmaev16qi,      0,                  B_VX,               0,                  BT_FN_V8HI_V16QI_V16QI_V8HI)
B_DEF      (s390_vmaleh,                vec_vmalev8hi,      0,                  B_VX,               0,                  BT_FN_UV4SI_UV8HI_UV8HI_UV4SI)
B_DEF      (s390_vmaeh,                 vec_vmaev8hi,       0,                  B_VX,               0,                  BT_FN_V4SI_V8HI_V8HI_V4SI)
B_DEF      (s390_vmalef,                vec_vmalev4si,      0,                  B_VX,               0,                  BT_FN_UV2DI_UV4SI_UV4SI_UV2DI)
B_DEF      (s390_vmaef,                 vec_vmaev4si,       0,                  B_VX,               0,                  BT_FN_V2DI_V4SI_V4SI_V2DI)

OB_DEF     (s390_vec_moadd,             s390_vec_moadd_u8,  s390_vec_moadd_s32, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_moadd_u8,          s390_vmalob,        0,                  0,                  BT_OV_UV8HI_UV16QI_UV16QI_UV8HI)
OB_DEF_VAR (s390_vec_moadd_s8,          s390_vmaob,         0,                  0,                  BT_OV_V8HI_V16QI_V16QI_V8HI)
OB_DEF_VAR (s390_vec_moadd_u16,         s390_vmaloh,        0,                  0,                  BT_OV_UV4SI_UV8HI_UV8HI_UV4SI)
OB_DEF_VAR (s390_vec_moadd_s16,         s390_vmaoh,         0,                  0,                  BT_OV_V4SI_V8HI_V8HI_V4SI)
OB_DEF_VAR (s390_vec_moadd_u32,         s390_vmalof,        0,                  0,                  BT_OV_UV2DI_UV4SI_UV4SI_UV2DI)
OB_DEF_VAR (s390_vec_moadd_s32,         s390_vmaof,         0,                  0,                  BT_OV_V2DI_V4SI_V4SI_V2DI)

B_DEF      (s390_vmalob,                vec_vmalov16qi,     0,                  B_VX,               0,                  BT_FN_UV8HI_UV16QI_UV16QI_UV8HI)
B_DEF      (s390_vmaob,                 vec_vmaov16qi,      0,                  B_VX,               0,                  BT_FN_V8HI_V16QI_V16QI_V8HI)
B_DEF      (s390_vmaloh,                vec_vmalov8hi,      0,                  B_VX,               0,                  BT_FN_UV4SI_UV8HI_UV8HI_UV4SI)
B_DEF      (s390_vmaoh,                 vec_vmaov8hi,       0,                  B_VX,               0,                  BT_FN_V4SI_V8HI_V8HI_V4SI)
B_DEF      (s390_vmalof,                vec_vmalov4si,      0,                  B_VX,               0,                  BT_FN_UV2DI_UV4SI_UV4SI_UV2DI)
B_DEF      (s390_vmaof,                 vec_vmaov4si,       0,                  B_VX,               0,                  BT_FN_V2DI_V4SI_V4SI_V2DI)

OB_DEF     (s390_vec_mulh,              s390_vec_mulh_u8,   s390_vec_mulh_s32,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_mulh_u8,           s390_vmlhb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_mulh_s8,           s390_vmhb,          0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_mulh_u16,          s390_vmlhh,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_mulh_s16,          s390_vmhh,          0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_mulh_u32,          s390_vmlhf,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_mulh_s32,          s390_vmhf,          0,                  0,                  BT_OV_V4SI_V4SI_V4SI)

B_DEF      (s390_vmlhb,                 vec_umulhv16qi,     0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vmhb,                  vec_smulhv16qi,     0,                  B_VX,               0,                  BT_FN_V16QI_V16QI_V16QI)
B_DEF      (s390_vmlhh,                 vec_umulhv8hi,      0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vmhh,                  vec_smulhv8hi,      0,                  B_VX,               0,                  BT_FN_V8HI_V8HI_V8HI)
B_DEF      (s390_vmlhf,                 vec_umulhv4si,      0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vmhf,                  vec_smulhv4si,      0,                  B_VX,               0,                  BT_FN_V4SI_V4SI_V4SI)

OB_DEF     (s390_vec_mule,              s390_vec_mule_u8,   s390_vec_mule_s32,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_mule_u8,           s390_vmleb,         0,                  0,                  BT_OV_UV8HI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_mule_s8,           s390_vmeb,          0,                  0,                  BT_OV_V8HI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_mule_u16,          s390_vmleh,         0,                  0,                  BT_OV_UV4SI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_mule_s15,          s390_vmeh,          0,                  0,                  BT_OV_V4SI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_mule_u32,          s390_vmlef,         0,                  0,                  BT_OV_UV2DI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_mule_s32,          s390_vmef,          0,                  0,                  BT_OV_V2DI_V4SI_V4SI)

B_DEF      (s390_vmleb,                 vec_widen_umult_even_v16qi,0,           B_VX,               0,                  BT_FN_UV8HI_UV16QI_UV16QI)
B_DEF      (s390_vmeb,                  vec_widen_smult_even_v16qi,0,           B_VX,               0,                  BT_FN_V8HI_V16QI_V16QI)
B_DEF      (s390_vmleh,                 vec_widen_umult_even_v8hi,0,            B_VX,               0,                  BT_FN_UV4SI_UV8HI_UV8HI)
B_DEF      (s390_vmeh,                  vec_widen_smult_even_v8hi,0,            B_VX,               0,                  BT_FN_V4SI_V8HI_V8HI)
B_DEF      (s390_vmlef,                 vec_widen_umult_even_v4si,0,            B_VX,               0,                  BT_FN_UV2DI_UV4SI_UV4SI)
B_DEF      (s390_vmef,                  vec_widen_smult_even_v4si,0,            B_VX,               0,                  BT_FN_V2DI_V4SI_V4SI)

OB_DEF     (s390_vec_mulo,              s390_vec_mulo_u8,   s390_vec_mulo_s32,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_mulo_u8,           s390_vmlob,         0,                  0,                  BT_OV_UV8HI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_mulo_s8,           s390_vmob,          0,                  0,                  BT_OV_V8HI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_mulo_u16,          s390_vmloh,         0,                  0,                  BT_OV_UV4SI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_mulo_s16,          s390_vmoh,          0,                  0,                  BT_OV_V4SI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_mulo_u32,          s390_vmlof,         0,                  0,                  BT_OV_UV2DI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_mulo_s32,          s390_vmof,          0,                  0,                  BT_OV_V2DI_V4SI_V4SI)

B_DEF      (s390_vmlob,                 vec_widen_umult_odd_v16qi,0,            B_VX,               0,                  BT_FN_UV8HI_UV16QI_UV16QI)
B_DEF      (s390_vmob,                  vec_widen_smult_odd_v16qi,0,            B_VX,               0,                  BT_FN_V8HI_V16QI_V16QI)
B_DEF      (s390_vmloh,                 vec_widen_umult_odd_v8hi,0,             B_VX,               0,                  BT_FN_UV4SI_UV8HI_UV8HI)
B_DEF      (s390_vmoh,                  vec_widen_smult_odd_v8hi,0,             B_VX,               0,                  BT_FN_V4SI_V8HI_V8HI)
B_DEF      (s390_vmlof,                 vec_widen_umult_odd_v4si,0,             B_VX,               0,                  BT_FN_UV2DI_UV4SI_UV4SI)
B_DEF      (s390_vmof,                  vec_widen_smult_odd_v4si,0,             B_VX,               0,                  BT_FN_V2DI_V4SI_V4SI)

OB_DEF     (s390_vec_nor,               s390_vec_nor_b8,    s390_vec_nor_dbl_c, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_nor_b8,            s390_vno,           0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_nor_s8_a,          s390_vno,           B_DEP,              0,                  BT_OV_V16QI_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_nor_s8_b,          s390_vno,           0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_nor_s8_c,          s390_vno,           B_DEP,              0,                  BT_OV_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_nor_u8_a,          s390_vno,           B_DEP,              0,                  BT_OV_UV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_nor_u8_b,          s390_vno,           0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_nor_u8_c,          s390_vno,           B_DEP,              0,                  BT_OV_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_nor_b16,           s390_vno,           0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_nor_s16_a,         s390_vno,           B_DEP,              0,                  BT_OV_V8HI_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_nor_s16_b,         s390_vno,           0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_nor_s16_c,         s390_vno,           B_DEP,              0,                  BT_OV_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_nor_u16_a,         s390_vno,           B_DEP,              0,                  BT_OV_UV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_nor_u16_b,         s390_vno,           0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_nor_u16_c,         s390_vno,           B_DEP,              0,                  BT_OV_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_nor_b32,           s390_vno,           0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_nor_s32_a,         s390_vno,           B_DEP,              0,                  BT_OV_V4SI_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_nor_s32_b,         s390_vno,           0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_nor_s32_c,         s390_vno,           B_DEP,              0,                  BT_OV_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_nor_u32_a,         s390_vno,           B_DEP,              0,                  BT_OV_UV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_nor_u32_b,         s390_vno,           0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_nor_u32_c,         s390_vno,           B_DEP,              0,                  BT_OV_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_nor_b64,           s390_vno,           0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_nor_s64_a,         s390_vno,           B_DEP,              0,                  BT_OV_V2DI_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_nor_s64_b,         s390_vno,           0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_nor_s64_c,         s390_vno,           B_DEP,              0,                  BT_OV_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_nor_u64_a,         s390_vno,           B_DEP,              0,                  BT_OV_UV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_nor_u64_b,         s390_vno,           0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_nor_u64_c,         s390_vno,           B_DEP,              0,                  BT_OV_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_nor_flt_a,         s390_vno,           B_VXE | B_DEP,      0,                  BT_OV_V4SF_BV4SI_V4SF)
OB_DEF_VAR (s390_vec_nor_flt_b,         s390_vno,           B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_nor_flt_c,         s390_vno,           B_VXE | B_DEP,      0,                  BT_OV_V4SF_V4SF_BV4SI)
OB_DEF_VAR (s390_vec_nor_dbl_a,         s390_vno,           B_DEP,              0,                  BT_OV_V2DF_BV2DI_V2DF)
OB_DEF_VAR (s390_vec_nor_dbl_b,         s390_vno,           0,                  0,                  BT_OV_V2DF_V2DF_V2DF)
OB_DEF_VAR (s390_vec_nor_dbl_c,         s390_vno,           B_DEP,              0,                  BT_OV_V2DF_V2DF_BV2DI)

B_DEF      (s390_vno,                   vec_norv16qi3,      0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_or,                s390_vec_or_b8,     s390_vec_or_dbl_c,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_or_b8,             s390_vo,            0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_or_s8_a,           s390_vo,            B_DEP,              0,                  BT_OV_V16QI_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_or_s8_b,           s390_vo,            0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_or_s8_c,           s390_vo,            B_DEP,              0,                  BT_OV_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_or_u8_a,           s390_vo,            B_DEP,              0,                  BT_OV_UV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_or_u8_b,           s390_vo,            0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_or_u8_c,           s390_vo,            B_DEP,              0,                  BT_OV_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_or_b16,            s390_vo,            0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_or_s16_a,          s390_vo,            B_DEP,              0,                  BT_OV_V8HI_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_or_s16_b,          s390_vo,            0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_or_s16_c,          s390_vo,            B_DEP,              0,                  BT_OV_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_or_u16_a,          s390_vo,            B_DEP,              0,                  BT_OV_UV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_or_u16_b,          s390_vo,            0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_or_u16_c,          s390_vo,            B_DEP,              0,                  BT_OV_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_or_b32,            s390_vo,            0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_or_s32_a,          s390_vo,            B_DEP,              0,                  BT_OV_V4SI_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_or_s32_b,          s390_vo,            0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_or_s32_c,          s390_vo,            B_DEP,              0,                  BT_OV_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_or_u32_a,          s390_vo,            B_DEP,              0,                  BT_OV_UV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_or_u32_b,          s390_vo,            0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_or_u32_c,          s390_vo,            B_DEP,              0,                  BT_OV_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_or_b64,            s390_vo,            0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_or_s64_a,          s390_vo,            B_DEP,              0,                  BT_OV_V2DI_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_or_s64_b,          s390_vo,            0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_or_s64_c,          s390_vo,            B_DEP,              0,                  BT_OV_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_or_u64_a,          s390_vo,            B_DEP,              0,                  BT_OV_UV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_or_u64_b,          s390_vo,            0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_or_u64_c,          s390_vo,            B_DEP,              0,                  BT_OV_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_or_flt_a,          s390_vo,            B_VXE | B_DEP,      0,                  BT_OV_V4SF_BV4SI_V4SF)
OB_DEF_VAR (s390_vec_or_flt_b,          s390_vo,            B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_or_flt_c,          s390_vo,            B_VXE | B_DEP,      0,                  BT_OV_V4SF_V4SF_BV4SI)
OB_DEF_VAR (s390_vec_or_dbl_a,          s390_vo,            B_DEP,              0,                  BT_OV_V2DF_BV2DI_V2DF)
OB_DEF_VAR (s390_vec_or_dbl_b,          s390_vo,            0,                  0,                  BT_OV_V2DF_V2DF_V2DF)
OB_DEF_VAR (s390_vec_or_dbl_c,          s390_vo,            B_DEP,              0,                  BT_OV_V2DF_V2DF_BV2DI)

B_DEF      (s390_vo,                    iorv16qi3,          0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_popcnt,            s390_vec_popcnt_s8, s390_vec_popcnt_u64,B_VX,               BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_popcnt_s8,         s390_vpopctb,       0,                  0,                  BT_OV_UV16QI_V16QI)
OB_DEF_VAR (s390_vec_popcnt_u8,         s390_vpopctb,       0,                  0,                  BT_OV_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_popcnt_s16,        s390_vpopcth,       0,                  0,                  BT_OV_UV8HI_V8HI)
OB_DEF_VAR (s390_vec_popcnt_u16,        s390_vpopcth,       0,                  0,                  BT_OV_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_popcnt_s32,        s390_vpopctf,       0,                  0,                  BT_OV_UV4SI_V4SI)
OB_DEF_VAR (s390_vec_popcnt_u32,        s390_vpopctf,       0,                  0,                  BT_OV_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_popcnt_s64,        s390_vpopctg,       0,                  0,                  BT_OV_UV2DI_V2DI)
OB_DEF_VAR (s390_vec_popcnt_u64,        s390_vpopctg,       0,                  0,                  BT_OV_UV2DI_UV2DI)

B_DEF      (s390_vpopctb,               popcountv16qi2,     0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI)
B_DEF      (s390_vpopcth,               popcountv8hi2,      0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI)
B_DEF      (s390_vpopctf,               popcountv4si2,      0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI)
B_DEF      (s390_vpopctg,               popcountv2di2,      0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI)

OB_DEF     (s390_vec_rl,                s390_vec_rl_u8,     s390_vec_rl_s64,    B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_rl_u8,             s390_verllvb,       0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_rl_s8,             s390_verllvb,       0,                  0,                  BT_OV_V16QI_V16QI_UV16QI)
OB_DEF_VAR (s390_vec_rl_u16,            s390_verllvh,       0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_rl_s16,            s390_verllvh,       0,                  0,                  BT_OV_V8HI_V8HI_UV8HI)
OB_DEF_VAR (s390_vec_rl_u32,            s390_verllvf,       0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_rl_s32,            s390_verllvf,       0,                  0,                  BT_OV_V4SI_V4SI_UV4SI)
OB_DEF_VAR (s390_vec_rl_u64,            s390_verllvg,       0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_rl_s64,            s390_verllvg,       0,                  0,                  BT_OV_V2DI_V2DI_UV2DI)

B_DEF      (s390_verllvb,               vrotlv16qi3,        0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_verllvh,               vrotlv8hi3,         0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_verllvf,               vrotlv4si3,         0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_verllvg,               vrotlv2di3,         0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI_UV2DI)

OB_DEF     (s390_vec_rli,               s390_vec_rli_u8,    s390_vec_rli_s64,   B_VX,               BT_FN_OV4SI_OV4SI_ULONG)
OB_DEF_VAR (s390_vec_rli_u8,            s390_verllb,        0,                  0,                  BT_OV_UV16QI_UV16QI_ULONG)
OB_DEF_VAR (s390_vec_rli_s8,            s390_verllb,        0,                  0,                  BT_OV_V16QI_V16QI_ULONG)
OB_DEF_VAR (s390_vec_rli_u16,           s390_verllh,        0,                  0,                  BT_OV_UV8HI_UV8HI_ULONG)
OB_DEF_VAR (s390_vec_rli_s16,           s390_verllh,        0,                  0,                  BT_OV_V8HI_V8HI_ULONG)
OB_DEF_VAR (s390_vec_rli_u32,           s390_verllf,        0,                  0,                  BT_OV_UV4SI_UV4SI_ULONG)
OB_DEF_VAR (s390_vec_rli_s32,           s390_verllf,        0,                  0,                  BT_OV_V4SI_V4SI_ULONG)
OB_DEF_VAR (s390_vec_rli_u64,           s390_verllg,        0,                  0,                  BT_OV_UV2DI_UV2DI_ULONG)
OB_DEF_VAR (s390_vec_rli_s64,           s390_verllg,        0,                  0,                  BT_OV_V2DI_V2DI_ULONG)

B_DEF      (s390_verllb,                rotlv16qi3,         0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UINT)
B_DEF      (s390_verllh,                rotlv8hi3,          0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UINT)
B_DEF      (s390_verllf,                rotlv4si3,          0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UINT)
B_DEF      (s390_verllg,                rotlv2di3,          0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI_UINT)

OB_DEF     (s390_vec_rl_mask,           s390_vec_rl_mask_s8,s390_vec_rl_mask_u64,B_VX,              BT_FN_OV4SI_OV4SI_OV4SI_UCHAR)
OB_DEF_VAR (s390_vec_rl_mask_s8,        s390_verimb,        0,                  O3_U8,              BT_OV_V16QI_V16QI_UV16QI_UCHAR)
OB_DEF_VAR (s390_vec_rl_mask_u8,        s390_verimb,        0,                  O3_U8,              BT_OV_UV16QI_UV16QI_UV16QI_UCHAR)
OB_DEF_VAR (s390_vec_rl_mask_s16,       s390_verimh,        0,                  O3_U8,              BT_OV_V8HI_V8HI_UV8HI_UCHAR)
OB_DEF_VAR (s390_vec_rl_mask_u16,       s390_verimh,        0,                  O3_U8,              BT_OV_UV8HI_UV8HI_UV8HI_UCHAR)
OB_DEF_VAR (s390_vec_rl_mask_s32,       s390_verimf,        0,                  O3_U8,              BT_OV_V4SI_V4SI_UV4SI_UCHAR)
OB_DEF_VAR (s390_vec_rl_mask_u32,       s390_verimf,        0,                  O3_U8,              BT_OV_UV4SI_UV4SI_UV4SI_UCHAR)
OB_DEF_VAR (s390_vec_rl_mask_s64,       s390_verimg,        0,                  O3_U8,              BT_OV_V2DI_V2DI_UV2DI_UCHAR)
OB_DEF_VAR (s390_vec_rl_mask_u64,       s390_verimg,        0,                  O3_U8,              BT_OV_UV2DI_UV2DI_UV2DI_UCHAR)

B_DEF      (s390_verimb,                verimv16qi,         0,                  B_VX,               O4_U8,              BT_FN_UV16QI_UV16QI_UV16QI_UV16QI_INT)
B_DEF      (s390_verimh,                verimv8hi,          0,                  B_VX,               O4_U8,              BT_FN_UV8HI_UV8HI_UV8HI_UV8HI_INT)
B_DEF      (s390_verimf,                verimv4si,          0,                  B_VX,               O4_U8,              BT_FN_UV4SI_UV4SI_UV4SI_UV4SI_INT)
B_DEF      (s390_verimg,                verimv2di,          0,                  B_VX,               O4_U8,              BT_FN_UV2DI_UV2DI_UV2DI_UV2DI_INT)

OB_DEF     (s390_vec_sll,               s390_vec_sll_u8q,   s390_vec_sll_b64s,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_sll_u8q,           s390_vsl,           0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_sll_u8h,           s390_vsl,           B_DEP,              0,                  BT_OV_UV16QI_UV16QI_UV8HI)
OB_DEF_VAR (s390_vec_sll_u8s,           s390_vsl,           B_DEP,              0,                  BT_OV_UV16QI_UV16QI_UV4SI)
OB_DEF_VAR (s390_vec_sll_s8q,           s390_vsl,           0,                  0,                  BT_OV_V16QI_V16QI_UV16QI)
OB_DEF_VAR (s390_vec_sll_s8h,           s390_vsl,           B_DEP,              0,                  BT_OV_V16QI_V16QI_UV8HI)
OB_DEF_VAR (s390_vec_sll_s8s,           s390_vsl,           B_DEP,              0,                  BT_OV_V16QI_V16QI_UV4SI)
OB_DEF_VAR (s390_vec_sll_b8q,           s390_vsl,           B_DEP,              0,                  BT_OV_BV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_sll_b8h,           s390_vsl,           B_DEP,              0,                  BT_OV_BV16QI_BV16QI_UV8HI)
OB_DEF_VAR (s390_vec_sll_b8s,           s390_vsl,           B_DEP,              0,                  BT_OV_BV16QI_BV16QI_UV4SI)
OB_DEF_VAR (s390_vec_sll_u16q,          s390_vsl,           0,                  0,                  BT_OV_UV8HI_UV8HI_UV16QI)
OB_DEF_VAR (s390_vec_sll_u16h,          s390_vsl,           B_DEP,              0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_sll_u16s,          s390_vsl,           B_DEP,              0,                  BT_OV_UV8HI_UV8HI_UV4SI)
OB_DEF_VAR (s390_vec_sll_s16q,          s390_vsl,           0,                  0,                  BT_OV_V8HI_V8HI_UV16QI)
OB_DEF_VAR (s390_vec_sll_s16h,          s390_vsl,           B_DEP,              0,                  BT_OV_V8HI_V8HI_UV8HI)
OB_DEF_VAR (s390_vec_sll_s16s,          s390_vsl,           B_DEP,              0,                  BT_OV_V8HI_V8HI_UV4SI)
OB_DEF_VAR (s390_vec_sll_b16q,          s390_vsl,           B_DEP,              0,                  BT_OV_BV8HI_BV8HI_UV16QI)
OB_DEF_VAR (s390_vec_sll_b16h,          s390_vsl,           B_DEP,              0,                  BT_OV_BV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_sll_b16s,          s390_vsl,           B_DEP,              0,                  BT_OV_BV8HI_BV8HI_UV4SI)
OB_DEF_VAR (s390_vec_sll_u32q,          s390_vsl,           0,                  0,                  BT_OV_UV4SI_UV4SI_UV16QI)
OB_DEF_VAR (s390_vec_sll_u32h,          s390_vsl,           B_DEP,              0,                  BT_OV_UV4SI_UV4SI_UV8HI)
OB_DEF_VAR (s390_vec_sll_u32s,          s390_vsl,           B_DEP,              0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_sll_s32q,          s390_vsl,           0,                  0,                  BT_OV_V4SI_V4SI_UV16QI)
OB_DEF_VAR (s390_vec_sll_s32h,          s390_vsl,           B_DEP,              0,                  BT_OV_V4SI_V4SI_UV8HI)
OB_DEF_VAR (s390_vec_sll_s32s,          s390_vsl,           B_DEP,              0,                  BT_OV_V4SI_V4SI_UV4SI)
OB_DEF_VAR (s390_vec_sll_b32q,          s390_vsl,           B_DEP,              0,                  BT_OV_BV4SI_BV4SI_UV16QI)
OB_DEF_VAR (s390_vec_sll_b32h,          s390_vsl,           B_DEP,              0,                  BT_OV_BV4SI_BV4SI_UV8HI)
OB_DEF_VAR (s390_vec_sll_b32s,          s390_vsl,           B_DEP,              0,                  BT_OV_BV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_sll_u64q,          s390_vsl,           0,                  0,                  BT_OV_UV2DI_UV2DI_UV16QI)
OB_DEF_VAR (s390_vec_sll_u64h,          s390_vsl,           B_DEP,              0,                  BT_OV_UV2DI_UV2DI_UV8HI)
OB_DEF_VAR (s390_vec_sll_u64s,          s390_vsl,           B_DEP,              0,                  BT_OV_UV2DI_UV2DI_UV4SI)
OB_DEF_VAR (s390_vec_sll_s64q,          s390_vsl,           0,                  0,                  BT_OV_V2DI_V2DI_UV16QI)
OB_DEF_VAR (s390_vec_sll_s64h,          s390_vsl,           B_DEP,              0,                  BT_OV_V2DI_V2DI_UV8HI)
OB_DEF_VAR (s390_vec_sll_s64s,          s390_vsl,           B_DEP,              0,                  BT_OV_V2DI_V2DI_UV4SI)
OB_DEF_VAR (s390_vec_sll_b64q,          s390_vsl,           B_DEP,              0,                  BT_OV_BV2DI_BV2DI_UV16QI)
OB_DEF_VAR (s390_vec_sll_b64h,          s390_vsl,           B_DEP,              0,                  BT_OV_BV2DI_BV2DI_UV8HI)
OB_DEF_VAR (s390_vec_sll_b64s,          s390_vsl,           B_DEP,              0,                  BT_OV_BV2DI_BV2DI_UV4SI)

B_DEF      (s390_vsl,                   vec_sllv16qiv16qi,  0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_slb,               s390_vec_slb_u8_u8, s390_vec_slb_dbl_s64,B_VX,              BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_slb_u8_u8,         s390_vslb,          0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_slb_u8_s8,         s390_vslb,          0,                  0,                  BT_OV_UV16QI_UV16QI_V16QI)
OB_DEF_VAR (s390_vec_slb_s8_u8,         s390_vslb,          0,                  0,                  BT_OV_V16QI_V16QI_UV16QI)
OB_DEF_VAR (s390_vec_slb_s8_s8,         s390_vslb,          0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_slb_u16_u16,       s390_vslb,          0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_slb_u16_s16,       s390_vslb,          0,                  0,                  BT_OV_UV8HI_UV8HI_V8HI)
OB_DEF_VAR (s390_vec_slb_s16_u16,       s390_vslb,          0,                  0,                  BT_OV_V8HI_V8HI_UV8HI)
OB_DEF_VAR (s390_vec_slb_s16_s16,       s390_vslb,          0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_slb_u32_u32,       s390_vslb,          0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_slb_u32_s32,       s390_vslb,          0,                  0,                  BT_OV_UV4SI_UV4SI_V4SI)
OB_DEF_VAR (s390_vec_slb_s32_u32,       s390_vslb,          0,                  0,                  BT_OV_V4SI_V4SI_UV4SI)
OB_DEF_VAR (s390_vec_slb_s32_s32,       s390_vslb,          0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_slb_u64_u64,       s390_vslb,          0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_slb_u64_s64,       s390_vslb,          0,                  0,                  BT_OV_UV2DI_UV2DI_V2DI)
OB_DEF_VAR (s390_vec_slb_s64_u64,       s390_vslb,          0,                  0,                  BT_OV_V2DI_V2DI_UV2DI)
OB_DEF_VAR (s390_vec_slb_s64_s64,       s390_vslb,          0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_slb_flt_u64,       s390_vslb,          B_VXE,              0,                  BT_OV_V4SF_V4SF_UV4SI)
OB_DEF_VAR (s390_vec_slb_dbl_u64,       s390_vslb,          0,                  0,                  BT_OV_V2DF_V2DF_UV2DI)
OB_DEF_VAR (s390_vec_slb_flt_s64,       s390_vslb,          B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SI)
OB_DEF_VAR (s390_vec_slb_dbl_s64,       s390_vslb,          0,                  0,                  BT_OV_V2DF_V2DF_V2DI)

B_DEF      (s390_vslb,                  vec_slbv16qi,       0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_sld,               s390_vec_sld_b8,    s390_vec_sld_dbl,   B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_b8,            s390_vsldb,         0,                  O3_U4,              BT_OV_BV16QI_BV16QI_BV16QI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_s8,            s390_vsldb,         0,                  O3_U4,              BT_OV_V16QI_V16QI_V16QI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_u8,            s390_vsldb,         0,                  O3_U4,              BT_OV_UV16QI_UV16QI_UV16QI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_b16,           s390_vsldb,         0,                  O3_U4,              BT_OV_BV8HI_BV8HI_BV8HI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_s16,           s390_vsldb,         0,                  O3_U4,              BT_OV_V8HI_V8HI_V8HI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_u16,           s390_vsldb,         0,                  O3_U4,              BT_OV_UV8HI_UV8HI_UV8HI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_b32,           s390_vsldb,         0,                  O3_U4,              BT_OV_BV4SI_BV4SI_BV4SI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_s32,           s390_vsldb,         0,                  O3_U4,              BT_OV_V4SI_V4SI_V4SI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_u32,           s390_vsldb,         0,                  O3_U4,              BT_OV_UV4SI_UV4SI_UV4SI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_b64,           s390_vsldb,         0,                  O3_U4,              BT_OV_BV2DI_BV2DI_BV2DI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_s64,           s390_vsldb,         0,                  O3_U4,              BT_OV_V2DI_V2DI_V2DI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_u64,           s390_vsldb,         0,                  O3_U4,              BT_OV_UV2DI_UV2DI_UV2DI_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_flt,           s390_vsldb,         B_VXE,              O3_U4,              BT_OV_V4SF_V4SF_V4SF_ULONGLONG)
OB_DEF_VAR (s390_vec_sld_dbl,           s390_vsldb,         0,                  O3_U4,              BT_OV_V2DF_V2DF_V2DF_ULONGLONG)

B_DEF      (s390_vsldb,                 vec_sldv16qi,       0,                  B_VX,               O3_U4,              BT_FN_UV16QI_UV16QI_UV16QI_INT)

OB_DEF     (s390_vec_sldw,              s390_vec_sldw_s8,   s390_vec_sldw_dbl,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_INT)
OB_DEF_VAR (s390_vec_sldw_s8,           s390_vsldb,         0,                  O3_U4,              BT_OV_V16QI_V16QI_V16QI_INT)
OB_DEF_VAR (s390_vec_sldw_u8,           s390_vsldb,         0,                  O3_U4,              BT_OV_UV16QI_UV16QI_UV16QI_INT)
OB_DEF_VAR (s390_vec_sldw_s16,          s390_vsldb,         0,                  O3_U4,              BT_OV_V8HI_V8HI_V8HI_INT)
OB_DEF_VAR (s390_vec_sldw_u16,          s390_vsldb,         0,                  O3_U4,              BT_OV_UV8HI_UV8HI_UV8HI_INT)
OB_DEF_VAR (s390_vec_sldw_s32,          s390_vsldb,         0,                  O3_U4,              BT_OV_V4SI_V4SI_V4SI_INT)
OB_DEF_VAR (s390_vec_sldw_u32,          s390_vsldb,         0,                  O3_U4,              BT_OV_UV4SI_UV4SI_UV4SI_INT)
OB_DEF_VAR (s390_vec_sldw_s64,          s390_vsldb,         0,                  O3_U4,              BT_OV_V2DI_V2DI_V2DI_INT)
OB_DEF_VAR (s390_vec_sldw_u64,          s390_vsldb,         0,                  O3_U4,              BT_OV_UV2DI_UV2DI_UV2DI_INT)
OB_DEF_VAR (s390_vec_sldw_dbl,          s390_vsldb,         B_DEP,              O3_U4,              BT_OV_V2DF_V2DF_V2DF_INT)

OB_DEF     (s390_vec_sral,              s390_vec_sral_u8q,  s390_vec_sral_b64s, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_sral_u8q,          s390_vsra,          0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_sral_u8h,          s390_vsra,          B_DEP,              0,                  BT_OV_UV16QI_UV16QI_UV8HI)
OB_DEF_VAR (s390_vec_sral_u8s,          s390_vsra,          B_DEP,              0,                  BT_OV_UV16QI_UV16QI_UV4SI)
OB_DEF_VAR (s390_vec_sral_s8q,          s390_vsra,          0,                  0,                  BT_OV_V16QI_V16QI_UV16QI)
OB_DEF_VAR (s390_vec_sral_s8h,          s390_vsra,          B_DEP,              0,                  BT_OV_V16QI_V16QI_UV8HI)
OB_DEF_VAR (s390_vec_sral_s8s,          s390_vsra,          B_DEP,              0,                  BT_OV_V16QI_V16QI_UV4SI)
OB_DEF_VAR (s390_vec_sral_b8q,          s390_vsra,          B_DEP,              0,                  BT_OV_BV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_sral_b8h,          s390_vsra,          B_DEP,              0,                  BT_OV_BV16QI_BV16QI_UV8HI)
OB_DEF_VAR (s390_vec_sral_b8s,          s390_vsra,          B_DEP,              0,                  BT_OV_BV16QI_BV16QI_UV4SI)
OB_DEF_VAR (s390_vec_sral_u16q,         s390_vsra,          0,                  0,                  BT_OV_UV8HI_UV8HI_UV16QI)
OB_DEF_VAR (s390_vec_sral_u16h,         s390_vsra,          B_DEP,              0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_sral_u16s,         s390_vsra,          B_DEP,              0,                  BT_OV_UV8HI_UV8HI_UV4SI)
OB_DEF_VAR (s390_vec_sral_s16q,         s390_vsra,          0,                  0,                  BT_OV_V8HI_V8HI_UV16QI)
OB_DEF_VAR (s390_vec_sral_s16h,         s390_vsra,          B_DEP,              0,                  BT_OV_V8HI_V8HI_UV8HI)
OB_DEF_VAR (s390_vec_sral_s16s,         s390_vsra,          B_DEP,              0,                  BT_OV_V8HI_V8HI_UV4SI)
OB_DEF_VAR (s390_vec_sral_b16q,         s390_vsra,          B_DEP,              0,                  BT_OV_BV8HI_BV8HI_UV16QI)
OB_DEF_VAR (s390_vec_sral_b16h,         s390_vsra,          B_DEP,              0,                  BT_OV_BV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_sral_b16s,         s390_vsra,          B_DEP,              0,                  BT_OV_BV8HI_BV8HI_UV4SI)
OB_DEF_VAR (s390_vec_sral_u32q,         s390_vsra,          0,                  0,                  BT_OV_UV4SI_UV4SI_UV16QI)
OB_DEF_VAR (s390_vec_sral_u32h,         s390_vsra,          B_DEP,              0,                  BT_OV_UV4SI_UV4SI_UV8HI)
OB_DEF_VAR (s390_vec_sral_u32s,         s390_vsra,          B_DEP,              0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_sral_s32q,         s390_vsra,          0,                  0,                  BT_OV_V4SI_V4SI_UV16QI)
OB_DEF_VAR (s390_vec_sral_s32h,         s390_vsra,          B_DEP,              0,                  BT_OV_V4SI_V4SI_UV8HI)
OB_DEF_VAR (s390_vec_sral_s32s,         s390_vsra,          B_DEP,              0,                  BT_OV_V4SI_V4SI_UV4SI)
OB_DEF_VAR (s390_vec_sral_b32q,         s390_vsra,          B_DEP,              0,                  BT_OV_BV4SI_BV4SI_UV16QI)
OB_DEF_VAR (s390_vec_sral_b32h,         s390_vsra,          B_DEP,              0,                  BT_OV_BV4SI_BV4SI_UV8HI)
OB_DEF_VAR (s390_vec_sral_b32s,         s390_vsra,          B_DEP,              0,                  BT_OV_BV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_sral_u64q,         s390_vsra,          0,                  0,                  BT_OV_UV2DI_UV2DI_UV16QI)
OB_DEF_VAR (s390_vec_sral_u64h,         s390_vsra,          B_DEP,              0,                  BT_OV_UV2DI_UV2DI_UV8HI)
OB_DEF_VAR (s390_vec_sral_u64s,         s390_vsra,          B_DEP,              0,                  BT_OV_UV2DI_UV2DI_UV4SI)
OB_DEF_VAR (s390_vec_sral_s64q,         s390_vsra,          0,                  0,                  BT_OV_V2DI_V2DI_UV16QI)
OB_DEF_VAR (s390_vec_sral_s64h,         s390_vsra,          B_DEP,              0,                  BT_OV_V2DI_V2DI_UV8HI)
OB_DEF_VAR (s390_vec_sral_s64s,         s390_vsra,          B_DEP,              0,                  BT_OV_V2DI_V2DI_UV4SI)
OB_DEF_VAR (s390_vec_sral_b64q,         s390_vsra,          B_DEP,              0,                  BT_OV_BV2DI_BV2DI_UV16QI)
OB_DEF_VAR (s390_vec_sral_b64h,         s390_vsra,          B_DEP,              0,                  BT_OV_BV2DI_BV2DI_UV8HI)
OB_DEF_VAR (s390_vec_sral_b64s,         s390_vsra,          B_DEP,              0,                  BT_OV_BV2DI_BV2DI_UV4SI)

B_DEF      (s390_vsra,                  vec_sralv16qiv16qi, 0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_srab,              s390_vec_srab_u8_u8,s390_vec_srab_dbl_s64,B_VX,             BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_srab_u8_u8,        s390_vsrab,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_srab_u8_s8,        s390_vsrab,         0,                  0,                  BT_OV_UV16QI_UV16QI_V16QI)
OB_DEF_VAR (s390_vec_srab_s8_u8,        s390_vsrab,         0,                  0,                  BT_OV_V16QI_V16QI_UV16QI)
OB_DEF_VAR (s390_vec_srab_s8_s8,        s390_vsrab,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_srab_u16_u16,      s390_vsrab,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_srab_u16_s16,      s390_vsrab,         0,                  0,                  BT_OV_UV8HI_UV8HI_V8HI)
OB_DEF_VAR (s390_vec_srab_s16_u16,      s390_vsrab,         0,                  0,                  BT_OV_V8HI_V8HI_UV8HI)
OB_DEF_VAR (s390_vec_srab_s16_s16,      s390_vsrab,         0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_srab_u32_u32,      s390_vsrab,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_srab_u32_s32,      s390_vsrab,         0,                  0,                  BT_OV_UV4SI_UV4SI_V4SI)
OB_DEF_VAR (s390_vec_srab_s32_u32,      s390_vsrab,         0,                  0,                  BT_OV_V4SI_V4SI_UV4SI)
OB_DEF_VAR (s390_vec_srab_s32_s32,      s390_vsrab,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_srab_u64_u64,      s390_vsrab,         0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_srab_u64_s64,      s390_vsrab,         0,                  0,                  BT_OV_UV2DI_UV2DI_V2DI)
OB_DEF_VAR (s390_vec_srab_s64_u64,      s390_vsrab,         0,                  0,                  BT_OV_V2DI_V2DI_UV2DI)
OB_DEF_VAR (s390_vec_srab_s64_s64,      s390_vsrab,         0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_srab_flt_u64,      s390_vsrab,         B_VXE,              0,                  BT_OV_V4SF_V4SF_UV4SI)
OB_DEF_VAR (s390_vec_srab_dbl_u64,      s390_vsrab,         0,                  0,                  BT_OV_V2DF_V2DF_UV2DI)
OB_DEF_VAR (s390_vec_srab_flt_s64,      s390_vsrab,         B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SI)
OB_DEF_VAR (s390_vec_srab_dbl_s64,      s390_vsrab,         0,                  0,                  BT_OV_V2DF_V2DF_V2DI)

B_DEF      (s390_vsrab,                 vec_srabv16qi,      0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_srl,               s390_vec_srl_u8q,   s390_vec_srl_b64s,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_srl_u8q,           s390_vsrl,          0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_srl_u8h,           s390_vsrl,          B_DEP,              0,                  BT_OV_UV16QI_UV16QI_UV8HI)
OB_DEF_VAR (s390_vec_srl_u8s,           s390_vsrl,          B_DEP,              0,                  BT_OV_UV16QI_UV16QI_UV4SI)
OB_DEF_VAR (s390_vec_srl_s8q,           s390_vsrl,          0,                  0,                  BT_OV_V16QI_V16QI_UV16QI)
OB_DEF_VAR (s390_vec_srl_s8h,           s390_vsrl,          B_DEP,              0,                  BT_OV_V16QI_V16QI_UV8HI)
OB_DEF_VAR (s390_vec_srl_s8s,           s390_vsrl,          B_DEP,              0,                  BT_OV_V16QI_V16QI_UV4SI)
OB_DEF_VAR (s390_vec_srl_b8q,           s390_vsrl,          B_DEP,              0,                  BT_OV_BV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_srl_b8h,           s390_vsrl,          B_DEP,              0,                  BT_OV_BV16QI_BV16QI_UV8HI)
OB_DEF_VAR (s390_vec_srl_b8s,           s390_vsrl,          B_DEP,              0,                  BT_OV_BV16QI_BV16QI_UV4SI)
OB_DEF_VAR (s390_vec_srl_u16q,          s390_vsrl,          0,                  0,                  BT_OV_UV8HI_UV8HI_UV16QI)
OB_DEF_VAR (s390_vec_srl_u16h,          s390_vsrl,          B_DEP,              0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_srl_u16s,          s390_vsrl,          B_DEP,              0,                  BT_OV_UV8HI_UV8HI_UV4SI)
OB_DEF_VAR (s390_vec_srl_s16q,          s390_vsrl,          0,                  0,                  BT_OV_V8HI_V8HI_UV16QI)
OB_DEF_VAR (s390_vec_srl_s16h,          s390_vsrl,          B_DEP,              0,                  BT_OV_V8HI_V8HI_UV8HI)
OB_DEF_VAR (s390_vec_srl_s16s,          s390_vsrl,          B_DEP,              0,                  BT_OV_V8HI_V8HI_UV4SI)
OB_DEF_VAR (s390_vec_srl_b16q,          s390_vsrl,          B_DEP,              0,                  BT_OV_BV8HI_BV8HI_UV16QI)
OB_DEF_VAR (s390_vec_srl_b16h,          s390_vsrl,          B_DEP,              0,                  BT_OV_BV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_srl_b16s,          s390_vsrl,          B_DEP,              0,                  BT_OV_BV8HI_BV8HI_UV4SI)
OB_DEF_VAR (s390_vec_srl_u32q,          s390_vsrl,          0,                  0,                  BT_OV_UV4SI_UV4SI_UV16QI)
OB_DEF_VAR (s390_vec_srl_u32h,          s390_vsrl,          B_DEP,              0,                  BT_OV_UV4SI_UV4SI_UV8HI)
OB_DEF_VAR (s390_vec_srl_u32s,          s390_vsrl,          B_DEP,              0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_srl_s32q,          s390_vsrl,          0,                  0,                  BT_OV_V4SI_V4SI_UV16QI)
OB_DEF_VAR (s390_vec_srl_s32h,          s390_vsrl,          B_DEP,              0,                  BT_OV_V4SI_V4SI_UV8HI)
OB_DEF_VAR (s390_vec_srl_s32s,          s390_vsrl,          B_DEP,              0,                  BT_OV_V4SI_V4SI_UV4SI)
OB_DEF_VAR (s390_vec_srl_b32q,          s390_vsrl,          B_DEP,              0,                  BT_OV_BV4SI_BV4SI_UV16QI)
OB_DEF_VAR (s390_vec_srl_b32h,          s390_vsrl,          B_DEP,              0,                  BT_OV_BV4SI_BV4SI_UV8HI)
OB_DEF_VAR (s390_vec_srl_b32s,          s390_vsrl,          B_DEP,              0,                  BT_OV_BV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_srl_u64q,          s390_vsrl,          0,                  0,                  BT_OV_UV2DI_UV2DI_UV16QI)
OB_DEF_VAR (s390_vec_srl_u64h,          s390_vsrl,          B_DEP,              0,                  BT_OV_UV2DI_UV2DI_UV8HI)
OB_DEF_VAR (s390_vec_srl_u64s,          s390_vsrl,          B_DEP,              0,                  BT_OV_UV2DI_UV2DI_UV4SI)
OB_DEF_VAR (s390_vec_srl_s64q,          s390_vsrl,          0,                  0,                  BT_OV_V2DI_V2DI_UV16QI)
OB_DEF_VAR (s390_vec_srl_s64h,          s390_vsrl,          B_DEP,              0,                  BT_OV_V2DI_V2DI_UV8HI)
OB_DEF_VAR (s390_vec_srl_s64s,          s390_vsrl,          B_DEP,              0,                  BT_OV_V2DI_V2DI_UV4SI)
OB_DEF_VAR (s390_vec_srl_b64q,          s390_vsrl,          B_DEP,              0,                  BT_OV_BV2DI_BV2DI_UV16QI)
OB_DEF_VAR (s390_vec_srl_b64h,          s390_vsrl,          B_DEP,              0,                  BT_OV_BV2DI_BV2DI_UV8HI)
OB_DEF_VAR (s390_vec_srl_b64s,          s390_vsrl,          B_DEP,              0,                  BT_OV_BV2DI_BV2DI_UV4SI)

B_DEF      (s390_vsrl,                  vec_srlv16qiv16qi,  0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_srb,               s390_vec_srb_u8_u8, s390_vec_srb_dbl_s64,B_VX,              BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_srb_u8_u8,         s390_vsrlb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_srb_u8_s8,         s390_vsrlb,         0,                  0,                  BT_OV_UV16QI_UV16QI_V16QI)
OB_DEF_VAR (s390_vec_srb_s8_u8,         s390_vsrlb,         0,                  0,                  BT_OV_V16QI_V16QI_UV16QI)
OB_DEF_VAR (s390_vec_srb_s8_s8,         s390_vsrlb,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_srb_u16_u16,       s390_vsrlb,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_srb_u16_s16,       s390_vsrlb,         0,                  0,                  BT_OV_UV8HI_UV8HI_V8HI)
OB_DEF_VAR (s390_vec_srb_s16_u16,       s390_vsrlb,         0,                  0,                  BT_OV_V8HI_V8HI_UV8HI)
OB_DEF_VAR (s390_vec_srb_s16_s16,       s390_vsrlb,         0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_srb_u32_u32,       s390_vsrlb,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_srb_u32_s32,       s390_vsrlb,         0,                  0,                  BT_OV_UV4SI_UV4SI_V4SI)
OB_DEF_VAR (s390_vec_srb_s32_u32,       s390_vsrlb,         0,                  0,                  BT_OV_V4SI_V4SI_UV4SI)
OB_DEF_VAR (s390_vec_srb_s32_s32,       s390_vsrlb,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_srb_u64_u64,       s390_vsrlb,         0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_srb_u64_s64,       s390_vsrlb,         0,                  0,                  BT_OV_UV2DI_UV2DI_V2DI)
OB_DEF_VAR (s390_vec_srb_s64_u64,       s390_vsrlb,         0,                  0,                  BT_OV_V2DI_V2DI_UV2DI)
OB_DEF_VAR (s390_vec_srb_s64_s64,       s390_vsrlb,         0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_srb_flt_u64,       s390_vsrlb,         B_VXE,              0,                  BT_OV_V4SF_V4SF_UV4SI)
OB_DEF_VAR (s390_vec_srb_dbl_u64,       s390_vsrlb,         0,                  0,                  BT_OV_V2DF_V2DF_UV2DI)
OB_DEF_VAR (s390_vec_srb_flt_s64,       s390_vsrlb,         B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SI)
OB_DEF_VAR (s390_vec_srb_dbl_s64,       s390_vsrlb,         0,                  0,                  BT_OV_V2DF_V2DF_V2DI)

B_DEF      (s390_vsrlb,                 vec_srbv16qi,       0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_subc,              s390_vec_subc_u8,   s390_vec_subc_u64,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_subc_u8,           s390_vscbib,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_subc_u16,          s390_vscbih,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_subc_u32,          s390_vscbif,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_subc_u64,          s390_vscbig,        0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)

B_DEF      (s390_vscbib,                vscbib_v16qi,       0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vscbih,                vscbih_v8hi,        0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vscbif,                vscbif_v4si,        0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vscbig,                vscbig_v2di,        0,                  B_VX,               0,                  BT_FN_UV2DI_UV2DI_UV2DI)

/* The builtin definitions requires these to use vector unsigned char.
   But we want the GCC low-level builtins and the insn patterns to
   allow int128_t and TImode.  So we rely on s390_expand_builtin to
   switch modes.  */

B_DEF      (s390_vec_sub_u128,          subti3,             0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vec_subc_u128,         vscbiq_ti,          0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vec_sube_u128,         vsbiq,              0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vec_subec_u128,        vsbcbiq,            0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_UV16QI)

B_DEF      (s390_vsq,                   subti3,             0,                  B_VX,               0,                  BT_FN_INT128_INT128_INT128)
B_DEF      (s390_vscbiq,                vscbiq_ti,          0,                  B_VX,               0,                  BT_FN_INT128_INT128_INT128)
B_DEF      (s390_vsbiq,                 vsbiq,              0,                  B_VX,               0,                  BT_FN_INT128_INT128_INT128_INT128)
B_DEF      (s390_vsbcbiq,               vsbcbiq,            0,                  B_VX,               0,                  BT_FN_INT128_INT128_INT128_INT128)

OB_DEF     (s390_vec_sum2,              s390_vec_sum2_u16,  s390_vec_sum2_u32,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_sum2_u16,          s390_vsumgh,        0,                  0,                  BT_OV_UV2DI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_sum2_u32,          s390_vsumgf,        0,                  0,                  BT_OV_UV2DI_UV4SI_UV4SI)

B_DEF      (s390_vsumgh,                vec_sum2v8hi,       0,                  B_VX,               0,                  BT_FN_UV2DI_UV8HI_UV8HI)
B_DEF      (s390_vsumgf,                vec_sum2v4si,       0,                  B_VX,               0,                  BT_FN_UV2DI_UV4SI_UV4SI)

OB_DEF     (s390_vec_sum_u128,          s390_vec_sum_u128_u32,s390_vec_sum_u128_u64,B_VX,           BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_sum_u128_u32,      s390_vsumqf,        0,                  0,                  BT_OV_UV16QI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_sum_u128_u64,      s390_vsumqg,        0,                  0,                  BT_OV_UV16QI_UV2DI_UV2DI)

B_DEF      (s390_vsumqf,                vec_sum_u128v4si,   0,                  B_VX,               0,                  BT_FN_UV16QI_UV4SI_UV4SI)
B_DEF      (s390_vsumqg,                vec_sum_u128v2di,   0,                  B_VX,               0,                  BT_FN_UV16QI_UV2DI_UV2DI)

OB_DEF     (s390_vec_sum4,              s390_vec_sum4_u8,   s390_vec_sum4_u16,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_sum4_u8,           s390_vsumb,         0,                  0,                  BT_OV_UV4SI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_sum4_u16,          s390_vsumh,         0,                  0,                  BT_OV_UV4SI_UV8HI_UV8HI)

B_DEF      (s390_vsumb,                 vec_sum4v16qi,      0,                  B_VX,               0,                  BT_FN_UV4SI_UV16QI_UV16QI)
B_DEF      (s390_vsumh,                 vec_sum4v8hi,       0,                  B_VX,               0,                  BT_FN_UV4SI_UV8HI_UV8HI)

OB_DEF     (s390_vec_test_mask,         s390_vec_test_mask_s8,s390_vec_test_mask_dbl,B_VX,          BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_test_mask_s8,      s390_vtm,           0,                  0,                  BT_OV_INT_V16QI_UV16QI)
OB_DEF_VAR (s390_vec_test_mask_u8,      s390_vtm,           0,                  0,                  BT_OV_INT_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_test_mask_s16,     s390_vtm,           0,                  0,                  BT_OV_INT_V8HI_UV8HI)
OB_DEF_VAR (s390_vec_test_mask_u16,     s390_vtm,           0,                  0,                  BT_OV_INT_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_test_mask_s32,     s390_vtm,           0,                  0,                  BT_OV_INT_V4SI_UV4SI)
OB_DEF_VAR (s390_vec_test_mask_u32,     s390_vtm,           0,                  0,                  BT_OV_INT_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_test_mask_s64,     s390_vtm,           0,                  0,                  BT_OV_INT_V2DI_UV2DI)
OB_DEF_VAR (s390_vec_test_mask_u64,     s390_vtm,           0,                  0,                  BT_OV_INT_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_test_mask_flt,     s390_vtm,           B_VXE,              0,                  BT_OV_INT_V4SF_UV4SI)
OB_DEF_VAR (s390_vec_test_mask_dbl,     s390_vtm,           0,                  0,                  BT_OV_INT_V2DF_UV2DI)

B_DEF      (s390_vtm,                   vec_test_mask_intv16qi,0,               B_VX,               0,                  BT_FN_INT_UV16QI_UV16QI)

B_DEF      (s390_vec_msum_u128,         vec_msumv2di,       0,                  B_VXE,              O4_U2,              BT_FN_UV16QI_UV2DI_UV2DI_UV16QI_INT)
B_DEF      (s390_vmslg,                 vmslg,              0,                  B_VXE,              O4_U2,              BT_FN_INT128_UV2DI_UV2DI_INT128_INT)

OB_DEF     (s390_vec_eqv,               s390_vec_eqv_b8,    s390_vec_eqv_dbl_c, B_VXE,              BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_eqv_b8,            s390_vnx,           0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_eqv_s8_a,          s390_vnx,           B_DEP,              0,                  BT_OV_V16QI_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_eqv_s8_b,          s390_vnx,           0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_eqv_s8_c,          s390_vnx,           B_DEP,              0,                  BT_OV_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_eqv_u8_a,          s390_vnx,           B_DEP,              0,                  BT_OV_UV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_eqv_u8_b,          s390_vnx,           0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_eqv_u8_c,          s390_vnx,           B_DEP,              0,                  BT_OV_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_eqv_b16,           s390_vnx,           0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_eqv_s16_a,         s390_vnx,           B_DEP,              0,                  BT_OV_V8HI_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_eqv_s16_b,         s390_vnx,           0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_eqv_s16_c,         s390_vnx,           B_DEP,              0,                  BT_OV_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_eqv_u16_a,         s390_vnx,           B_DEP,              0,                  BT_OV_UV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_eqv_u16_b,         s390_vnx,           0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_eqv_u16_c,         s390_vnx,           B_DEP,              0,                  BT_OV_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_eqv_b32,           s390_vnx,           0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_eqv_s32_a,         s390_vnx,           B_DEP,              0,                  BT_OV_V4SI_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_eqv_s32_b,         s390_vnx,           0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_eqv_s32_c,         s390_vnx,           B_DEP,              0,                  BT_OV_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_eqv_u32_a,         s390_vnx,           B_DEP,              0,                  BT_OV_UV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_eqv_u32_b,         s390_vnx,           0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_eqv_u32_c,         s390_vnx,           B_DEP,              0,                  BT_OV_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_eqv_b64,           s390_vnx,           0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_eqv_s64_a,         s390_vnx,           B_DEP,              0,                  BT_OV_V2DI_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_eqv_s64_b,         s390_vnx,           0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_eqv_s64_c,         s390_vnx,           B_DEP,              0,                  BT_OV_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_eqv_u64_a,         s390_vnx,           B_DEP,              0,                  BT_OV_UV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_eqv_u64_b,         s390_vnx,           0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_eqv_u64_c,         s390_vnx,           B_DEP,              0,                  BT_OV_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_eqv_flt_a,         s390_vnx,           B_VXE | B_DEP,      0,                  BT_OV_V4SF_BV4SI_V4SF)
OB_DEF_VAR (s390_vec_eqv_flt_b,         s390_vnx,           B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_eqv_flt_c,         s390_vnx,           B_VXE | B_DEP,      0,                  BT_OV_V4SF_V4SF_BV4SI)
OB_DEF_VAR (s390_vec_eqv_dbl_a,         s390_vnx,           B_DEP,              0,                  BT_OV_V2DF_BV2DI_V2DF)
OB_DEF_VAR (s390_vec_eqv_dbl_b,         s390_vnx,           0,                  0,                  BT_OV_V2DF_V2DF_V2DF)
OB_DEF_VAR (s390_vec_eqv_dbl_c,         s390_vnx,           B_DEP,              0,                  BT_OV_V2DF_V2DF_BV2DI)

B_DEF      (s390_vnx,                   notxorv16qi3,       0,                  B_VXE,              0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_nand,              s390_vec_nand_b8,   s390_vec_nand_dbl_c,B_VXE,              BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_nand_b8,           s390_vnn,           0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_nand_s8_a,         s390_vnn,           B_DEP,              0,                  BT_OV_V16QI_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_nand_s8_b,         s390_vnn,           0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_nand_s8_c,         s390_vnn,           B_DEP,              0,                  BT_OV_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_nand_u8_a,         s390_vnn,           B_DEP,              0,                  BT_OV_UV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_nand_u8_b,         s390_vnn,           0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_nand_u8_c,         s390_vnn,           B_DEP,              0,                  BT_OV_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_nand_b16,          s390_vnn,           0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_nand_s16_a,        s390_vnn,           B_DEP,              0,                  BT_OV_V8HI_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_nand_s16_b,        s390_vnn,           0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_nand_s16_c,        s390_vnn,           B_DEP,              0,                  BT_OV_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_nand_u16_a,        s390_vnn,           B_DEP,              0,                  BT_OV_UV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_nand_u16_b,        s390_vnn,           0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_nand_u16_c,        s390_vnn,           B_DEP,              0,                  BT_OV_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_nand_b32,          s390_vnn,           0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_nand_s32_a,        s390_vnn,           B_DEP,              0,                  BT_OV_V4SI_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_nand_s32_b,        s390_vnn,           0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_nand_s32_c,        s390_vnn,           B_DEP,              0,                  BT_OV_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_nand_u32_a,        s390_vnn,           B_DEP,              0,                  BT_OV_UV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_nand_u32_b,        s390_vnn,           0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_nand_u32_c,        s390_vnn,           B_DEP,              0,                  BT_OV_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_nand_b64,          s390_vnn,           0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_nand_s64_a,        s390_vnn,           B_DEP,              0,                  BT_OV_V2DI_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_nand_s64_b,        s390_vnn,           0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_nand_s64_c,        s390_vnn,           B_DEP,              0,                  BT_OV_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_nand_u64_a,        s390_vnn,           B_DEP,              0,                  BT_OV_UV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_nand_u64_b,        s390_vnn,           0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_nand_u64_c,        s390_vnn,           B_DEP,              0,                  BT_OV_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_nand_flt_a,        s390_vnn,           B_DEP,              0,                  BT_OV_V4SF_BV4SI_V4SF)
OB_DEF_VAR (s390_vec_nand_flt_b,        s390_vnn,           0,                  0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_nand_flt_c,        s390_vnn,           B_DEP,              0,                  BT_OV_V4SF_V4SF_BV4SI)
OB_DEF_VAR (s390_vec_nand_dbl_a,        s390_vnn,           B_DEP,              0,                  BT_OV_V2DF_BV2DI_V2DF)
OB_DEF_VAR (s390_vec_nand_dbl_b,        s390_vnn,           0,                  0,                  BT_OV_V2DF_V2DF_V2DF)
OB_DEF_VAR (s390_vec_nand_dbl_c,        s390_vnn,           B_DEP,              0,                  BT_OV_V2DF_V2DF_BV2DI)

B_DEF      (s390_vnn,                   notandv16qi3,       0,                  B_VXE,              0,                  BT_FN_UV16QI_UV16QI_UV16QI)

OB_DEF     (s390_vec_orc,               s390_vec_orc_b8,    s390_vec_orc_dbl_c, B_VXE,              BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_orc_b8,            s390_voc,           0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vec_orc_s8_a,          s390_voc,           B_DEP,              0,                  BT_OV_V16QI_BV16QI_V16QI)
OB_DEF_VAR (s390_vec_orc_s8_b,          s390_voc,           0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vec_orc_s8_c,          s390_voc,           B_DEP,              0,                  BT_OV_V16QI_V16QI_BV16QI)
OB_DEF_VAR (s390_vec_orc_u8_a,          s390_voc,           B_DEP,              0,                  BT_OV_UV16QI_BV16QI_UV16QI)
OB_DEF_VAR (s390_vec_orc_u8_b,          s390_voc,           0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vec_orc_u8_c,          s390_voc,           B_DEP,              0,                  BT_OV_UV16QI_UV16QI_BV16QI)
OB_DEF_VAR (s390_vec_orc_b16,           s390_voc,           0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vec_orc_s16_a,         s390_voc,           B_DEP,              0,                  BT_OV_V8HI_BV8HI_V8HI)
OB_DEF_VAR (s390_vec_orc_s16_b,         s390_voc,           0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vec_orc_s16_c,         s390_voc,           B_DEP,              0,                  BT_OV_V8HI_V8HI_BV8HI)
OB_DEF_VAR (s390_vec_orc_u16_a,         s390_voc,           B_DEP,              0,                  BT_OV_UV8HI_BV8HI_UV8HI)
OB_DEF_VAR (s390_vec_orc_u16_b,         s390_voc,           0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vec_orc_u16_c,         s390_voc,           B_DEP,              0,                  BT_OV_UV8HI_UV8HI_BV8HI)
OB_DEF_VAR (s390_vec_orc_b32,           s390_voc,           0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vec_orc_s32_a,         s390_voc,           B_DEP,              0,                  BT_OV_V4SI_BV4SI_V4SI)
OB_DEF_VAR (s390_vec_orc_s32_b,         s390_voc,           0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vec_orc_s32_c,         s390_voc,           B_DEP,              0,                  BT_OV_V4SI_V4SI_BV4SI)
OB_DEF_VAR (s390_vec_orc_u32_a,         s390_voc,           B_DEP,              0,                  BT_OV_UV4SI_BV4SI_UV4SI)
OB_DEF_VAR (s390_vec_orc_u32_b,         s390_voc,           0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)
OB_DEF_VAR (s390_vec_orc_u32_c,         s390_voc,           B_DEP,              0,                  BT_OV_UV4SI_UV4SI_BV4SI)
OB_DEF_VAR (s390_vec_orc_b64,           s390_voc,           0,                  0,                  BT_OV_BV2DI_BV2DI_BV2DI)
OB_DEF_VAR (s390_vec_orc_s64_a,         s390_voc,           B_DEP,              0,                  BT_OV_V2DI_BV2DI_V2DI)
OB_DEF_VAR (s390_vec_orc_s64_b,         s390_voc,           0,                  0,                  BT_OV_V2DI_V2DI_V2DI)
OB_DEF_VAR (s390_vec_orc_s64_c,         s390_voc,           B_DEP,              0,                  BT_OV_V2DI_V2DI_BV2DI)
OB_DEF_VAR (s390_vec_orc_u64_a,         s390_voc,           B_DEP,              0,                  BT_OV_UV2DI_BV2DI_UV2DI)
OB_DEF_VAR (s390_vec_orc_u64_b,         s390_voc,           0,                  0,                  BT_OV_UV2DI_UV2DI_UV2DI)
OB_DEF_VAR (s390_vec_orc_u64_c,         s390_voc,           B_DEP,              0,                  BT_OV_UV2DI_UV2DI_BV2DI)
OB_DEF_VAR (s390_vec_orc_flt_a,         s390_voc,           B_DEP,              0,                  BT_OV_V4SF_BV4SI_V4SF)
OB_DEF_VAR (s390_vec_orc_flt_b,         s390_voc,           0,                  0,                  BT_OV_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_orc_flt_c,         s390_voc,           B_DEP,              0,                  BT_OV_V4SF_V4SF_BV4SI)
OB_DEF_VAR (s390_vec_orc_dbl_a,         s390_voc,           B_DEP,              0,                  BT_OV_V2DF_BV2DI_V2DF)
OB_DEF_VAR (s390_vec_orc_dbl_b,         s390_voc,           0,                  0,                  BT_OV_V2DF_V2DF_V2DF)
OB_DEF_VAR (s390_vec_orc_dbl_c,         s390_voc,           B_DEP,              0,                  BT_OV_V2DF_V2DF_BV2DI)

B_DEF      (s390_voc,                   ior_notv16qi3,      0,                  B_VXE,              0,                  BT_FN_UV16QI_UV16QI_UV16QI)

B_DEF      (s390_vfaeb,                 vfaev16qi,          0,                  B_VX,               O3_U4,              BT_FN_UV16QI_UV16QI_UV16QI_INT)
B_DEF      (s390_vfaeh,                 vfaev8hi,           0,                  B_VX,               O3_U4,              BT_FN_UV8HI_UV8HI_UV8HI_INT)
B_DEF      (s390_vfaef,                 vfaev4si,           0,                  B_VX,               O3_U4,              BT_FN_UV4SI_UV4SI_UV4SI_INT)
B_DEF      (s390_vfaezb,                vfaezv16qi,         0,                  B_VX,               O3_U4,              BT_FN_UV16QI_UV16QI_UV16QI_INT)
B_DEF      (s390_vfaezh,                vfaezv8hi,          0,                  B_VX,               O3_U4,              BT_FN_UV8HI_UV8HI_UV8HI_INT)
B_DEF      (s390_vfaezf,                vfaezv4si,          0,                  B_VX,               O3_U4,              BT_FN_UV4SI_UV4SI_UV4SI_INT)
B_DEF      (s390_vfaebs,                vfaesv16qi,         0,                  B_VX,               O3_U4,              BT_FN_UV16QI_UV16QI_UV16QI_INT_INTPTR)
B_DEF      (s390_vfaehs,                vfaesv8hi,          0,                  B_VX,               O3_U4,              BT_FN_UV8HI_UV8HI_UV8HI_INT_INTPTR)
B_DEF      (s390_vfaefs,                vfaesv4si,          0,                  B_VX,               O3_U4,              BT_FN_UV4SI_UV4SI_UV4SI_INT_INTPTR)
B_DEF      (s390_vfaezbs,               vfaezsv16qi,        0,                  B_VX,               O3_U4,              BT_FN_UV16QI_UV16QI_UV16QI_INT_INTPTR)
B_DEF      (s390_vfaezhs,               vfaezsv8hi,         0,                  B_VX,               O3_U4,              BT_FN_UV8HI_UV8HI_UV8HI_INT_INTPTR)
B_DEF      (s390_vfaezfs,               vfaezsv4si,         0,                  B_VX,               O3_U4,              BT_FN_UV4SI_UV4SI_UV4SI_INT_INTPTR)

OB_DEF     (s390_vec_find_any_eq_idx,   s390_vfaeb_idx_s8,  s390_vfaef_idx_u32b,B_VX,               BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vfaeb_idx_s8,          s390_vfaeb,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vfaeb_idx_u8a,         s390_vfaeb,         0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vfaeb_idx_u8b,         s390_vfaeb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vfaeh_idx_s16,         s390_vfaeh,         0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vfaeh_idx_u16a,        s390_vfaeh,         0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vfaeh_idx_u16b,        s390_vfaeh,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vfaef_idx_s32,         s390_vfaef,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vfaef_idx_u32a,        s390_vfaef,         0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vfaef_idx_u32b,        s390_vfaef,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_find_any_ne_idx,   s390_vfaeb_inv_idx_s8,s390_vfaef_inv_idx_u32b,B_VX,         BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vfaeb_inv_idx_s8,      s390_vfaeb,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vfaeb_inv_idx_u8a,     s390_vfaeb,         0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vfaeb_inv_idx_u8b,     s390_vfaeb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vfaeh_inv_idx_s16,     s390_vfaeh,         0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vfaeh_inv_idx_u16a,    s390_vfaeh,         0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vfaeh_inv_idx_u16b,    s390_vfaeh,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vfaef_inv_idx_s32,     s390_vfaef,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vfaef_inv_idx_u32a,    s390_vfaef,         0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vfaef_inv_idx_u32b,    s390_vfaef,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_find_any_eq_or_0_idx,s390_vfaezb_idx_s8,s390_vfaezf_idx_u32b,B_VX,             BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vfaezb_idx_s8,         s390_vfaezb,        0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vfaezb_idx_u8a,        s390_vfaezb,        0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vfaezb_idx_u8b,        s390_vfaezb,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vfaezh_idx_s16,        s390_vfaezh,        0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vfaezh_idx_u16a,       s390_vfaezh,        0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vfaezh_idx_u16b,       s390_vfaezh,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vfaezf_idx_s32,        s390_vfaezf,        0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vfaezf_idx_u32a,       s390_vfaezf,        0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vfaezf_idx_u32b,       s390_vfaezf,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_find_any_ne_or_0_idx,s390_vfaezb_inv_idx_s8,s390_vfaezf_inv_idx_u32b,B_VX,     BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vfaezb_inv_idx_s8,     s390_vfaezb,        0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vfaezb_inv_idx_u8a,    s390_vfaezb,        0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vfaezb_inv_idx_u8b,    s390_vfaezb,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vfaezh_inv_idx_s16,    s390_vfaezh,        0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vfaezh_inv_idx_u16a,   s390_vfaezh,        0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vfaezh_inv_idx_u16b,   s390_vfaezh,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vfaezf_inv_idx_s32,    s390_vfaezf,        0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vfaezf_inv_idx_u32a,   s390_vfaezf,        0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vfaezf_inv_idx_u32b,   s390_vfaezf,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_find_any_eq,       s390_vfaeb_s8,      s390_vfaef_b32,     B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vfaeb_s8,              s390_vfaeb,         0,                  0,                  BT_OV_BV16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vfaeb_u8,              s390_vfaeb,         0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vfaeb_b8,              s390_vfaeb,         0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vfaeh_s16,             s390_vfaeh,         0,                  0,                  BT_OV_BV8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vfaeh_u16,             s390_vfaeh,         0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vfaeh_b16,             s390_vfaeh,         0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vfaef_s32,             s390_vfaef,         0,                  0,                  BT_OV_BV4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vfaef_u32,             s390_vfaef,         0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vfaef_b32,             s390_vfaef,         0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_find_any_ne,       s390_vfaeb_inv_s8,  s390_vfaef_inv_b32, B_VX,               BT_FN_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vfaeb_inv_s8,          s390_vfaeb,         0,                  0,                  BT_OV_BV16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vfaeb_inv_u8,          s390_vfaeb,         0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vfaeb_inv_b8,          s390_vfaeb,         0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vfaeh_inv_s16,         s390_vfaeh,         0,                  0,                  BT_OV_BV8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vfaeh_inv_u16,         s390_vfaeh,         0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vfaeh_inv_b16,         s390_vfaeh,         0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vfaef_inv_s32,         s390_vfaef,         0,                  0,                  BT_OV_BV4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vfaef_inv_u32,         s390_vfaef,         0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vfaef_inv_b32,         s390_vfaef,         0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_find_any_eq_idx_cc,s390_vfaebs_idx_s8, s390_vfaefs_idx_u32b,B_VX,              BT_FN_INT_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vfaebs_idx_s8,         s390_vfaebs,        0,                  0,                  BT_OV_V16QI_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vfaebs_idx_u8a,        s390_vfaebs,        0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vfaebs_idx_u8b,        s390_vfaebs,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vfaehs_idx_s16,        s390_vfaehs,        0,                  0,                  BT_OV_V8HI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vfaehs_idx_u16a,       s390_vfaehs,        0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vfaehs_idx_u16b,       s390_vfaehs,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vfaefs_idx_s32,        s390_vfaefs,        0,                  0,                  BT_OV_V4SI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vfaefs_idx_u32a,       s390_vfaefs,        0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vfaefs_idx_u32b,       s390_vfaefs,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_find_any_ne_idx_cc,s390_vfaebs_inv_idx_s8,s390_vfaefs_inv_idx_u32b,B_VX,       BT_FN_INT_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vfaebs_inv_idx_s8,     s390_vfaebs,        0,                  0,                  BT_OV_V16QI_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vfaebs_inv_idx_u8a,    s390_vfaebs,        0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vfaebs_inv_idx_u8b,    s390_vfaebs,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vfaehs_inv_idx_s16,    s390_vfaehs,        0,                  0,                  BT_OV_V8HI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vfaehs_inv_idx_u16a,   s390_vfaehs,        0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vfaehs_inv_idx_u16b,   s390_vfaehs,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vfaefs_inv_idx_s32,    s390_vfaefs,        0,                  0,                  BT_OV_V4SI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vfaefs_inv_idx_u32a,   s390_vfaefs,        0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vfaefs_inv_idx_u32b,   s390_vfaefs,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_find_any_eq_or_0_idx_cc,s390_vfaezbs_idx_s8,s390_vfaezfs_idx_u32b,B_VX,        BT_FN_INT_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vfaezbs_idx_s8,        s390_vfaezbs,       0,                  0,                  BT_OV_V16QI_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vfaezbs_idx_u8a,       s390_vfaezbs,       0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vfaezbs_idx_u8b,       s390_vfaezbs,       0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vfaezhs_idx_s16,       s390_vfaezhs,       0,                  0,                  BT_OV_V8HI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vfaezhs_idx_u16a,      s390_vfaezhs,       0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vfaezhs_idx_u16b,      s390_vfaezhs,       0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vfaezfs_idx_s32,       s390_vfaezfs,       0,                  0,                  BT_OV_V4SI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vfaezfs_idx_u32a,      s390_vfaezfs,       0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vfaezfs_idx_u32b,      s390_vfaezfs,       0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_find_any_ne_or_0_idx_cc,s390_vfaezbs_inv_idx_s8,s390_vfaezfs_inv_idx_u32b,B_VX,BT_FN_INT_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vfaezbs_inv_idx_s8,    s390_vfaezbs,       0,                  0,                  BT_OV_V16QI_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vfaezbs_inv_idx_u8a,   s390_vfaezbs,       0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vfaezbs_inv_idx_u8b,   s390_vfaezbs,       0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vfaezhs_inv_idx_s16,   s390_vfaezhs,       0,                  0,                  BT_OV_V8HI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vfaezhs_inv_idx_u16a,  s390_vfaezhs,       0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vfaezhs_inv_idx_u16b,  s390_vfaezhs,       0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vfaezfs_inv_idx_s32,   s390_vfaezfs,       0,                  0,                  BT_OV_V4SI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vfaezfs_inv_idx_u32a,  s390_vfaezfs,       0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vfaezfs_inv_idx_u32b,  s390_vfaezfs,       0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_find_any_eq_cc,    s390_vfaebs_s8,     s390_vfaefs_b32,    B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vfaebs_s8,             s390_vfaebs,        0,                  0,                  BT_OV_BV16QI_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vfaebs_u8,             s390_vfaebs,        0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vfaebs_b8,             s390_vfaebs,        0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vfaehs_s16,            s390_vfaehs,        0,                  0,                  BT_OV_BV8HI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vfaehs_u16,            s390_vfaehs,        0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vfaehs_b16,            s390_vfaehs,        0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vfaefs_s32,            s390_vfaefs,        0,                  0,                  BT_OV_BV4SI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vfaefs_u32,            s390_vfaefs,        0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vfaefs_b32,            s390_vfaefs,        0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_find_any_ne_cc,    s390_vfaebs_inv_s8, s390_vfaefs_inv_b32,B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vfaebs_inv_s8,         s390_vfaebs,        0,                  0,                  BT_OV_BV16QI_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vfaebs_inv_u8,         s390_vfaebs,        0,                  0,                  BT_OV_BV16QI_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vfaebs_inv_b8,         s390_vfaebs,        0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vfaehs_inv_s16,        s390_vfaehs,        0,                  0,                  BT_OV_BV8HI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vfaehs_inv_u16,        s390_vfaehs,        0,                  0,                  BT_OV_BV8HI_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vfaehs_inv_b16,        s390_vfaehs,        0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vfaefs_inv_s32,        s390_vfaefs,        0,                  0,                  BT_OV_BV4SI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vfaefs_inv_u32,        s390_vfaefs,        0,                  0,                  BT_OV_BV4SI_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vfaefs_inv_b32,        s390_vfaefs,        0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI_INTPTR)

B_DEF      (s390_vfeeb,                 vfeev16qi,          0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vfeeh,                 vfeev8hi,           0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vfeef,                 vfeev4si,           0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vfeezb,                vfeezv16qi,         0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vfeezh,                vfeezv8hi,          0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vfeezf,                vfeezv4si,          0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vfeebs,                vfeesv16qi,         0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_INTPTR)
B_DEF      (s390_vfeehs,                vfeesv8hi,          0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI_INTPTR)
B_DEF      (s390_vfeefs,                vfeesv4si,          0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI_INTPTR)
B_DEF      (s390_vfeezbs,               vfeezsv16qi,        0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_INTPTR)
B_DEF      (s390_vfeezhs,               vfeezsv8hi,         0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI_INTPTR)
B_DEF      (s390_vfeezfs,               vfeezsv4si,         0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_cmpeq_idx,         s390_vfeeb_s8,      s390_vfeef_u32b,    B_VX,               BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vfeeb_s8,              s390_vfeeb,         0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vfeeb_u8a,             s390_vfeeb,         0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vfeeb_u8b,             s390_vfeeb,         0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vfeeh_s16,             s390_vfeeh,         0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vfeeh_u16a,            s390_vfeeh,         0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vfeeh_u16b,            s390_vfeeh,         0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vfeef_s32,             s390_vfeef,         0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vfeef_u32a,            s390_vfeef,         0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vfeef_u32b,            s390_vfeef,         0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_cmpeq_or_0_idx,    s390_vfeezb_s8,     s390_vfeezf_u32b,   B_VX,               BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vfeezb_s8,             s390_vfeezb,        0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vfeezb_u8a,            s390_vfeezb,        0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vfeezb_u8b,            s390_vfeezb,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vfeezh_s16,            s390_vfeezh,        0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vfeezh_u16a,           s390_vfeezh,        0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vfeezh_u16b,           s390_vfeezh,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vfeezf_s32,            s390_vfeezf,        0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vfeezf_u32a,           s390_vfeezf,        0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vfeezf_u32b,           s390_vfeezf,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_cmpeq_idx_cc,      s390_vfeebs_s8,     s390_vfeefs_u32b,   B_VX,               BT_FN_INT_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vfeebs_s8,             s390_vfeebs,        0,                  0,                  BT_OV_V16QI_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vfeebs_u8a,            s390_vfeebs,        0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vfeebs_u8b,            s390_vfeebs,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vfeehs_s16,            s390_vfeehs,        0,                  0,                  BT_OV_V8HI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vfeehs_u16a,           s390_vfeehs,        0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vfeehs_u16b,           s390_vfeehs,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vfeefs_s32,            s390_vfeefs,        0,                  0,                  BT_OV_V4SI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vfeefs_u32a,           s390_vfeefs,        0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vfeefs_u32b,           s390_vfeefs,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_cmpeq_or_0_idx_cc, s390_vfeezbs_s8,    s390_vfeezfs_u32b,  B_VX,               BT_FN_INT_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vfeezbs_s8,            s390_vfeezbs,       0,                  0,                  BT_OV_V16QI_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vfeezbs_u8a,           s390_vfeezbs,       0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vfeezbs_u8b,           s390_vfeezbs,       0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vfeezhs_s16,           s390_vfeezhs,       0,                  0,                  BT_OV_V8HI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vfeezhs_u16a,          s390_vfeezhs,       0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vfeezhs_u16b,          s390_vfeezhs,       0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vfeezfs_s32,           s390_vfeezfs,       0,                  0,                  BT_OV_V4SI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vfeezfs_u32a,          s390_vfeezfs,       0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vfeezfs_u32b,          s390_vfeezfs,       0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_INTPTR)

B_DEF      (s390_vfeneb,                vfenev16qi,         0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vfeneh,                vfenev8hi,          0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vfenef,                vfenev4si,          0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vfenezb,               vfenezv16qi,        0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI)
B_DEF      (s390_vfenezh,               vfenezv8hi,         0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI)
B_DEF      (s390_vfenezf,               vfenezv4si,         0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI)
B_DEF      (s390_vfenebs,               vfenesv16qi,        0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_INTPTR)
B_DEF      (s390_vfenehs,               vfenesv8hi,         0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI_INTPTR)
B_DEF      (s390_vfenefs,               vfenesv4si,         0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI_INTPTR)
B_DEF      (s390_vfenezbs,              vfenezsv16qi,       0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_UV16QI_INTPTR)
B_DEF      (s390_vfenezhs,              vfenezsv8hi,        0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_UV8HI_INTPTR)
B_DEF      (s390_vfenezfs,              vfenezsv4si,        0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_cmpne_idx,         s390_vfeneb_s8,     s390_vfenef_u32b,   B_VX,               BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vfeneb_s8,             s390_vfeneb,        0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vfeneb_u8a,            s390_vfeneb,        0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vfeneb_u8b,            s390_vfeneb,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vfeneh_s16,            s390_vfeneh,        0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vfeneh_u16a,           s390_vfeneh,        0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vfeneh_u16b,           s390_vfeneh,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vfenef_s32,            s390_vfenef,        0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vfenef_u32a,           s390_vfenef,        0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vfenef_u32b,           s390_vfenef,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_cmpne_or_0_idx,    s390_vfenezb_s8,    s390_vfenezf_u32b,  B_VX,               BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vfenezb_s8,            s390_vfenezb,       0,                  0,                  BT_OV_V16QI_V16QI_V16QI)
OB_DEF_VAR (s390_vfenezb_u8a,           s390_vfenezb,       0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI)
OB_DEF_VAR (s390_vfenezb_u8b,           s390_vfenezb,       0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vfenezh_s16,           s390_vfenezh,       0,                  0,                  BT_OV_V8HI_V8HI_V8HI)
OB_DEF_VAR (s390_vfenezh_u16a,          s390_vfenezh,       0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI)
OB_DEF_VAR (s390_vfenezh_u16b,          s390_vfenezh,       0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vfenezf_s32,           s390_vfenezf,       0,                  0,                  BT_OV_V4SI_V4SI_V4SI)
OB_DEF_VAR (s390_vfenezf_u32a,          s390_vfenezf,       0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI)
OB_DEF_VAR (s390_vfenezf_u32b,          s390_vfenezf,       0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_cmpne_idx_cc,      s390_vfenebs_s8,    s390_vfenefs_u32b,  B_VX,               BT_FN_INT_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vfenebs_s8,            s390_vfenebs,       0,                  0,                  BT_OV_V16QI_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vfenebs_u8a,           s390_vfenebs,       0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vfenebs_u8b,           s390_vfenebs,       0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vfenehs_s16,           s390_vfenehs,       0,                  0,                  BT_OV_V8HI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vfenehs_u16a,          s390_vfenehs,       0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vfenehs_u16b,          s390_vfenehs,       0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vfenefs_s32,           s390_vfenefs,       0,                  0,                  BT_OV_V4SI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vfenefs_u32a,          s390_vfenefs,       0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vfenefs_u32b,          s390_vfenefs,       0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_cmpne_or_0_idx_cc, s390_vfenezbs_s8,   s390_vfenezfs_u32b, B_VX,               BT_FN_INT_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vfenezbs_s8,           s390_vfenezbs,      0,                  0,                  BT_OV_V16QI_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vfenezbs_u8a,          s390_vfenezbs,      0,                  0,                  BT_OV_UV16QI_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vfenezbs_u8b,          s390_vfenezbs,      0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vfenezhs_s16,          s390_vfenezhs,      0,                  0,                  BT_OV_V8HI_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vfenezhs_u16a,         s390_vfenezhs,      0,                  0,                  BT_OV_UV8HI_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vfenezhs_u16b,         s390_vfenezhs,      0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vfenezfs_s32,          s390_vfenezfs,      0,                  0,                  BT_OV_V4SI_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vfenezfs_u32a,         s390_vfenezfs,      0,                  0,                  BT_OV_UV4SI_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vfenezfs_u32b,         s390_vfenezfs,      0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_INTPTR)

B_DEF      (s390_vistrb,                vistrv16qi,         0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI)
B_DEF      (s390_vistrh,                vistrv8hi,          0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI)
B_DEF      (s390_vistrf,                vistrv4si,          0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI)
B_DEF      (s390_vistrbs,               vistrsv16qi,        0,                  B_VX,               0,                  BT_FN_UV16QI_UV16QI_INTPTR)
B_DEF      (s390_vistrhs,               vistrsv8hi,         0,                  B_VX,               0,                  BT_FN_UV8HI_UV8HI_INTPTR)
B_DEF      (s390_vistrfs,               vistrsv4si,         0,                  B_VX,               0,                  BT_FN_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_cp_until_zero,     s390_vistrb_s8,     s390_vistrf_u32,    B_VX,               BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vistrb_s8,             s390_vistrb,        0,                  0,                  BT_OV_V16QI_V16QI)
OB_DEF_VAR (s390_vistrb_b8,             s390_vistrb,        0,                  0,                  BT_OV_BV16QI_BV16QI)
OB_DEF_VAR (s390_vistrb_u8,             s390_vistrb,        0,                  0,                  BT_OV_UV16QI_UV16QI)
OB_DEF_VAR (s390_vistrh_s16,            s390_vistrh,        0,                  0,                  BT_OV_V8HI_V8HI)
OB_DEF_VAR (s390_vistrh_b16,            s390_vistrh,        0,                  0,                  BT_OV_BV8HI_BV8HI)
OB_DEF_VAR (s390_vistrh_u16,            s390_vistrh,        0,                  0,                  BT_OV_UV8HI_UV8HI)
OB_DEF_VAR (s390_vistrf_s32,            s390_vistrf,        0,                  0,                  BT_OV_V4SI_V4SI)
OB_DEF_VAR (s390_vistrf_b32,            s390_vistrf,        0,                  0,                  BT_OV_BV4SI_BV4SI)
OB_DEF_VAR (s390_vistrf_u32,            s390_vistrf,        0,                  0,                  BT_OV_UV4SI_UV4SI)

OB_DEF     (s390_vec_cp_until_zero_cc,  s390_vistrbs_s8,    s390_vistrfs_u32,   B_VX,               BT_FN_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vistrbs_s8,            s390_vistrbs,       0,                  0,                  BT_OV_V16QI_V16QI_INTPTR)
OB_DEF_VAR (s390_vistrbs_b8,            s390_vistrbs,       0,                  0,                  BT_OV_BV16QI_BV16QI_INTPTR)
OB_DEF_VAR (s390_vistrbs_u8,            s390_vistrbs,       0,                  0,                  BT_OV_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vistrhs_s16,           s390_vistrhs,       0,                  0,                  BT_OV_V8HI_V8HI_INTPTR)
OB_DEF_VAR (s390_vistrhs_b16,           s390_vistrhs,       0,                  0,                  BT_OV_BV8HI_BV8HI_INTPTR)
OB_DEF_VAR (s390_vistrhs_u16,           s390_vistrhs,       0,                  0,                  BT_OV_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vistrfs_s32,           s390_vistrfs,       0,                  0,                  BT_OV_V4SI_V4SI_INTPTR)
OB_DEF_VAR (s390_vistrfs_b32,           s390_vistrfs,       0,                  0,                  BT_OV_BV4SI_BV4SI_INTPTR)
OB_DEF_VAR (s390_vistrfs_u32,           s390_vistrfs,       0,                  0,                  BT_OV_UV4SI_UV4SI_INTPTR)

B_DEF      (s390_vstrcb,                vstrcv16qi,         0,                  B_VX,               O4_U4,              BT_FN_UV16QI_UV16QI_UV16QI_UV16QI_INT)
B_DEF      (s390_vstrch,                vstrcv8hi,          0,                  B_VX,               O4_U4,              BT_FN_UV8HI_UV8HI_UV8HI_UV8HI_INT)
B_DEF      (s390_vstrcf,                vstrcv4si,          0,                  B_VX,               O4_U4,              BT_FN_UV4SI_UV4SI_UV4SI_UV4SI_INT)
B_DEF      (s390_vstrczb,               vstrczv16qi,        0,                  B_VX,               O4_U4,              BT_FN_UV16QI_UV16QI_UV16QI_UV16QI_INT)
B_DEF      (s390_vstrczh,               vstrczv8hi,         0,                  B_VX,               O4_U4,              BT_FN_UV8HI_UV8HI_UV8HI_UV8HI_INT)
B_DEF      (s390_vstrczf,               vstrczv4si,         0,                  B_VX,               O4_U4,              BT_FN_UV4SI_UV4SI_UV4SI_UV4SI_INT)
B_DEF      (s390_vstrcbs,               vstrcsv16qi,        0,                  B_VX,               O4_U4,              BT_FN_UV16QI_UV16QI_UV16QI_UV16QI_INT_INTPTR)
B_DEF      (s390_vstrchs,               vstrcsv8hi,         0,                  B_VX,               O4_U4,              BT_FN_UV8HI_UV8HI_UV8HI_UV8HI_INT_INTPTR)
B_DEF      (s390_vstrcfs,               vstrcsv4si,         0,                  B_VX,               O4_U4,              BT_FN_UV4SI_UV4SI_UV4SI_UV4SI_INT_INTPTR)
B_DEF      (s390_vstrczbs,              vstrczsv16qi,       0,                  B_VX,               O4_U4,              BT_FN_UV16QI_UV16QI_UV16QI_UV16QI_INT_INTPTR)
B_DEF      (s390_vstrczhs,              vstrczsv8hi,        0,                  B_VX,               O4_U4,              BT_FN_UV8HI_UV8HI_UV8HI_UV8HI_INT_INTPTR)
B_DEF      (s390_vstrczfs,              vstrczsv4si,        0,                  B_VX,               O4_U4,              BT_FN_UV4SI_UV4SI_UV4SI_UV4SI_INT_INTPTR)

OB_DEF     (s390_vec_cmprg_idx,         s390_vstrcb_idx_u8, s390_vstrcf_idx_u32,B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vstrcb_idx_u8,         s390_vstrcb,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vstrch_idx_u16,        s390_vstrch,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vstrcf_idx_u32,        s390_vstrcf,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_cmpnrg_idx,        s390_vstrcb_inv_idx_u8,s390_vstrcf_inv_idx_u32,B_VX,        BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vstrcb_inv_idx_u8,     s390_vstrcb,        0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vstrch_inv_idx_u16,    s390_vstrch,        0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vstrcf_inv_idx_u32,    s390_vstrcf,        0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_cmprg_or_0_idx,    s390_vstrczb_idx_u8,s390_vstrczf_idx_u32,B_VX,              BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vstrczb_idx_u8,        s390_vstrczb,       0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vstrczh_idx_u16,       s390_vstrczh,       0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vstrczf_idx_u32,       s390_vstrczf,       0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_cmpnrg_or_0_idx,   s390_vstrczb_inv_idx_u8,s390_vstrczf_inv_idx_u32,B_VX,      BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vstrczb_inv_idx_u8,    s390_vstrczb,       0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vstrczh_inv_idx_u16,   s390_vstrczh,       0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vstrczf_inv_idx_u32,   s390_vstrczf,       0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_cmprg,             s390_vstrcb_u8,     s390_vstrcf_u32,    B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vstrcb_u8,             s390_vstrcb,        0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vstrch_u16,            s390_vstrch,        0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vstrcf_u32,            s390_vstrcf,        0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_cmpnrg,            s390_vstrcb_inv_u8, s390_vstrcf_inv_u32,B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vstrcb_inv_u8,         s390_vstrcb,        0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI_UV16QI)
OB_DEF_VAR (s390_vstrch_inv_u16,        s390_vstrch,        0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI_UV8HI)
OB_DEF_VAR (s390_vstrcf_inv_u32,        s390_vstrcf,        0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI_UV4SI)

OB_DEF     (s390_vec_cmprg_idx_cc,      s390_vstrcbs_idx_u8,s390_vstrcfs_idx_u32,B_VX,              BT_FN_OV4SI_OV4SI_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vstrcbs_idx_u8,        s390_vstrcbs,       0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vstrchs_idx_u16,       s390_vstrchs,       0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vstrcfs_idx_u32,       s390_vstrcfs,       0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_cmpnrg_idx_cc,     s390_vstrcbs_inv_idx_u8,s390_vstrcfs_inv_idx_u32,B_VX,      BT_FN_OV4SI_OV4SI_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vstrcbs_inv_idx_u8,    s390_vstrcbs,       0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI_INTPTR) /* vstrcb */
OB_DEF_VAR (s390_vstrchs_inv_idx_u16,   s390_vstrchs,       0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI_INTPTR)    /* vstrch */
OB_DEF_VAR (s390_vstrcfs_inv_idx_u32,   s390_vstrcfs,       0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI_INTPTR)    /* vstrcf */

OB_DEF     (s390_vec_cmprg_or_0_idx_cc, s390_vstrczbs_idx_u8,s390_vstrczfs_idx_u32,B_VX,            BT_FN_OV4SI_OV4SI_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vstrczbs_idx_u8,       s390_vstrczbs,      0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vstrczhs_idx_u16,      s390_vstrczhs,      0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vstrczfs_idx_u32,      s390_vstrczfs,      0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_cmpnrg_or_0_idx_cc,s390_vstrczbs_inv_idx_u8,s390_vstrczfs_inv_idx_u32,B_VX,    BT_FN_OV4SI_OV4SI_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vstrczbs_inv_idx_u8,   s390_vstrczbs,      0,                  0,                  BT_OV_UV16QI_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vstrczhs_inv_idx_u16,  s390_vstrczhs,      0,                  0,                  BT_OV_UV8HI_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vstrczfs_inv_idx_u32,  s390_vstrczfs,      0,                  0,                  BT_OV_UV4SI_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_cmprg_cc,          s390_vstrcbs_u8,    s390_vstrcfs_u32,   B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vstrcbs_u8,            s390_vstrcbs,       0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vstrchs_u16,           s390_vstrchs,       0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vstrcfs_u32,           s390_vstrcfs,       0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI_UV4SI_INTPTR)

OB_DEF     (s390_vec_cmpnrg_cc,         s390_vstrcbs_inv_u8,s390_vstrcfs_inv_u32,B_VX,              BT_FN_OV4SI_OV4SI_OV4SI_OV4SI_INTPTR)
OB_DEF_VAR (s390_vstrcbs_inv_u8,        s390_vstrcbs,       0,                  0,                  BT_OV_BV16QI_UV16QI_UV16QI_UV16QI_INTPTR)
OB_DEF_VAR (s390_vstrchs_inv_u16,       s390_vstrchs,       0,                  0,                  BT_OV_BV8HI_UV8HI_UV8HI_UV8HI_INTPTR)
OB_DEF_VAR (s390_vstrcfs_inv_u32,       s390_vstrcfs,       0,                  0,                  BT_OV_BV4SI_UV4SI_UV4SI_UV4SI_INTPTR)

B_DEF      (vec_all_unltv4sf,           vec_all_unltv4sf,   0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_all_unltv2df,           vec_all_unltv2df,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)
B_DEF      (vec_all_unlev4sf,           vec_all_unlev4sf,   0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_all_unlev2df,           vec_all_unlev2df,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

B_DEF      (vec_any_unltv4sf,           vec_any_unltv4sf,   0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_any_unltv2df,           vec_any_unltv2df,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)
B_DEF      (vec_any_unlev4sf,           vec_any_unlev4sf,   0,                  B_INT | B_VXE,      0,                  BT_FN_INT_V4SF_V4SF)
B_DEF      (vec_any_unlev2df,           vec_any_unlev2df,   0,                  B_INT | B_VX,       0,                  BT_FN_INT_V2DF_V2DF)

OB_DEF     (s390_vec_all_nge,           s390_vec_all_nge_flt,s390_vec_all_nge_dbl,B_VX,             BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_all_nge_flt,       vec_all_unltv4sf,   B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)                     /* vfchesbs */
OB_DEF_VAR (s390_vec_all_nge_dbl,       vec_all_unltv2df,   0,                  0,                  BT_OV_INT_V2DF_V2DF)                     /* vfchedbs */

OB_DEF     (s390_vec_all_ngt,           s390_vec_all_ngt_flt,s390_vec_all_ngt_dbl,B_VX,             BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_all_ngt_flt,       vec_all_unlev4sf,   B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)                     /* vfchsbs */
OB_DEF_VAR (s390_vec_all_ngt_dbl,       vec_all_unlev2df,   0,                  0,                  BT_OV_INT_V2DF_V2DF)                     /* vfchdbs */

OB_DEF     (s390_vec_any_nge,           s390_vec_any_nge_flt,s390_vec_any_nge_dbl,B_VX,             BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_any_nge_flt,       vec_any_unltv4sf,   B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_any_nge_dbl,       vec_any_unltv2df,   0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_any_ngt,           s390_vec_any_ngt_flt,s390_vec_any_ngt_dbl,B_VX,             BT_FN_INT_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_any_ngt_flt,       vec_any_unlev4sf,   B_VXE,              0,                  BT_OV_INT_V4SF_V4SF)
OB_DEF_VAR (s390_vec_any_ngt_dbl,       vec_any_unlev2df,   0,                  0,                  BT_OV_INT_V2DF_V2DF)

OB_DEF     (s390_vec_ctd,               s390_vec_ctd_s64,   s390_vec_ctd_u64,   B_VX,               BT_FN_V2DF_UV4SI_INT)
OB_DEF_VAR (s390_vec_ctd_s64,           s390_vec_ctd_s64,   0,                  O2_U5,              BT_OV_V2DF_V2DI_INT)                     /* vcdgb */
OB_DEF_VAR (s390_vec_ctd_u64,           s390_vec_ctd_u64,   0,                  O2_U5,              BT_OV_V2DF_UV2DI_INT)                    /* vcdlgb */

B_DEF      (s390_vec_ctd_s64,           vec_ctd_s64,        0,                  B_VX,               O2_U3,              BT_FN_V2DF_V2DI_INT)                     /* vcdgb */
B_DEF      (s390_vec_ctd_u64,           vec_ctd_u64,        0,                  B_VX,               O2_U3,              BT_FN_V2DF_UV2DI_INT)                    /* vcdlgb */
B_DEF      (s390_vcdgb,                 vcdgb,              0,                  B_VX,               O2_U4 | O3_U3,      BT_FN_V2DF_V2DI_INT_INT)
B_DEF      (s390_vcdlgb,                vcdlgb,             0,                  B_VX,               O2_U4 | O3_U3,      BT_FN_V2DF_UV2DI_INT_INT)
B_DEF      (s390_vec_ctsl,              vec_ctsl,           0,                  B_VX,               O2_U3,              BT_FN_V2DI_V2DF_INT)                     /* vcgdb */
B_DEF      (s390_vec_ctul,              vec_ctul,           0,                  B_VX,               O2_U3,              BT_FN_UV2DI_V2DF_INT)                    /* vclgdb */
B_DEF      (s390_vcgdb,                 vcgdb,              0,                  B_VX,               O2_U4 | O3_U3,      BT_FN_V2DI_V2DF_INT_INT)
B_DEF      (s390_vclgdb,                vclgdb,             0,                  B_VX,               O2_U4 | O3_U3,      BT_FN_UV2DI_V2DF_INT_INT)
B_DEF      (s390_vfisb,                 vec_fpintv4sf,      0,                  B_VXE,              O2_U4 | O3_U3,      BT_FN_V4SF_V4SF_UCHAR_UCHAR)
B_DEF      (s390_vfidb,                 vec_fpintv2df,      0,                  B_VX,               O2_U4 | O3_U3,      BT_FN_V2DF_V2DF_UCHAR_UCHAR)
B_DEF      (s390_vec_ld2f,              vec_ld2f,           0,                  B_VX,               0,                  BT_FN_V2DF_FLTCONSTPTR)                  /* vldeb */
B_DEF      (s390_vec_st2f,              vec_st2f,           0,                  B_VX,               0,                  BT_FN_VOID_V2DF_FLTPTR)                  /* vledb */

B_DEF      (s390_vfmasb,                fmav4sf4,           0,                  B_VXE,              0,                  BT_FN_V4SF_V4SF_V4SF_V4SF)
B_DEF      (s390_vfmadb,                fmav2df4,           0,                  B_VX,               0,                  BT_FN_V2DF_V2DF_V2DF_V2DF)
B_DEF      (s390_vfmssb,                fmsv4sf4,           0,                  B_VXE,              0,                  BT_FN_V4SF_V4SF_V4SF_V4SF)
B_DEF      (s390_vfmsdb,                fmsv2df4,           0,                  B_VX,               0,                  BT_FN_V2DF_V2DF_V2DF_V2DF)
B_DEF      (s390_vfnmasb,               neg_fmav4sf4,       0,                  B_VXE,              0,                  BT_FN_V4SF_V4SF_V4SF_V4SF)
B_DEF      (s390_vfnmadb,               neg_fmav2df4,       0,                  B_VXE,              0,                  BT_FN_V2DF_V2DF_V2DF_V2DF)
B_DEF      (s390_vfnmssb,               neg_fmsv4sf4,       0,                  B_VXE,              0,                  BT_FN_V4SF_V4SF_V4SF_V4SF)
B_DEF      (s390_vfnmsdb,               neg_fmsv2df4,       0,                  B_VXE,              0,                  BT_FN_V2DF_V2DF_V2DF_V2DF)

B_DEF      (s390_vfsqsb,                sqrtv4sf2,          0,                  B_VXE,              0,                  BT_FN_V4SF_V4SF)
B_DEF      (s390_vfsqdb,                sqrtv2df2,          0,                  B_VX,               0,                  BT_FN_V2DF_V2DF)

OB_DEF     (s390_vec_double,            s390_vec_double_s64,s390_vec_double_u64,B_VX,               BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_double_s64,        s390_vcdgb,         0,                  0,                  BT_OV_V2DF_V2DI)
OB_DEF_VAR (s390_vec_double_u64,        s390_vcdlgb,        0,                  0,                  BT_OV_V2DF_UV2DI)

B_DEF      (s390_vec_double_s64,        vec_double_s64,     0,                  B_INT | B_VX,       0,                  BT_FN_V2DF_V2DI)  /* vcdgb */
B_DEF      (s390_vec_double_u64,        vec_double_u64,     0,                  B_INT | B_VX,       0,                  BT_FN_V2DF_UV2DI) /* vcdlgb */

B_DEF      (s390_vflls,                 vflls,              0,                  B_VX,               0,                  BT_FN_V2DF_V4SF)                         /* vldeb */
B_DEF      (s390_vflrd,                 vflrd,              0,                  B_VX,               O2_U4 | O3_U4,      BT_FN_V4SF_V2DF_INT_INT)                 /* vledb */

OB_DEF     (s390_vec_madd,              s390_vec_madd_flt,  s390_vec_madd_dbl,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_madd_flt,          s390_vfmasb,        B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_madd_dbl,          s390_vfmadb,        0,                  0,                  BT_OV_V2DF_V2DF_V2DF_V2DF)

OB_DEF     (s390_vec_msub,              s390_vec_msub_flt,  s390_vec_msub_dbl,  B_VX,               BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_msub_flt,          s390_vfmssb,        B_VXE,              0,                  BT_OV_V4SF_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_msub_dbl,          s390_vfmsdb,        0,                  0,                  BT_OV_V2DF_V2DF_V2DF_V2DF)

OB_DEF     (s390_vec_nmadd,             s390_vec_nmadd_flt, s390_vec_nmadd_dbl, B_VXE,              BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_nmadd_flt,         s390_vfnmasb,       0,                  0,                  BT_OV_V4SF_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_nmadd_dbl,         s390_vfnmadb,       0,                  0,                  BT_OV_V2DF_V2DF_V2DF_V2DF)

OB_DEF     (s390_vec_nmsub,             s390_vec_nmsub_flt, s390_vec_nmsub_dbl, B_VXE,              BT_FN_OV4SI_OV4SI_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_nmsub_flt,         s390_vfnmssb,       0,                  0,                  BT_OV_V4SF_V4SF_V4SF_V4SF)
OB_DEF_VAR (s390_vec_nmsub_dbl,         s390_vfnmsdb,       0,                  0,                  BT_OV_V2DF_V2DF_V2DF_V2DF)

B_DEF      (s390_vflnsb,                negabsv4sf2,        0,                  B_VXE,              0,                  BT_FN_V4SF_V4SF)
B_DEF      (s390_vflndb,                negabsv2df2,        0,                  B_VX,               0,                  BT_FN_V2DF_V2DF)

OB_DEF     (s390_vec_nabs,              s390_vec_nabs_flt,  s390_vec_nabs_dbl,  B_VXE,              BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_nabs_flt,          s390_vflnsb,        0,                  0,                  BT_OV_V4SF_V4SF)
OB_DEF_VAR (s390_vec_nabs_dbl,          s390_vflndb,        B_VX,               0,                  BT_OV_V2DF_V2DF)

OB_DEF     (s390_vec_sqrt,              s390_vec_sqrt_flt,  s390_vec_sqrt_dbl,  B_VXE,              BT_FN_OV4SI_OV4SI)
OB_DEF_VAR (s390_vec_sqrt_flt,          s390_vfsqsb,        0,                  0,                  BT_OV_V4SF_V4SF)
OB_DEF_VAR (s390_vec_sqrt_dbl,          s390_vfsqdb,        B_VX,               0,                  BT_OV_V2DF_V2DF)

/* Test data class with boolean result *AND* cc mode.  */

B_DEF      (s390_vftcisb,               vftciv4sf_intcc,    0,                  B_VXE,              O2_U12,             BT_FN_V4SI_V4SF_INT_INTPTR)
B_DEF      (s390_vftcidb,               vftciv2df_intcc,    0,                  B_VX,               O2_U12,             BT_FN_V2DI_V2DF_INT_INTPTR)

/* Test data class with boolean result *AND* cc mode.  */

B_DEF      (s390_vftcisbcc,             vftciv4sf_intcc,    0,                  B_INT | B_VXE,      O2_U12,             BT_FN_BV4SI_V4SF_USHORT_INTPTR)          /* vftcisb */
B_DEF      (s390_vftcidbcc,             vftciv2df_intcc,    0,                  B_INT | B_VX,       O2_U12,             BT_FN_BV2DI_V2DF_USHORT_INTPTR)          /* vftcidb */

OB_DEF     (s390_vec_fp_test_data_class,s390_vec_fp_test_data_class_flt,s390_vec_fp_test_data_class_dbl,B_VX,BT_FN_OV4SI_OV4SI_INT_INTPTR)
OB_DEF_VAR (s390_vec_fp_test_data_class_flt,s390_vftcisbcc, B_VXE,              O2_U12,             BT_OV_BV4SI_V4SF_USHORT_INTPTR)          /* vftcisb */
OB_DEF_VAR (s390_vec_fp_test_data_class_dbl,s390_vftcidbcc, 0,                  O2_U12,             BT_OV_BV2DI_V2DF_USHORT_INTPTR)          /* vftcidb */
